﻿#ifndef _MIR_EXCEPTIONS_H_
#define _MIR_EXCEPTIONS_H_
/*
 * Copyright (c) 2011,
 * Technical University Of Gdansk,
 * Faculty of Electronics, Telecommunications and Informatics,
 * Multimedia Systems Department
 * and/or its affiliates. All rights reserved.
 */

/*
 * valid for IBPP 2.5.3.1

                         std::exception
                                |
                         IBPP::Exception
                       /                 \
                      /                   \
  IBPP::LogicException    ExceptionBase    IBPP::SQLException
        |        \         /   |     \     /
        |   LogicExceptionImpl |   SQLExceptionImpl
        |                      |
    IBPP::WrongType            |
               \               |
              IBPP::WrongTypeImpl
*/


#include <string>

#include <ibpp.h>

#include <unicode/unistr.h>

namespace SyNaT
{
  namespace MIR
  {

    using namespace std;

    /**
     * A <code>CommonMirException</code> provides base type for all database
     * exception types and it shouldn't be used directly.
     *
     * @author  Łukasz Matuszewski
     * @version $Id: mir_exceptions.h 370 2011-09-02 11:02:22Z lmatusz $
     */
    class CommonMirException : public IBPP::Exception
    {
      protected:
        string m_what;
        string m_origin;

      public:
        CommonMirException();
        CommonMirException(const CommonMirException &e);
        CommonMirException(const UnicodeString &origin);
        CommonMirException(const UnicodeString &origin, const UnicodeString &what);
        CommonMirException(const string &origin);
        CommonMirException(const string &origin, const string &what);
        CommonMirException(const char *origin);
        CommonMirException(const char *origin, const char *what);
    		virtual ~CommonMirException() throw();

		    virtual const char*          Origin() const throw();
        virtual const char*          ErrorMessage() const throw();
        virtual const char*          what() const throw();
        virtual UnicodeString        icuorigin() const throw();
        virtual UnicodeString        icuwhat() const throw();
        virtual void                 setOrigin(string &origin) throw();
        virtual void                 setWhat(string &what) throw();
        virtual string               getOrigin() const throw();
        virtual string               getWhat() const throw();
    };


    class DatabaseNotAssignedMirException : public CommonMirException
    {
      public:
        DatabaseNotAssignedMirException();
        DatabaseNotAssignedMirException(const DatabaseNotAssignedMirException &e);
        DatabaseNotAssignedMirException(const UnicodeString &origin);
        DatabaseNotAssignedMirException(const UnicodeString &origin, const UnicodeString &what);
        DatabaseNotAssignedMirException(const string &origin);
        DatabaseNotAssignedMirException(const string &origin, const string &what);
        DatabaseNotAssignedMirException(const char *origin);
        DatabaseNotAssignedMirException(const char *origin, const char *what);
    };


    class TransactionNotAssignedMirException : public CommonMirException
    {
      public:
        TransactionNotAssignedMirException();
        TransactionNotAssignedMirException(const TransactionNotAssignedMirException &e);
        TransactionNotAssignedMirException(const UnicodeString &origin);
        TransactionNotAssignedMirException(const UnicodeString &origin, const UnicodeString &what);
        TransactionNotAssignedMirException(const string &origin);
        TransactionNotAssignedMirException(const string &origin, const string &what);
        TransactionNotAssignedMirException(const char *origin);
        TransactionNotAssignedMirException(const char *origin, const char *what);
    };


    class UserNotLoggedMirException : public CommonMirException
    {
      public:
        UserNotLoggedMirException();
        UserNotLoggedMirException(const UserNotLoggedMirException &e);
        UserNotLoggedMirException(const UnicodeString &origin);
        UserNotLoggedMirException(const UnicodeString &origin, const UnicodeString &what);
        UserNotLoggedMirException(const string &origin);
        UserNotLoggedMirException(const string &origin, const string &what);
        UserNotLoggedMirException(const char *origin);
        UserNotLoggedMirException(const char *origin, const char *what);
    };


    class UserDoesntExistsMirException : public CommonMirException
    {
      public:
        UserDoesntExistsMirException();
        UserDoesntExistsMirException(const UserDoesntExistsMirException &e);
        UserDoesntExistsMirException(const UnicodeString &origin);
        UserDoesntExistsMirException(const UnicodeString &origin, const UnicodeString &what);
        UserDoesntExistsMirException(const string &origin);
        UserDoesntExistsMirException(const string &origin, const string &what);
        UserDoesntExistsMirException(const char *origin);
        UserDoesntExistsMirException(const char *origin, const char *what);
    };


    class IncorrectPasswordMirException : public CommonMirException
    {
      public:
        IncorrectPasswordMirException();
        IncorrectPasswordMirException(const IncorrectPasswordMirException &e);
        IncorrectPasswordMirException(const UnicodeString &origin);
        IncorrectPasswordMirException(const UnicodeString &origin, const UnicodeString &what);
        IncorrectPasswordMirException(const string &origin);
        IncorrectPasswordMirException(const string &origin, const string &what);
        IncorrectPasswordMirException(const char *origin);
        IncorrectPasswordMirException(const char *origin, const char *what);
    };


    class DbTableIsEmptyMirException : public CommonMirException
    {
      public:
        DbTableIsEmptyMirException();
        DbTableIsEmptyMirException(const DbTableIsEmptyMirException &e);
        DbTableIsEmptyMirException(const UnicodeString &origin);
        DbTableIsEmptyMirException(const UnicodeString &origin, const UnicodeString &what);
        DbTableIsEmptyMirException(const string &origin);
        DbTableIsEmptyMirException(const string &origin, const string &what);
        DbTableIsEmptyMirException(const char *origin);
        DbTableIsEmptyMirException(const char *origin, const char *what);
    };


    class BaseURIIsAReferenceMirException : public CommonMirException
    {
      public:
        BaseURIIsAReferenceMirException();
        BaseURIIsAReferenceMirException(const BaseURIIsAReferenceMirException &e);
        BaseURIIsAReferenceMirException(const UnicodeString &origin);
        BaseURIIsAReferenceMirException(const UnicodeString &origin, const UnicodeString &what);
        BaseURIIsAReferenceMirException(const string &origin);
        BaseURIIsAReferenceMirException(const string &origin, const string &what);
        BaseURIIsAReferenceMirException(const char *origin);
        BaseURIIsAReferenceMirException(const char *origin, const char *what);
    };

    class RegexMatcherMirException : public CommonMirException
    {
      public:
        RegexMatcherMirException();
        RegexMatcherMirException(const RegexMatcherMirException &e);
        RegexMatcherMirException(const UnicodeString &origin);
        RegexMatcherMirException(const UnicodeString &origin, const UnicodeString &what);
        RegexMatcherMirException(const string &origin);
        RegexMatcherMirException(const string &origin, const string &what);
        RegexMatcherMirException(const char *origin);
        RegexMatcherMirException(const char *origin, const char *what);
    };

  }
}

#endif //_MIR_EXCEPTIONS_H_
