#ifndef _MIR_FIREBIRD_H_
#define _MIR_FIREBIRD_H_
/*
 * Copyright (c) 2011,
 * Technical University Of Gdansk,
 * Faculty of Electronics, Telecommunications and Informatics,
 * Multimedia Systems Department
 * and/or its affiliates. All rights reserved.
 */

#include <Wt/Dbo/SqlConnection>
#include <Wt/Dbo/SqlStatement>

#include "mir_databasemodule.h"

#include <ibpp.h>
namespace SyNaT
{
  namespace MIR
  {
    using namespace Wt::Dbo;
    using namespace Wt;
    using namespace IBPP;

    /*! \class Firebird mir_firebird.h mir_firebird.h
     *  \brief A FirebirdSQL connection
     *
     * This class provides the backend implementation for FirebirdSQL databases.
     *
     * \ingroup dbo
     */
    class Firebird : public SqlConnection
    {
    public:
      /*! \brief Creates new FirebirdSQL backend connection.
       *
       * The connection is not yet open, and requires a connect() before it
       * can be used.
       */
      Firebird();

      /*! \brief Opens a new FirebirdSQL backend connection.
       *
       * The \p dm may be any connected database module.
       */
      Firebird(DatabaseModule *dm);

      /*! \brief Copies a FirebirdSQL connection.
       */
      Firebird(const Firebird& other);

      /*! \brief Destructor.
       *
       * Closes the connection.
       */
      virtual ~Firebird();

      virtual Firebird *clone() const;

      /*! \brief Tries to connect.
       *
       * Throws an exception if there was a problem, otherwise true.
       */
      bool connect(const std::string& db);

      /*! \brief Returns the underlying connection.
       */
      DatabaseModule *connection()
      {
        return m_dm;
      }

      virtual void executeSql(const std::string &sql);

      virtual void startTransaction();
      virtual void commitTransaction();
      virtual void rollbackTransaction();
      virtual IBPP::Transaction transaction()
      {
        return *this->m_tr;
      }

      virtual SqlStatement *prepareStatement(const std::string& sql);

      /** @name Methods that return dialect information
       */
      //@{
      virtual std::string autoincrementSql() const;
      virtual std::string autoincrementType() const;
      virtual std::string autoincrementInsertSuffix() const;
      virtual const char *dateTimeType(SqlDateTimeType type) const;
      virtual const char *blobType() const;
      //@}

      virtual void        setWritaleTransaction(bool amWrite)
      {
        this->m_writableTransaction = amWrite;
      }
    private:
      std::string     connInfo_;
      DatabaseModule       *m_dm;
      IBPP::Transaction    *m_tr;
      IBPP::Transaction     m_tra;
      bool            m_writableTransaction;
      bool            m_dmOwned;
    };

  }
}

#endif /* _MIR_FIREBIRD_H_ */
