#ifndef _MIR_CHARSVPARSER_H_
#define _MIR_CHARSVPARSER_H_
/*
 * Copyright (c) 2011,
 * Technical University Of Gdansk,
 * Faculty of Electronics, Telecommunications and Informatics,
 * Multimedia Systems Department
 * and/or its affiliates. All rights reserved.
 */

#include <unicode/unistr.h>

namespace SyNaT
{
  namespace MIR
  {
    /**
     * A <code>CHARSVParser</code> is UChar32 separated values parser (it is taken
     * from PostgreSQL Admin, but modified to take any separator and/or quote).
     *
     * @author  Łukasz Matuszewski
     * @version $Id: mir_charsvparser.h 334 2011-08-18 09:12:33Z lmatusz $
     */
    class CHARSVParser
    {
      public:
        CHARSVParser() : m_string(""), m_pos(0), m_separator(','), m_quote('"') { }
        CHARSVParser(const UnicodeString& str): m_string(str), m_pos(0), m_separator(','), m_quote('"') { }
        CHARSVParser(const UnicodeString& str, UChar32 separator): m_string(str), m_pos(0), m_separator(separator), m_quote('"') { }
        CHARSVParser(const UnicodeString& str, UChar32 separator, UChar32 quote): m_string(str), m_pos(0), m_separator(separator), m_quote(quote) { }

        bool HasMoreTokens();

        // Get the next token (CSV field). Will return empty string if !HasMoreTokens()
        UnicodeString GetNextToken();

        void setLine(const UnicodeString& str)
        {
          this->m_string = str;
          this->m_pos = 0;
        }

        void setLine(const UnicodeString& str, UChar32 separator)
        {
          this->m_string = str;
          this->m_pos = 0;
          this->m_separator = separator;
        }

        void setLine(const UnicodeString& str, UChar32 separator, UChar32 quote)
        {
          this->m_string = str;
          this->m_pos = 0;
          this->m_separator = separator;
          this->m_quote = quote;
        }

        int32_t find_first_not_of(const UnicodeString &input, UChar32 not_of,int32_t pos = 0)
        {
          int32_t i = pos;
          for(; i < input.length(); i++)
            if (input.char32At(i) != not_of)
              break;
          if (i == input.length())
            i = -1;
          return i;
        }

      protected:
        UnicodeString  m_string;        // the string we tokenize into fields
        int32_t        m_pos;           // the current position in m_string
        UChar32        m_separator;     // the field separator
        UChar32        m_quote;         // the quote sign around field value
    };
  }
}

#endif //_MIR_CHARSVPARSER_H_
