﻿#ifndef _MIR_UTILS_H_
#define _MIR_UTILS_H_
/*
 * Copyright (c) 2011,
 * Technical University Of Gdansk,
 * Faculty of Electronics, Telecommunications and Informatics,
 * Multimedia Systems Department
 * and/or its affiliates. All rights reserved.
 */

#include <boost/regex.hpp>
#include <boost/ptr_container/ptr_map.hpp>
#include <boost/thread/mutex.hpp>
#include <boost/filesystem.hpp>

#include <string>

#include <ibpp.h>

#include "mir_uri.h"
#include "mir_enums.h"
#include "mir_databasemodule.h"
#include "mir_contenttyperetriever.h"

#include <unicode/unistr.h>
#include <unicode/regex.h>

namespace SyNaT
{
  namespace MIR
  {
    using namespace std;
    using namespace IBPP;
    using namespace boost::filesystem;

    /**
     * A <code>Utils</code> represents different utility functions (some of are
     * static some not) used into parsing engine in
     * SyNaT::MIR::MR::DownloadThread.
     *
     * @author  Łukasz Matuszewski
     * @version $Id: mir_utils.h 370 2011-09-02 11:02:22Z lmatusz $
     */
    class Utils
    {
      public:
        const static boost::regex                              mcs_regex_metacharset;
        const static boost::regex                              mcs_regex_contentType;
        const static boost::regex                              mcs_regex_contentTypeCharset;
        const static boost::regex                              mcs_regex_contentdispositionFilename;

        const static string                                    mcs_hex[256];
        Utils(const string &sessionID);

        virtual ~Utils();

        static KindOfContentTypeValue                          match_content_type(const string &httpResponseHeader);

        static string                                          match_content_type_ext(const string &httpResponseHeader);

        static string                                          match_charset(const string &httpResponse, const string &httpResponseHeader);

        static UnicodeString                                   CreateTempFileName();

        static UnicodeString                                   buildDestHref(UnicodeString &icudirplusfilename, IcuURI &icu_a_href, ContentTypeRetriever &ctr, boost::filesystem::path &bprootwww);

        static bool                                            isMusicRobotWorking(DatabaseModule *db, Transaction *tr);

        IcuURI                                                 match_img_tag(const UnicodeString &icu_part_of_page, const IcuURI &baseURI, bool &hasImgSrc, bool &hasImgTag, int32_t &start, int32_t &len, int32_t &startOfTag, int32_t &lenOfTag, KindOfAttrValue &koav, const string &sessionID, string &sessionIDVal);

        IcuURI                                                 match_link_tag(const UnicodeString &icu_part_of_page, const IcuURI &baseURI, bool &hasLinkHref, bool &hasLinkTag, int32_t &start, int32_t &len, int32_t &startOfTag, int32_t &lenOfTag, KindOfAttrValue &koav, const string &sessionID, string &sessionIDVal);

        IcuURI                                                 match_base_tag(const UnicodeString &icu_part_of_page, bool &hasBaseHref, bool &hasBaseTag, int32_t &start, int32_t &len, int32_t &startOfTag, int32_t &lenOfTag, IcuURI *icuuri);

        IcuURI                                                 match_a_tag(const UnicodeString &icu_part_of_page, const IcuURI &baseURI, bool &hasAHref, bool &hasATag, int32_t &start, int32_t &len, int32_t &startOfTag, int32_t &lenOfTag, KindOfAttrValue &koav, const string &sessionID, string &sessionIDVal);
        
        static IcuURI                                          buildURIWithPathAfterServer(const IcuURI *icuuri);
        
        static IcuURI                                          buildURIWithQueryParam(const IcuURI *icuuri, const UnicodeString &qparam, const UnicodeString &qvalue);

        static void                                            replaceNewlines(UnicodeString &input);

        static UnicodeString                                   subAllNotAllowedCharsInFilenames(UnicodeString input, bool replaceSlashes = true, bool doNotReplaceLastDot = false);
         
        typedef boost::ptr_map<string,UnicodeString>           ptr_map_string_UnicodeString;

        /**
         * Przyjmuje, że path jest w formacie ISO-8859-1 z ewentualnie escapeowanymi znakami ze stringa w formacie UTF-8
         */
        static UnicodeString                                   buildParameterMapFromPath(string path, ptr_map_string_UnicodeString &params, string &charset);

        static int                                             isASCIIalpha(int ch);

        static int                                             isASCIIdigit(int ch);

        /**
         *
         * @param s The string in UTF-8 to be encoded
         * @return The encoded string
         */
        static string                                          encode(const string &input, const bool encodeSlash = false);

        static string                                          decode(const string &escapedInput);

        static UnicodeString                                   encode(const UnicodeString &input, const bool encodeSlash = false);

        static UnicodeString                                   decode(const UnicodeString &escapedInput);

        static string                                          icusToUTF8(UnicodeString str);

        static void                                            changePermsOfDirs(const path &dir, const string perms, const path &startingPoint = path(""));

		    static string                   											 findContentTypeByExt(const string &ext);
      protected:
        string                                                 m_sessionID;

        RegexPattern                                          *m_regexpatt_imgtag;
        RegexPattern                                          *m_regexpatt_linktag;
        RegexPattern                                          *m_regexpatt_atag;
        RegexPattern                                          *m_regexpatt_basetag;
        RegexPattern                                          *m_regexpatt_attributeWOQ;
        RegexPattern                                          *m_regexpatt_attributeWSQ;
        RegexPattern                                          *m_regexpatt_attributeWDQ;
        RegexPattern                                          *m_regexpatt_spaces;
        RegexPattern                                          *m_regexpatt_srcReplacer;
        RegexPattern                                          *m_regexpatt_srcReplacerWOEQ;
        RegexPattern                                          *m_regexpatt_hrefReplacer;
        RegexPattern                                          *m_regexpatt_hrefReplacerWOEQ;
    };
  }
}

#endif //_MIR_UTILS_H_
