#ifndef _MIR_DEFERRED_WIDGET_H_
#define _MIR_DEFERRED_WIDGET_H_

#include <Wt/WContainerWidget>

namespace SyNaT
{
  namespace MIR
  {
    using namespace Wt;
    /*
     * A utility container widget which defers creation of its single
     * child widget until the container is loaded (which is done on-demand
     * by a WMenu). The constructor takes the create function for the
     * widget as a parameter.
     *
     * We use this to defer widget creation until needed, which also defers
     * loading auxiliary javascript libraries.
     */
    #if !defined(WT_TARGET_JAVA)
    template <typename Function>
    class DeferredWidget : public Wt::WContainerWidget
    {
      public:
        DeferredWidget(Function f, WContainerWidget *wcw = NULL)
          : f_(f), wcw_(wcw) { }

        virtual ~DeferredWidget()
        {
          if(this->wcw_)
            delete this->wcw_;
        }
      private:
        void load() {
          addWidget(this->f_());
          WContainerWidget::load();
        }

      Function f_;
      WContainerWidget *wcw_;
    };

    template <typename Function>
    DeferredWidget<Function> *deferCreate(Function f, WContainerWidget *wcw = NULL)
    {
      return new DeferredWidget<Function>(f, wcw);
    }
    #else
    class DeferredWidget : public Wt::WContainerWidget {
    public:
      DeferredWidget(boost::bound f) {}
    };
    DeferredWidget *deferCreate(boost::bound b)
    {
      return new DeferredWidget(b);
    }

    #endif

  }
}

#endif // _MIR_DEFERRED_WIDGET_H_
