#ifndef _MIR_WCUSTOMTABLEVIEW_H_
#define _MIR_WCUSTOMTABLEVIEW_H_
/*
 * Copyright (c) 2011,
 * Technical University Of Gdansk,
 * Faculty of Electronics, Telecommunications and Informatics,
 * Multimedia Systems Department
 * and/or its affiliates. All rights reserved.
 */

#include <Wt/WContainerWidget>
#include <Wt/WCompositeWidget>
#include <Wt/WAbstractItemModel>
#include <Wt/WTableView>
#include <Wt/WPushButton>

#include <vector>

namespace SyNaT
{
  namespace MIR
  {
    using namespace Wt;
    using namespace std;

    class WCustomTableViewPushButton;

    class WCustomTableView : public WCompositeWidget
    {
      friend class WCustomTableViewPushButton;

      public:
        WCustomTableView(WContainerWidget *parent = 0)
          : WCompositeWidget(new WContainerWidget(), parent), m_checkable(false)
          , m_current_page(0), m_nav_bar_created(false), m_num_of_pages_in_nav_bar(4),
          m_first_page_in_nav_bar(0), m_last_page_in_nav_bar(m_num_of_pages_in_nav_bar-1)
        {
          this->m_page_nav_bar = new WContainerWidget();
          this->addChild(this->m_page_nav_bar);
          this->m_page_nav_bar->setClearSides(Left | Right);
          this->m_wtv = new WTableView();
          this->addChild(this->m_wtv);
        }

        virtual ~WCustomTableView()
        {
        }

        virtual void load();

        WAbstractItemModel *model() const
        {
          return this->m_wtv->model();
        }

        virtual void setModel(WAbstractItemModel* model)
        {
          this->m_wtv->setModel(model);
        }

        void setSortingEnabled(int column, bool enabled)
        {
          this->m_wtv->setSortingEnabled(column, enabled);
        }

        virtual void setRowHeight(const WLength& rowHeight)
        {
          this->m_wtv->setRowHeight(rowHeight);
        }

        virtual void setColumnWidth(int column, const WLength& width)
        {
          this->m_wtv->setColumnWidth(column, width);
        }

        void sortByColumn(int column, SortOrder order)
        {
          this->m_wtv->sortByColumn(column, order);
        }

        virtual void setAlternatingRowColors(bool enable)
        {
          this->m_wtv->setAlternatingRowColors(enable);
        }

        void setItemDelegateForColumn(int column, WAbstractItemDelegate *delegate)
        {
          this->m_wtv->setItemDelegateForColumn(column, delegate);
        }

        virtual void setSelectable(bool selectable)
        {
          this->m_wtv->setSelectable(selectable);
        }

        virtual void setCheckable(const bool checkable)
        {
          this->m_checkable = checkable;
        }

        virtual void resize(const int& widthPx, const int& heightPx)
        {
          this->WCompositeWidget::resize(widthPx, heightPx);
          this->m_wtv->resize(widthPx, heightPx-20);
        }

        virtual void resize(const WLength& width, const WLength& height)
        {
          this->WCompositeWidget::resize(width, height);
        }

        virtual void resize_inner_table_view(const WLength& width, const WLength& height)
        {
          this->m_wtv->resize(width, height);
        }

        virtual void scrollTo(const WModelIndex& index,
            Wt::WTableView::ScrollHint hint = Wt::WTableView::EnsureVisible)
        {
          this->m_wtv->scrollTo(index, hint);
        }

        virtual int pageCount() const
        {
          return this->m_wtv->pageCount();
        }

        virtual int pageSize() const
        {
          return this->m_wtv->pageSize();
        }

        virtual void setNumOfPagesInNavBar(int8_t num_of_pages)
        {
          this->m_num_of_pages_in_nav_bar = num_of_pages;
        }

      protected:
        virtual WWidget *create_page_nav_bar();

        virtual void update_page_nav_bar();

      private:
        WTableView                           *m_wtv;

        WContainerWidget                     *m_page_nav_bar;

        vector<WCustomTableViewPushButton *>  m_page_numbers;

        bool                                  m_checkable;

        int                                   m_first_page_in_nav_bar;

        int                                   m_last_page_in_nav_bar;

        int                                   m_current_page;

        int                                   m_num_of_pages_in_nav_bar;

        bool                                  m_nav_bar_created;

        bool                                  m_nav_bar_needs_update;
    };

  }
}
#endif /* _MIR_WCUSTOMTABLEVIEW_H_ */
