#include "utils/mir_sha1.h"

#include <Wt/WString>
#include <Wt/WLogger>
#include <Wt/WApplication>
#include <Wt/WBreak>
#include <Wt/WLabel>
#include <Wt/WWidget>
#include <Wt/WText>
#include <Wt/WLineEdit>
#include <Wt/WPushButton>
#include <Wt/WMenu>
#include <Wt/WLength>
#include <Wt/WLengthValidator>
#include <Wt/WRegExpValidator>
#include <Wt/WContainerWidget>
#include <Wt/WTableCell>
#include <Wt/WTable>
#include <Wt/WHBoxLayout>
#include <Wt/WLength>

#define POCO_NO_UNWINDOWS
#include "Poco/Net/MailMessage.h"
#include "Poco/Net/MailRecipient.h"
#include "Poco/Net/SMTPClientSession.h"
#include "Poco/Net/SecureSMTPClientSession.h"
#include "Poco/Net/StringPartSource.h"
#include "Poco/Net/Context.h"
#undef POCO_NO_UNWINDOWS

#include <vector>
#include <sstream>

#include <boost/foreach.hpp>
#include <boost/ptr_container/ptr_vector.hpp>

#include <unicode/unistr.h>

#include "utils/mir_cssconsts.h"
#include "utils/mir_cssclasses.h"
#include "utils/mir_trkeys.h"
#include "utils/mir_defines.h"
#include "utils/mir_stringutils.h"
#include "utils/mir_stringcloner.h"
#include "utils/mir_internalpaths.h"
#include "mir_databasetypes.h"
#include "mir_databasemodule.h"
#include "widgets/mir_deferredwidget.h"
#include "controls/mir_sp_controlswidget.h"
#include "mir_sp_menuandcontents.h"
#include "mir_sp_synatpgapp.h"
#include "controls/mir_sp_logincontrols.h"

#include <ibpp.h>

namespace SyNaT
{
  namespace MIR
  {
    namespace SP
    {
      using namespace std;
      using namespace Wt;
      using namespace IBPP;
      using Poco::Net::MailMessage;
      using Poco::Net::MailRecipient;
      using Poco::Net::SMTPClientSession;
      using Poco::Net::SecureSMTPClientSession;
      using Poco::Net::StringPartSource;
      using Poco::Net::Context;

      LoginControls::LoginControls(SyNaTPGApp *app, WMenu *mainmenu, DatabaseModule *dm, string internal_path_to_select, const bool createLoginScreen, const string styleClass, const bool forceClass)
        : ControlsWidget(false, styleClass, forceClass), m_mainmenu(mainmenu), m_dm(dm), m_app(app), m_internal_path_to_select(internal_path_to_select)
         ,m_tcell(0)
      {
        if (createLoginScreen)
          this->addWidget(this->create_signin_page());
      }

      LoginControls::~LoginControls()
      {
        if(this->m_tcell)
          this->m_tcell->removeWidget(this->m_app->m_wtappmessage);
      }


      WWidget *LoginControls::create_signin_page(void)
      {
        WTable *table = new WTable();
        table->setMargin(WLength::Auto, Left | Right);

        this->m_wlblLogin = new WLabel(tr(MIR_TXT_LOGIN));
        WTableCell *cell = table->elementAt(0, 0);
        cell->addWidget(this->m_wlblLogin);

        cell = table->elementAt(1, 0);
        this->m_wlblLoginInvalidMsg = new WLabel(tr(MIR_LBL_2SPACES));
        this->m_wlblLoginInvalidMsg->setStyleClass(MIR_STRCAT3(MIR_CSS_HALFFONTSIZE," ",MIR_CSS_COLOR_RED));
        cell->addWidget(this->m_wlblLoginInvalidMsg);
        cell->setMinimumSize(WLength(MIR_PX_SIZE_OF_CELL_IN_REG_AND_LOGIN_FORMS, WLength::Pixel), cell->height());
        table->elementAt(1, 1);
        cell = table->elementAt(0, 1);
        this->m_wleLogin = new WLineEdit();
        this->m_wleLogin->setTextSize(15);
        this->m_wlblLogin->setBuddy(this->m_wleLogin);
        WValidator *validator = new WRegExpValidator(MIR_REGEXP_LOGIN);
        validator->setMandatory(true);
        this->m_wleLogin->setValidator(validator);
        cell->addWidget(this->m_wleLogin);
        cell->setRowSpan(2);

        cell = table->elementAt(3, 0);
        this->m_wlblPasswordInvalidMsg = new WLabel(tr(MIR_LBL_SPACE));
        //this->m_wlblPasswordInvalidMsg->setStyleClass(MIR_STRCAT3(MIR_CSS_HALFFONTSIZE," ",MIR_CSS_COLOR_RED));
        this->m_wlblPasswordInvalidMsg->setStyleClass(MIR_STRCAT3(MIR_CSS_HALFFONTSIZE," ",MIR_CSS_COLOR_RED));
        cell->addWidget(this->m_wlblPasswordInvalidMsg);
        table->elementAt(3, 1);
        cell = table->elementAt(2, 0);
        this->m_wlblPassword = new WLabel(tr(MIR_TXT_PASSWORD));
        cell->addWidget(this->m_wlblPassword);

        cell = table->elementAt(2, 1);
        this->m_wlePassword = new WLineEdit();
        this->m_wlePassword->setTextSize(15);
        this->m_wlblPassword->setBuddy(this->m_wlePassword);
        this->m_wlePassword->setEchoMode(WLineEdit::Password);
        validator = new WLengthValidator(2, 31);
        validator->setMandatory(true);
        this->m_wlePassword->setValidator(validator);
        cell->addWidget(this->m_wlePassword);
        cell->setRowSpan(2);

        cell = table->elementAt(4, 0);
        table->elementAt(4, 1);
        cell->setColumnSpan(2);
        WContainerWidget *wcw = new WContainerWidget();
        WPushButton *wpb = new WPushButton(tr(MIR_BTN_SIGNIN), wcw);
        wpb->clicked().connect(this, &LoginControls::slt_login);
        wpb = new WPushButton(tr(MIR_BTN_REGISTER), wcw);
        wpb->clicked().connect(this, &LoginControls::slt_register);
        WHBoxLayout *horizLayout = new WHBoxLayout(cell);
        horizLayout->addWidget(wcw, 0, AlignCenter);
        cell->setLayout(horizLayout);

        this->m_tcell = cell = table->elementAt(5, 0);
        table->elementAt(5, 1);
        cell->setColumnSpan(2);

        //horizLayout = new WHBoxLayout(cell);
        //horizLayout->addWidget(this->m_app->m_wtappmessage, 0, AlignCenter);
        //cell->setLayout(horizLayout);
        cell->addWidget(this->m_app->m_wtappmessage);
        cell->setStyleClass(MIR_STRCAT5(MIR_CSS_COLOR_RED, " ", MIR_CSS_HALFFONTSIZE, " ", MIR_CSS_TEXTALIGN_CENTER));
//        cell = table->elementAt(6, 0);
//        cell->addWidget(this->m_app->m_wtappmessage);
//
//        cell = table->elementAt(7, 0);
//        cell->addWidget(this->m_app->m_wtappmessage);
        this->m_was_rendered = true;
        return table;
      }


      void LoginControls::slt_login(void)
      {
        bool validated = true;
        if (this->m_wleLogin->validate() != WValidator::Valid)
        {
          this->m_wlblLogin->setStyleClass(MIR_STRINGIFY(MIR_CSS_FONT_BOLD));
          this->m_wlblLogin->setText(tr(MIR_TXT_STARRED_LOGIN));
          this->m_wlblLoginInvalidMsg->setText(tr(MIR_MSG_LOGINNOTVALIDATED));
          validated = false;
        }
        else
        {
          this->m_wlblLogin->setStyleClass("");
          this->m_wlblLogin->setText(tr(MIR_TXT_LOGIN));
          this->m_wlblLoginInvalidMsg->setText(tr(MIR_LBL_2SPACES));
        }

        if (this->m_wlePassword->validate() != WValidator::Valid)
        {
          this->m_wlblPassword->setStyleClass(MIR_STRINGIFY(MIR_CSS_FONT_BOLD));
          this->m_wlblPassword->setText(tr(MIR_TXT_STARRED_PASSWORD));
          this->m_wlblPasswordInvalidMsg->setText(tr(MIR_MSG_ARGISNOTCORRECT).arg(tr(MIR_MSG_ARGPASSWORD)));
          validated = false;
        }
        else
        {
          this->m_wlblPassword->setStyleClass("");
          this->m_wlblPassword->setText(tr(MIR_TXT_PASSWORD));
          this->m_wlblPasswordInvalidMsg->setText(tr(MIR_LBL_SPACE));
        }
        if (!validated)
        {
          this->m_app->m_wtappmessage->setText(tr(MIR_MSG_PLEASECORRECTFIELDS));
          return;
        }
        try
        {
          IBPP::Transaction tr = this->m_dm->createTransaction(amRead, ilReadCommitted, lrWait);
          tr->Start();
          this->m_dm->login(this->m_wleLogin->text().toUTF8(),
                            this->m_wlePassword->text().toUTF8(), &tr, false);
          tr->Commit();
        } catch(std::exception &e)
        {
          this->m_app->log("error") << e.what();
        }
        if (this->m_dm->logged() && this->m_dm->isUser(MIR_ACTIVE))
        {
          vector<WMenuItem *> items_(this->m_mainmenu->items());
          WMenuItem *mi, *mi1 = NULL;
          BOOST_FOREACH(mi, items_)
          {
            if (mi->text() == tr(MIR_MENU_SIGNIN))
            {
              this->m_app->doJavaScript("$(\"#"+ this->m_mainmenu->id()+" li:first\").css(\"display\", \"none\");");
              mi->hide();
            }
            else
            {
              if (!mi1)
                mi1 = mi;
              if (m_internal_path_to_select.find(mi->pathComponent()) != string::npos)
                mi1 = mi;

              mi->show();
            }
          }
          if (mi1)
            mi1->select();
        }
        else
        {
          if (this->m_dm->logged())
          {
            //this->m_wlblLoginInvalidMsg->setText(tr(MIR_MSG_USERISNOTACTIVATED));
            this->m_app->m_wtappmessage->setText(tr(MIR_MSG_USERISNOTACTIVATED));
          }
          else
          {
            //this->m_wlblLoginInvalidMsg->setText(tr(MIR_MSG_SIGNINFAILED));
            this->m_app->m_wtappmessage->setText(tr(MIR_MSG_SIGNINFAILED));
            //this->m_app->m_wtappmessage->setText(tr(MIR_MSG_PLEASECORRECTFIELDS));
          }
        }
      }

      void LoginControls::slt_register(void)
      {
        this->m_app->m_wtappmessage->setText(tr(MIR_MSG_EMPTY));
        this->m_tcell->removeWidget(this->m_app->m_wtappmessage);
        this->clear();
        this->m_tcell = NULL;
        WTable *table = new WTable(this);
        table->setMargin(WLength::Auto, Left | Right);

        table->setHeaderCount(1);
        WTableCell *cell = table->elementAt(0, 0);
        cell->setColumnSpan(2);
        cell->setStyleClass(MIR_STRINGIFY(MIR_CSS_TEXTALIGN_CENTER));
        cell->addWidget(new WText(tr(MIR_TABLE_HEAD_REGISTER), XHTMLText));

        this->m_wlblLogin = new WLabel(tr(MIR_TXT_LOGIN));
        cell = table->elementAt(1, 0);
        cell->addWidget(this->m_wlblLogin);

        cell = table->elementAt(2, 0);

        this->m_wlblLoginInvalidMsg = new WLabel(tr(MIR_LBL_2SPACES));
        //
        this->m_wlblLoginInvalidMsg->setStyleClass(MIR_STRCAT3(MIR_CSS_HALFFONTSIZE," ",MIR_CSS_COLOR_RED));
        //this->m_wlblLoginInvalidMsg->setStyleClass(MIR_STRINGIFY(MIR_CSS_INV_MSG_TRANS_OFF));
        cell->addWidget(this->m_wlblLoginInvalidMsg);
        cell->setMinimumSize(WLength(MIR_PX_SIZE_OF_CELL_IN_REG_AND_LOGIN_FORMS, WLength::Pixel), cell->height());
        cell = table->elementAt(2, 1);
        cell = table->elementAt(1, 1);
        this->m_wleLogin = new WLineEdit();
        WValidator *validator = new WRegExpValidator(MIR_REGEXP_LOGIN);
        validator->setMandatory(true);
        this->m_wleLogin->setValidator(validator);
        this->m_wleLogin->setTextSize(15);
        this->m_wlblLogin->setBuddy(this->m_wleLogin);
        cell->addWidget(this->m_wleLogin);
        cell->setRowSpan(2);


        this->m_wlblPassword = new WLabel(tr(MIR_TXT_PASSWORD));
        cell = table->elementAt(3, 0);
        cell->addWidget(this->m_wlblPassword);

        cell = table->elementAt(4, 0);
        this->m_wlblPasswordInvalidMsg = new WLabel(tr(MIR_LBL_SPACE));
        //this->m_wlblPasswordInvalidMsg->setStyleClass(MIR_STRCAT3(MIR_CSS_HALFFONTSIZE," ",MIR_CSS_COLOR_RED));
        this->m_wlblPasswordInvalidMsg->setStyleClass(MIR_STRCAT3(MIR_CSS_HALFFONTSIZE," ",MIR_CSS_COLOR_RED));
        cell->addWidget(this->m_wlblPasswordInvalidMsg);
        cell = table->elementAt(4, 1);
        cell = table->elementAt(3, 1);
        this->m_wlePassword = new WLineEdit();
        validator = new WLengthValidator(2, 31);
        validator->setMandatory(true);
        this->m_wlePassword->setValidator(validator);
        this->m_wlePassword->setTextSize(31);
        this->m_wlePassword->setEchoMode(WLineEdit::Password);
        this->m_wlblPassword->setBuddy(this->m_wlePassword);
        cell->addWidget(this->m_wlePassword);

        this->m_wlblPasswordDup = new WLabel(tr(MIR_TXT_PASSWORD_DUP));
        cell = table->elementAt(5, 0);
        cell->addWidget(this->m_wlblPasswordDup);

        cell = table->elementAt(6, 0);
        this->m_wlblPasswordDupInvalidMsg = new WLabel(tr(MIR_LBL_SPACE));
        //this->m_wlblPasswordDupInvalidMsg->setStyleClass(MIR_STRCAT3(MIR_CSS_HALFFONTSIZE," ",MIR_CSS_COLOR_RED));
        this->m_wlblPasswordDupInvalidMsg->setStyleClass(MIR_STRCAT3(MIR_CSS_HALFFONTSIZE," ",MIR_CSS_COLOR_RED));
        cell->addWidget(this->m_wlblPasswordDupInvalidMsg);
        cell = table->elementAt(6, 1);
        cell = table->elementAt(5, 1);
        this->m_wlePasswordDup = new WLineEdit();
        validator = new WLengthValidator(2, 31);
        validator->setMandatory(true);
        this->m_wlePasswordDup->setValidator(validator);
        this->m_wlePasswordDup->setTextSize(31);
        this->m_wlePasswordDup->setEchoMode(WLineEdit::Password);
        this->m_wlblPasswordDup->setBuddy(this->m_wlePasswordDup);
        cell->addWidget(this->m_wlePasswordDup);

        this->m_wlblName = new WLabel(tr(MIR_TXT_NAME));
        cell = table->elementAt(7, 0);
        cell->addWidget(this->m_wlblName);

        cell = table->elementAt(8, 0);
        this->m_wlblNameInvalidMsg = new WLabel(tr(MIR_LBL_SPACE));
        //this->m_wlblPasswordInvalidMsg->setStyleClass(MIR_STRCAT3(MIR_CSS_HALFFONTSIZE," ",MIR_CSS_COLOR_RED));
        this->m_wlblNameInvalidMsg->setStyleClass(MIR_STRCAT3(MIR_CSS_HALFFONTSIZE," ",MIR_CSS_COLOR_RED));
        cell->addWidget(this->m_wlblNameInvalidMsg);
        cell = table->elementAt(8, 1);
        cell = table->elementAt(7, 1);
        this->m_wleName = new WLineEdit();
        validator = new WLengthValidator(1, 31);
        validator->setMandatory(true);
        this->m_wleName->setValidator(validator);
        this->m_wleName->setTextSize(31);
        this->m_wlblName->setBuddy(this->m_wleName);
        cell->addWidget(this->m_wleName);
        cell->setRowSpan(2);

        this->m_wlblSurname = new WLabel(tr(MIR_TXT_SURNAME));
        cell = table->elementAt(9, 0);
        cell->addWidget(this->m_wlblSurname);

        cell = table->elementAt(10, 0);
        this->m_wlblSurnameInvalidMsg = new WLabel(tr(MIR_LBL_SPACE));
        //this->m_wlblSurnameInvalidMsg->setStyleClass(MIR_STRCAT3(MIR_CSS_HALFFONTSIZE," ",MIR_CSS_COLOR_RED));
        this->m_wlblSurnameInvalidMsg->setStyleClass(MIR_STRCAT3(MIR_CSS_HALFFONTSIZE," ",MIR_CSS_COLOR_RED));
        cell->addWidget(this->m_wlblSurnameInvalidMsg);
        cell = table->elementAt(10, 1);
        cell = table->elementAt(9, 1);
        this->m_wleSurname = new WLineEdit();
        validator = new WLengthValidator(1, 63);
        validator->setMandatory(true);
        this->m_wleSurname->setValidator(validator);
        this->m_wleSurname->setTextSize(31);
        this->m_wlblSurname->setBuddy(this->m_wleSurname);
        cell->addWidget(this->m_wleSurname);

        this->m_wlblEmail = new WLabel(tr(MIR_TXT_EMAIL));
        cell = table->elementAt(11, 0);
        cell->addWidget(this->m_wlblEmail);

        cell = table->elementAt(12, 0);
        this->m_wlblEmailInvalidMsg = new WLabel(tr(MIR_LBL_SPACE));
        //this->m_wlblEmailInvalidMsg->setStyleClass(MIR_STRCAT3(MIR_CSS_HALFFONTSIZE," ",MIR_CSS_COLOR_RED));
        this->m_wlblEmailInvalidMsg->setStyleClass(MIR_STRCAT3(MIR_CSS_HALFFONTSIZE," ",MIR_CSS_COLOR_RED));
        cell->addWidget(this->m_wlblEmailInvalidMsg);
        cell = table->elementAt(12, 1);
        cell = table->elementAt(11, 1);
        this->m_wleEmail = new WLineEdit();
        validator = new WRegExpValidator(MIR_REGEXP_EMAIL);
        validator->setMandatory(true);
        this->m_wleEmail->setValidator(validator);
        this->m_wleEmail->setTextSize(31);
        this->m_wlblEmail->setBuddy(this->m_wleEmail);
        cell->addWidget(this->m_wleEmail);
        cell->setRowSpan(2);

        cell = table->elementAt(13, 0);
        cell->setColumnSpan(2);
        WContainerWidget *wcw = new WContainerWidget();
        WPushButton *wpb = new WPushButton(tr(MIR_BTN_CREATEACCOUNT), wcw);
        wpb->clicked().connect(this, &LoginControls::slt_createaccount);
        wpb = new WPushButton(tr(MIR_BTN_CANCEL), wcw);
        wpb->clicked().connect(this, &LoginControls::slt_cancelreg);
        WHBoxLayout *horizLayout = new WHBoxLayout();
        horizLayout->addWidget(wcw, 0, AlignCenter);
        cell->setLayout(horizLayout);

        this->m_tcell = cell = table->elementAt(14, 0);
        cell->setColumnSpan(2);

//        horizLayout = new WHBoxLayout();
//
//        horizLayout->addWidget(this->m_app->m_wtappmessage, 0, AlignCenter);
//        cell->setLayout(horizLayout);
        cell->addWidget(this->m_app->m_wtappmessage);
        cell->setStyleClass(MIR_STRCAT5(MIR_CSS_COLOR_RED, " ", MIR_CSS_HALFFONTSIZE, " ", MIR_CSS_TEXTALIGN_CENTER));


//        this->m_app->setInternalPath("/register");
      }


      void LoginControls::slt_cancelreg(void)
      {
        this->m_app->m_wtappmessage->setText(tr(MIR_MSG_EMPTY));
        this->m_tcell->removeWidget(this->m_app->m_wtappmessage);
        this->clear();
        this->m_tcell = NULL;
        this->addWidget(this->create_signin_page());
      }

      void LoginControls::slt_createaccount(void)
      {
        bool validated = true;
        if (this->m_wleLogin->validate() != WValidator::Valid)
        {
          this->m_wlblLogin->setStyleClass(MIR_STRINGIFY(MIR_CSS_FONT_BOLD));
          this->m_wlblLogin->setText(tr(MIR_TXT_STARRED_LOGIN));
          this->m_wlblLoginInvalidMsg->setText(tr(MIR_MSG_LOGINNOTVALIDATED));
          validated = false;
        }
        else
        {
          this->m_wlblLogin->setStyleClass("");
          this->m_wlblLogin->setText(tr(MIR_TXT_LOGIN));
          this->m_wlblLoginInvalidMsg->setText(tr(MIR_LBL_2SPACES));
        }
        IBPP::Transaction tra = this->m_dm->createTransaction(amRead, ilReadCommitted, lrWait);
        IBPP::Transaction tr1 = this->m_dm->createTransaction(amRead, ilReadCommitted, lrWait);
        try
        {
          tr1->Start();
          this->m_dm->login("some_superuser", "some_superuser_pass", &tr1, false);
          tr1->Commit();
          ostringstream oss;
          oss << "SELECT UZY_ID FROM MIR_UZYTKOWNIK WHERE UZY_IDENTYFIKATOR = " << DatabaseModule::toSql(this->m_wleLogin->text().toUTF8());
          string sql = oss.str();
          tra->Start();
          Statement stmt = this->m_dm->createStatement(&tra, &sql);
          stmt->Execute();
          if(stmt->Fetch())
          {
            if (validated)
            {
              this->m_wlblLoginInvalidMsg->setText(tr(MIR_MSG_USERLOGINISUSED));
              this->m_wlblLogin->setStyleClass(MIR_STRINGIFY(MIR_CSS_FONT_BOLD));
              this->m_wlblLogin->setText(tr(MIR_TXT_STARRED_LOGIN));
              validated = false;
            }
          }
//          else
//          {
//            if (validated)
//            {
//              this->m_wlblLogin->setStyleClass("");
//              this->m_loginStarWasAppended = false;
//              this->m_wlblLogin->setText(tr(MIR_TXT_LOGIN));
//              this->m_wlblLoginInvalidMsg->setText("&nbsp;<br/>&nbsp;");
//            }
//          }
          tra->Commit();
        }
        catch(...)
        {
          if (tra->Started())
            tra->Rollback();
          if (tr1->Started())
            tr1->Rollback();
        }
        if (this->m_wleName->validate() != WValidator::Valid)
        {
          this->m_wlblName->setStyleClass(MIR_STRINGIFY(MIR_CSS_FONT_BOLD));
          this->m_wlblName->setText(tr(MIR_TXT_STARRED_NAME));
          //this->m_app->m_wtappmessage->setText(tr(MIR_MSG_ARGISNOTCORRECT).arg(tr(MIR_MSG_ARGNAME)));
          this->m_wlblNameInvalidMsg->setText(tr(MIR_MSG_ARGISNOTCORRECT).arg(tr(MIR_MSG_ARGNAME)));
          validated = false;
        }
        else
        {
          this->m_wlblName->setStyleClass("");
          this->m_wlblName->setText(tr(MIR_TXT_NAME));
          this->m_wlblNameInvalidMsg->setText(tr(MIR_LBL_SPACE));
        }
        if (this->m_wleSurname->validate() != WValidator::Valid)
        {
          this->m_wlblSurname->setStyleClass(MIR_STRINGIFY(MIR_CSS_FONT_BOLD));
          this->m_wlblSurname->setText(tr(MIR_TXT_STARRED_SURNAME));
          this->m_wlblSurnameInvalidMsg->setText(tr(MIR_MSG_ARGISNOTCORRECT).arg(tr(MIR_MSG_ARGSURNAME)));
          validated = false;
        }
        else
        {
          this->m_wlblSurname->setStyleClass("");
          this->m_wlblSurname->setText(tr(MIR_TXT_SURNAME));
          this->m_wlblSurnameInvalidMsg->setText(tr(MIR_LBL_SPACE));
        }
        if (this->m_wleEmail->validate() != WValidator::Valid)
        {
          this->m_wlblEmail->setStyleClass(MIR_STRINGIFY(MIR_CSS_FONT_BOLD));
          this->m_wlblEmail->setText(tr(MIR_TXT_STARRED_EMAIL));
          this->m_wlblEmailInvalidMsg->setText(tr(MIR_MSG_ARGISNOTCORRECT).arg(tr(MIR_MSG_ARGEMAIL)));
          validated = false;
        }
        else
        {
          this->m_wlblEmail->setStyleClass("");
          this->m_wlblEmail->setText(tr(MIR_TXT_EMAIL));
          this->m_wlblEmailInvalidMsg->setText(tr(MIR_LBL_SPACE));
        }
        if (this->m_wlePassword->validate() != WValidator::Valid)
        {
          this->m_wlblPassword->setStyleClass(MIR_STRINGIFY(MIR_CSS_FONT_BOLD));
          this->m_wlblPassword->setText(tr(MIR_TXT_STARRED_PASSWORD));
          this->m_wlblPasswordInvalidMsg->setText(tr(MIR_MSG_ARGISNOTCORRECT).arg(tr(MIR_MSG_ARGPASSWORD)));
          //this->m_app->m_wtappmessage->setText(tr(MIR_MSG_ARGISNOTCORRECT).arg(tr(MIR_MSG_ARGPASSWORD)));
          validated = false;
        }
        else
        {
          this->m_wlblPassword->setStyleClass("");
          this->m_wlblPassword->setText(tr(MIR_TXT_PASSWORD));
          this->m_wlblPasswordInvalidMsg->setText(tr(MIR_LBL_SPACE));
        }
        if (this->m_wlePasswordDup->text() != this->m_wlePassword->text())
        {
          this->m_wlblPasswordDup->setStyleClass(MIR_STRINGIFY(MIR_CSS_FONT_BOLD));
          this->m_wlblPasswordDup->setText(tr(MIR_TXT_STARRED_PASSWORD_DUP));
          this->m_wlblPasswordDupInvalidMsg->setText(tr(MIR_MSG_PASSWORDSMUSTBETHESAME));
          //this->m_app->m_wtappmessage->setText(tr(MIR_MSG_ARGISNOTCORRECT).arg(tr(MIR_MSG_ARGPASSWORD)));
          validated = false;
        }
        else
        {
          this->m_wlblPasswordDup->setStyleClass("");
          this->m_wlblPasswordDup->setText(tr(MIR_TXT_PASSWORD_DUP));
          this->m_wlblPasswordDupInvalidMsg->setText(tr(MIR_LBL_SPACE));
        }
        if (!validated)
        {
          this->m_app->m_wtappmessage->setText(tr(MIR_MSG_PLEASECORRECTFIELDS));
          return;
        }
        try
        {
          IBPP::Transaction tr = this->m_dm->createTransaction(amRead, ilReadCommitted, lrWait);
          tr->Start();
          this->m_dm->login(string("some_superuser"), string("some_superuserpass"), &tr, false);
          tr->Commit();
          boost::ptr_vector<DBField> dbFields;
          dbFields.push_back(new StringDBField(this->m_wleName->text().toUTF8(), "UZY_IMIE"));
          dbFields.push_back(new StringDBField(this->m_wleSurname->text().toUTF8(), "UZY_NAZWISKO"));
          dbFields.push_back(new StringDBField(this->m_wleLogin->text().toUTF8(), "UZY_IDENTYFIKATOR"));
          dbFields.push_back(new StringDBField(this->m_wleEmail->text().toUTF8(), "UZY_EMAIL"));
          dbFields.push_back(new INT64TDBField(MIR_NOTACTIVE, "UZY_PRAWA"));
          char    hashpass[20];
          CSHA1 cs;
          cs.Update((const UINT_8*) this->m_wlePassword->text().toUTF8().c_str(), (UINT_32) this->m_wlePassword->text().toUTF8().length());
          cs.Final();
          cs.GetHash((unsigned char *) hashpass);
          dbFields.push_back(new VOIDPDBField(hashpass, 20, "UZY_HASLO"));
          IBPP::Transaction tra1 = this->m_dm->createTransaction(amWrite, ilReadCommitted, lrWait);
          tra1->Start();
          int64_t uzy_id = this->m_dm->insertIntoTable("UZY", "MIR_UZYTKOWNIK", dbFields, &tra1);
          this->m_dm->logout();
          this->m_dm->login(this->m_wleLogin->text().toUTF8(), this->m_wlePassword->text().toUTF8(), &tra1, false);
          dbFields.resize(0);
          dbFields.push_back(new INT64TDBField(uzy_id, "UST_UZY_ID"));
          dbFields.push_back(new BoolDBField(false, "UST_TYP_WYBORU_W_PRZEG"));
          this->m_dm->insertIntoTable("UST", "MIR_USTAWIENIA", dbFields, &tra1);
          tra1->Commit();
        }
        catch(...)
        {
          this->m_wlblLoginInvalidMsg->setText(tr(MIR_MSG_USERLOGINISUSED));
          this->m_wlblLogin->setStyleClass(MIR_STRINGIFY(MIR_CSS_FONT_BOLD));
          this->m_wlblLogin->setText(tr(MIR_TXT_STARRED_LOGIN));
          this->m_app->m_wtappmessage->setText(tr(MIR_MSG_PLEASECORRECTFIELDS));
          return;
        }
        // tutaj następuje wysyłanie linka aktywacyjnego
        try
        {
          string smtphost = "sound.eti.pg.gda.pl";
          Poco::UInt16 port = 25;//587;
          string user = "synatserwer";
          string password = "";
          string to = this->m_wleEmail->text().toUTF8();
          string from = "synatserwer@sound.eti.pg.gda.pl";

          string charset = "utf-8";
          string contentType = "text/plain; charset=\"utf-8\"";
          string subject = "[SyNaT] Link aktywacyjny";
          subject = MailMessage::encodeWord(subject, charset);
          string content = "Oto link aktywacyjny, który należy przekleić do przeglądarki: \r\n";

          ostringstream oss;
          CSHA1 csha1;
          string strToCSHA1a1Update(string("flisaktoty ") + this->m_wleEmail->text().toUTF8());
          csha1.Update((const unsigned char *) strToCSHA1a1Update.c_str(), (unsigned int) strToCSHA1a1Update.length());
          csha1.Final();
          TCHAR hashparam[41];
          csha1.ReportHash(hashparam, CSHA1::REPORT_HEX_SHORT);
          hashparam[40] = 0;
          UnicodeString hash(hashparam);
          string hash_utf8;
          hash.toUTF8String(hash_utf8);
	  // zakładam user agentów z obsługą HTML 5
          oss << "http://synat.eti.pg.gda.pl/wt/SyNaTPG.wt/activation/" << this->m_wleLogin->text().toUTF8()
              << "/" << hash_utf8;

          oss << " \r\n" << "E-mail wygenerowany automatycznie, prosimy nie odpowiadać.";
          content += oss.str();

          MailMessage message;
          message.setSender(from);
          message.addRecipient(MailRecipient(MailRecipient::PRIMARY_RECIPIENT, to));
          message.setSubject(subject);
          message.setContentType(contentType);
          message.setContent(content, MailMessage::ENCODING_8BIT);

          SMTPClientSession session(smtphost, port);
          session.login();
          //Context::Ptr ctx(new Context(Context::CLIENT_USE, "", Context::VERIFY_NONE, 9, true));
          //session.startTLS(ctx);
          //session.login(SMTPClientSession::AUTH_LOGIN, user, password);
          session.sendMessage(message);
          session.close();
        }
        catch(...)
        {
        }
        this->m_app->m_wtappmessage->setText(tr(MIR_MSG_PLEASEACTIVATEACCOUNT));
        this->m_tcell->removeWidget(this->m_app->m_wtappmessage);
        this->clear();
        this->m_tcell = NULL;
        this->addWidget(this->create_signin_page());
      }
    }
  }
}
