﻿/*
 * mir_sha1.h must be first.
 * Another solution is to do following:
 **/
 //  //#if (ULONG_MAX == 0xFFFFFFFF)
 //  //#define UINT_32 unsigned long
 //  //#else
 //  #define UINT_32 unsigned int
 //  //#endif
#include "utils/mir_sha1.h"

#include <Wt/WApplication>
#include <Wt/WLogger>

#include <unicode/unistr.h>
#include <unicode/msgfmt.h>

#include <boost/foreach.hpp>
#include <boost/regex.hpp>
#include <boost/ptr_container/ptr_vector.hpp>

#define POCO_NO_UNWINDOWS
#include "Poco/Util/ServerApplication.h"
#undef POCO_NO_UNWINDOWS

#include <string>
#include <sstream>

//#include "utils/mir_stringutils.h"
#include "mir_exceptions.h"
#include "utils/mir_defines.h"
#include "mir_databasetypes.h"
#include "mir_databasemodule.h"

#include <ibpp.h>

#include <unicode/unistr.h>
#include <unicode/regex.h>
#include <unicode/msgfmt.h>

namespace SyNaT
{
  namespace MIR
  {
    using namespace std;
    using namespace IBPP;

    string DatabaseModule::toSql(const string &input, const bool addSingleQuotations)
    {
      ostringstream oss;
      if (addSingleQuotations)
        oss << "'";

      UnicodeString icuinput = UnicodeString::fromUTF8(input);
      icuinput.findAndReplace(UnicodeString::fromUTF8("'"), UnicodeString::fromUTF8("''"));
//      UErrorCode status_icuregex = U_ZERO_ERROR;
//      RegexMatcher *icuregex = new RegexMatcher(UnicodeString::fromUTF8("\\'"), 0, status_icuregex);
//      icuregex->reset(icuinput);
//      status_icuregex = U_ZERO_ERROR;
//      icuinput = icuregex->replaceAll(UnicodeString::fromUTF8("''"), status_icuregex);
      string stdicuinput;
      icuinput.toUTF8String(stdicuinput);
      oss << stdicuinput;

      if (addSingleQuotations)
        oss << "'";
//      delete icuregex;
      return oss.str();
    }
    
    string DatabaseModule::toSimilarToSql(const string &input, const bool addEscapeWord, const bool escapeHypens, const bool escapeQuestionMarks)
    {
      ostringstream oss;
      string sql = DatabaseModule::toSql(input, false);
      if (addEscapeWord)
        oss << "'";

      UnicodeString icuinput = UnicodeString::fromUTF8(input);

      icuinput.findAndReplace(UnicodeString::fromUTF8("_"), UnicodeString::fromUTF8("\\_"));
//      UErrorCode status_regex = U_ZERO_ERROR;
//
//      RegexMatcher *icusimilar_to_regex = new RegexMatcher(UnicodeString::fromUTF8("_"), 0, status_regex);
//      icusimilar_to_regex->reset(icuinput);
//      status_regex = U_ZERO_ERROR;
//      icuinput = icusimilar_to_regex->replaceAll(UnicodeString::fromUTF8("\\_"), status_regex);
//      delete icusimilar_to_regex;

      if (escapeHypens)
      {
        icuinput.findAndReplace(UnicodeString::fromUTF8("-"), UnicodeString::fromUTF8("\\-"));
//        status_regex = U_ZERO_ERROR;
//        RegexMatcher *icuhyphen_regex = new RegexMatcher(UnicodeString::fromUTF8("\\-"), 0, status_regex);
//        icuhyphen_regex->reset(icuinput);
//        if (icuhyphen_regex->find())
//        {
//          status_regex = U_ZERO_ERROR;
//        }
//        status_regex = U_ZERO_ERROR;
//        icuinput = icuhyphen_regex->replaceAll(UnicodeString::fromUTF8("\\\\-"), status_regex);
//        delete icuhyphen_regex;
      }

      if (escapeQuestionMarks)
      {
        icuinput.findAndReplace(UnicodeString::fromUTF8("?"), UnicodeString::fromUTF8("\\?"));
//        status_regex = U_ZERO_ERROR;
//        RegexMatcher *icuquestionmark_regex = new RegexMatcher(UnicodeString::fromUTF8("\\?"), 0, status_regex);
//        icuquestionmark_regex->reset(icuinput);
//        status_regex = U_ZERO_ERROR;
//        icuinput = icuquestionmark_regex->replaceAll(UnicodeString::fromUTF8("\\\\?"), status_regex);
//        delete icuquestionmark_regex;
      }

      string stdicuinput;
      icuinput.toUTF8String(stdicuinput);
      oss << stdicuinput;
      if (addEscapeWord)
        oss << "' ESCAPE '\\'";

      return oss.str();
    }

    DatabaseModule::DatabaseModule() : m_adminID(DATABASEMODULE_ADMIN_ID)
    {
      this->m_dbAssigned = false;
      this->m_loggedUserId = 0; // 0 means no one logged
    }

    DatabaseModule::DatabaseModule(const UnicodeString &serverName, const UnicodeString &databaseName,
                                const UnicodeString &userName, const UnicodeString &userPassword,
                                const UnicodeString &roleName, const UnicodeString &charSet,
                                const UnicodeString &createParams, const UnicodeString &login,
                                const UnicodeString &loginPassword, bool isSha1) /* throw(...) */ : m_adminID(DATABASEMODULE_ADMIN_ID)
    {
      this->m_dbAssigned = false;
      this->m_loggedUserId = 0; // 0 means no logged
      this->connect(serverName, databaseName, userName, userPassword, roleName, charSet, createParams, login, loginPassword, isSha1);
    }

    DatabaseModule::DatabaseModule(const string &serverName, const string &databaseName, 
                                const string &userName, const string &userPassword,
                                const string &roleName, const string &charSet,
                                const string &createParams, const string &login, 
                                const string &loginPassword, bool isSha1) /* throw(...) */ : m_adminID(DATABASEMODULE_ADMIN_ID)
    {
      this->m_dbAssigned = false;
      this->m_loggedUserId = 0; // 0 means no logged
      this->connect(serverName, databaseName, userName, userPassword, roleName, charSet, createParams, login, loginPassword, isSha1);
    }

    void DatabaseModule::connect(const UnicodeString &serverName, const UnicodeString &databaseName,
                                 const UnicodeString &userName, const UnicodeString &userPassword,
                                 const UnicodeString &roleName, const UnicodeString &charSet,
                                 const UnicodeString &createParams, const UnicodeString &login,
                                 const UnicodeString &loginPassword, bool isSha1) /* throw(...) */

    {
      string stdserverName, stddatabaseName,stduserName, stduserPassword, stdroleName, stdcharSet
             , stdcreateParams, stdlogin, stdloginPassword;

      serverName.toUTF8String(stdserverName);
      databaseName.toUTF8String(stddatabaseName);
      userName.toUTF8String(stddatabaseName);
      userPassword.toUTF8String(stduserPassword);
      roleName.toUTF8String(stdroleName);
      charSet.toUTF8String(stdcharSet);
      createParams.toUTF8String(stdcreateParams);
      login.toUTF8String(stdlogin);
      loginPassword.toUTF8String(stdloginPassword);

      this->m_db = DatabaseFactory(stdserverName, stddatabaseName,
                                 stduserName, stduserPassword,
                                 stdroleName, stdcharSet,
                                 stdcreateParams);
      this->m_dbAssigned = true;

      this->m_db->Connect();
      Transaction tr = TransactionFactory(this->m_db, amRead);
      tr->Start();
      this->login(stdlogin, stdloginPassword, &tr, isSha1);
      tr->Commit();
    }

    void DatabaseModule::connect(const string &serverName, const string &databaseName, 
                                   const string &userName, const string &userPassword,
                                   const string &roleName, const string &charSet,
                                   const string &createParams, const string &login, 
                                   const string &loginPassword, bool isSha1) /* throw(...) */
    {
      Database db = this->m_db;
      this->m_db = DatabaseFactory(serverName, databaseName,
                                   userName, userPassword,
                                   roleName, charSet, createParams);
      this->m_dbAssigned = true;

      this->m_db->Connect();
      Transaction tr = TransactionFactory(this->m_db, amRead);
      tr->Start();
      this->login(login, loginPassword, &tr, isSha1);
      tr->Commit();
      if (db.intf() && db->Connected())
      {
        db->Disconnect();
      }
    }

    DatabaseModule::DatabaseModule(const UnicodeString &serverName, const UnicodeString &databaseName,
                                const UnicodeString &userName, const UnicodeString &userPassword,
                                const UnicodeString &roleName, const UnicodeString &charSet,
                                const UnicodeString &createParams) /* throw(...) */ : m_adminID(DATABASEMODULE_ADMIN_ID)
    {
      this->m_dbAssigned = false;
      this->m_loggedUserId = 0; // 0 means no logged
      this->connect(serverName, databaseName, userName, userPassword, roleName, charSet, createParams);
    }

    DatabaseModule::DatabaseModule(const string &serverName, const string &databaseName,
                                const string &userName, const string &userPassword,
                                const string &roleName, const string &charSet,
                                const string &createParams) /* throw(...) */ : m_adminID(DATABASEMODULE_ADMIN_ID)
    {
      this->m_dbAssigned = false;
      this->m_loggedUserId = 0; // 0 means no logged
      this->connect(serverName, databaseName, userName, userPassword, roleName, charSet, createParams);
    }

    void DatabaseModule::connect(const UnicodeString &serverName, const UnicodeString &databaseName,
                                 const UnicodeString &userName, const UnicodeString &userPassword,
                                 const UnicodeString &roleName, const UnicodeString &charSet,
                                 const UnicodeString &createParams) /* throw(...) */

    {
      string stdserverName, stddatabaseName,stduserName, stduserPassword, stdroleName, stdcharSet
             , stdcreateParams;

      serverName.toUTF8String(stdserverName);
      databaseName.toUTF8String(stddatabaseName);
      userName.toUTF8String(stddatabaseName);
      userPassword.toUTF8String(stduserPassword);
      roleName.toUTF8String(stdroleName);
      charSet.toUTF8String(stdcharSet);
      createParams.toUTF8String(stdcreateParams);

      this->m_db = DatabaseFactory(stdserverName, stddatabaseName,
                                 stduserName, stduserPassword,
                                 stdroleName, stdcharSet,
                                 stdcreateParams);
      this->m_dbAssigned = true;

      this->m_db->Connect();
    }

    void DatabaseModule::connect(const string &serverName, const string &databaseName,
                                   const string &userName, const string &userPassword,
                                   const string &roleName, const string &charSet,
                                   const string &createParams) /* throw(...) */
    {
      Database db = this->m_db;
      this->m_db = DatabaseFactory(serverName, databaseName,
                                   userName, userPassword,
                                   roleName, charSet, createParams);
      this->m_dbAssigned = true;

      this->m_db->Connect();
    }

    Transaction DatabaseModule::createTransaction(TAM am, TIL il, 
                                                  TLR lr, TFF flags)
                                                  /* throw(...) */
    {
      //if(this->m_loggedUserId == 0)
      //  throw UserNotLoggedMirException(string("DatabaseModule::createTransaction(TAM am, TIL il, TLR lr, TFF flags)"));

      if(!this->m_dbAssigned)
        throw DatabaseNotAssignedMirException("DatabaseModule::createTransaction(TAM am, TIL il, TLR lr, TFF flags)");

      return TransactionFactory(this->m_db, am, il, lr, flags);
    }

    Statement DatabaseModule::createStatement(Transaction *tr, string *sql) /* throw(...) */
    {
      if(this->m_loggedUserId == 0)
        throw UserNotLoggedMirException("DatabaseModule::createTransaction(TAM am, TIL il, TLR lr, TFF flags)");

      if(!this->m_dbAssigned)
        throw DatabaseNotAssignedMirException("DatabaseModule::createTransaction(TAM am, TIL il, TLR lr, TFF flags)");

      if(!tr)
        throw TransactionNotAssignedMirException("DatabaseModule::createStatement(Transaction *tr)");
      
      if (sql)
        return StatementFactory(this->m_db, *tr, *sql);
      else
        return StatementFactory(this->m_db, *tr);
    }


    /**
     * @return 0 if next id cannot be computed, otherwise value greater then 0
     */
    int64_t DatabaseModule::getNextID(const UnicodeString &tablePrefix, Transaction *tr)
    {
      int64_t result = 0;
      Formattable args[] = { tablePrefix };
      UErrorCode success = U_ZERO_ERROR;
      try {
        Statement st = StatementFactory(this->m_db, *tr);
        UnicodeString icusql;
        string sql;
        MessageFormat::format("SELECT p.P_OUT FROM SEQ_ID_{0}_ID p", args, 1, icusql, success);
        icusql.toUTF8String(sql);
        st->Execute(sql);
        if (!st->Fetch())
          return result;
        st->Get(1, result);
      } catch(LogicException &e)
      {
        Wt::WApplication::instance()->log("error") << e.ErrorMessage() << " " << e.Origin() << " " << e.what();
        //Application::instance().logger().error(std::string(e.Origin()) + std::string(" ") + std::string(e.what()));
      }
      return result;
    }

    /*
     * @return 0 if next id cannot be computed, otherwise value greater then 0
     */
    int64_t DatabaseModule::getNextID(const string &tablePrefix, Transaction *tr)
    {
      int64_t result = 0;
      try {
        Statement st = StatementFactory(this->m_db, *tr);
        ostringstream sql;
        sql << "SELECT p.P_OUT FROM SEQ_ID_" << tablePrefix << "_ID p";
        st->Execute(sql.str());
        if (!st->Fetch())
          return result;
        st->Get(1, result);
      } catch(LogicException &e)
      {
        Wt::WApplication::instance()->log("error") << e.ErrorMessage() << " " << e.Origin() << " " << e.what();
        //Application::instance().logger().error(std::string(e.Origin()) + std::string(" ") + std::string(e.what()));
      }
      return result;
    }

    void DatabaseModule::checkTableLock(const int64_t tableId, const string &tablePrefix, 
                                        const string &tableName, 
                                        Transaction *tr)
                                        /* throw(...) */
    {
      if(!this->m_dbAssigned)
        throw DatabaseNotAssignedMirException("DatabaseModule::checkTableLock(const int64_t tableId, const string &tablePrefix, const string &tableName, Transaction *tr)");
      
      if(!tr)
        throw TransactionNotAssignedMirException("DatabaseModule::checkTableLock(const int64_t tableId, const string &tablePrefix, const string &tableName, Transaction *tr)");

      if(this->m_loggedUserId == 0)
        throw UserNotLoggedMirException("DatabaseModule::checkTableLock(const int64_t tableId, const string &tablePrefix, const string &tableName, Transaction *tr)");


      ostringstream ostrstmt;
      ostrstmt << "UPDATE " << tableName << " SET " << tablePrefix << "_ID = " 
               << tablePrefix << "_ID WHERE " << tablePrefix << "_ID = " << tableId;

      Statement stmt = StatementFactory(this->m_db, *tr, ostrstmt.str());
      
      stmt->Execute();    
    }

    int64_t DatabaseModule::insertIntoTable2(const string &tablePrefix, const string &tableName,
                                            boost::ptr_vector<DBField> &dbfv, Transaction *tr)
                                            /* throw(...) */
    {
      if(!this->m_dbAssigned)
        throw DatabaseNotAssignedMirException("DatabaseModule::insertIntoTable2(const string &prefix, const string &tableName, const DBFieldHashMap &dbfhm, Transaction &tr) ");

      if(!tr)
        throw TransactionNotAssignedMirException("DatabaseModule::insertIntoTable2(const string &prefix, const string &tableName, const DBFieldHashMap &dbfhm, Transaction &tr) ");

      if(this->m_loggedUserId == 0)
        throw UserNotLoggedMirException("DatabaseModule::insertIntoTable2(const string &prefix, const string &tableName, const DBFieldHashMap &dbfhm, Transaction &tr) ");

      ostringstream ostrstmt;
      ostrstmt << "INSERT INTO " << tableName;

      ostringstream dbFields;
      dbFields << " (" << tablePrefix << "_ID"
               << "," << tablePrefix << "_AUDYT_UT_UZY_ID"
               << "," << tablePrefix << "_AUDYT_DT";

      int64_t tableID = this->getNextID(tablePrefix, tr);
      ostringstream dbValues;
      dbValues << " VALUES(" << tableID
               << "," << this->m_loggedUserId
               << ",CURRENT_TIMESTAMP";

      DBFieldType dbfk;
      bool isVOIDPorCHARP = false;

      BOOST_FOREACH(DBField &dbf, dbfv)
      {
        dbFields << "," << dbf.getFieldName();
        //dbfk = dbf.getDBFieldType();
        //if (dbfk < BOOLT)
        //{
          dbValues << ",?";
        //  isVOIDPorCHARP = true;
        //} else
        //  dbValues << "," << dbf.getAsDBString();
      }
      dbFields << ")";
      dbValues << ")";

      ostrstmt << dbFields.str() << dbValues.str();

      string sql = ostrstmt.str();
      //Statement stmt = this->createStatement(tr, &ostrstmt.str());
      Statement stmt = StatementFactory(this->m_db, *tr, sql);

      if (isVOIDPorCHARP)
      {
        VOIDPDBField *vpdbf;
        CHARPDBField *cpdbf;
        BlobDBField  *bdbf;
        Blob          b;
        int           i = 1;
        void         *value = (void *) 0;
        int           size;

        BOOST_FOREACH(DBField &dbf, dbfv)
        {
          dbfk = dbf.getDBFieldType();
          switch(dbfk)
          {
            case VOIDP:
              vpdbf = (VOIDPDBField *) &dbf;
              vpdbf->getValue(value, size);
              stmt->Set(i++, value, size);
              free(value);
              value = (void *) 0;
              break;
            case CHARP:
              cpdbf = (CHARPDBField *) &dbf;
              cpdbf->getValue((unsigned char *&) value);
              stmt->Set(i++, (char *) value);
              free(value);
              value = (void *) 0;
              break;
            case BLOBT:
              bdbf = (BlobDBField *) &dbf;
              bdbf->getValue(b);
              stmt->Set(i++, b);
              break;
          }
        }
      }

      stmt->Execute();
      return tableID;
    }

    int64_t DatabaseModule::insertIntoTable(const string &tablePrefix, const string &tableName, 
                                            boost::ptr_vector<DBField> &dbfv, Transaction *tr)
                                            /* throw(...) */
    {
      if(!this->m_dbAssigned)
        throw DatabaseNotAssignedMirException("DatabaseModule::insertIntoTable(const string &prefix, const string &tableName, const DBFieldHashMap &dbfhm, Transaction &tr) ");
      
      if(!tr)
        throw TransactionNotAssignedMirException("DatabaseModule::insertIntoTable(const string &prefix, const string &tableName, const DBFieldHashMap &dbfhm, Transaction &tr) ");

      if(this->m_loggedUserId == 0)
        throw UserNotLoggedMirException("DatabaseModule::insertIntoTable(const string &prefix, const string &tableName, const DBFieldHashMap &dbfhm, Transaction &tr) ");

      ostringstream ostrstmt;
      ostrstmt << "INSERT INTO " << tableName;
      
      ostringstream dbFields;
      dbFields << " (" << tablePrefix << "_ID" 
               << "," << tablePrefix << "_AUDYT_UT_UZY_ID" 
               << "," << tablePrefix << "_AUDYT_DT";

      int64_t tableID = this->getNextID(tablePrefix, tr);
      ostringstream dbValues;
      dbValues << " VALUES(" << tableID 
               << "," << this->m_loggedUserId
               << ",CURRENT_TIMESTAMP";
      
      DBFieldType dbfk;
      bool isVOIDPorCHARP = false;

      BOOST_FOREACH(DBField &dbf, dbfv) 
      {
        dbFields << "," << dbf.getFieldName();
        dbfk = dbf.getDBFieldType();
        if (dbfk < BOOLT)
        {
          dbValues << ",?";
          isVOIDPorCHARP = true;
        } else
          dbValues << "," << dbf.getAsDBString();
      }
      dbFields << ")";
      dbValues << ")";

      ostrstmt << dbFields.str() << dbValues.str();

      string sql = ostrstmt.str();
      //Statement stmt = this->createStatement(tr, &ostrstmt.str());
      Statement stmt = StatementFactory(this->m_db, *tr, sql);

      if (isVOIDPorCHARP)
      {
        VOIDPDBField *vpdbf;
        CHARPDBField *cpdbf;
        BlobDBField  *bdbf;
        Blob          b;
        int           i = 1;
        void         *value = (void *) 0;
        int           size;

        BOOST_FOREACH(DBField &dbf, dbfv) 
        {
          dbfk = dbf.getDBFieldType();
          if (dbfk < BOOLT)
          {
            switch(dbfk)
            {
              case VOIDP:
                vpdbf = (VOIDPDBField *) &dbf;
                vpdbf->getValue(value, size);
                stmt->Set(i++, value, size);
                free(value);
                value = (void *) 0;
                break;
              case CHARP:
                cpdbf = (CHARPDBField *) &dbf;
                cpdbf->getValue((unsigned char *&) value);
                stmt->Set(i++, (char *) value);
                free(value);
                value = (void *) 0;
                break;
              case BLOBT:
                bdbf = (BlobDBField *) &dbf;
                bdbf->getValue(b);
                stmt->Set(i++, b);
                break;

            }
          }
        }
      }
      
      stmt->Execute();
      return tableID;
    }

    int64_t DatabaseModule::insertIntoTableNA(const string &tablePrefix, const string &tableName,
                                            boost::ptr_vector<DBField> &dbfv, Transaction *tr)
                                            /* throw(...) */
    {
      if(!this->m_dbAssigned)
        throw DatabaseNotAssignedMirException("DatabaseModule::insertIntoTableNA(const string &prefix, const string &tableName, const DBFieldHashMap &dbfhm, Transaction &tr) ");

      if(!tr)
        throw TransactionNotAssignedMirException("DatabaseModule::insertIntoTableNA(const string &prefix, const string &tableName, const DBFieldHashMap &dbfhm, Transaction &tr) ");

      if(this->m_loggedUserId == 0)
        throw UserNotLoggedMirException("DatabaseModule::insertIntoTableNA(const string &prefix, const string &tableName, const DBFieldHashMap &dbfhm, Transaction &tr) ");

      ostringstream ostrstmt;
      ostrstmt << "INSERT INTO " << tableName;

      ostringstream dbFields;
      dbFields << " (" << tablePrefix << "_ID";

      int64_t tableID = this->getNextID(tablePrefix, tr);
      ostringstream dbValues;
      dbValues << " VALUES(" << tableID;

      DBFieldType dbfk;
      bool isVOIDPorCHARP = false;

      BOOST_FOREACH(DBField &dbf, dbfv)
      {
        dbFields << "," << dbf.getFieldName();
        dbfk = dbf.getDBFieldType();
        if (dbfk < BOOLT)
        {
          dbValues << ",?";
          isVOIDPorCHARP = true;
        } else
          dbValues << "," << dbf.getAsDBString();
      }
      dbFields << ")";
      dbValues << ")";

      ostrstmt << dbFields.str() << dbValues.str();

      string sql = ostrstmt.str();
      Statement stmt = StatementFactory(this->m_db, *tr, sql);

      if (isVOIDPorCHARP)
      {
        VOIDPDBField *vpdbf;
        CHARPDBField *cpdbf;
        BlobDBField  *bdbf;
        Blob          b;
        int           i = 1;
        void         *value = (void *) 0;
        int           size;

        BOOST_FOREACH(DBField &dbf, dbfv)
        {
          dbfk = dbf.getDBFieldType();
          if (dbfk < BOOLT)
          {
            switch(dbfk)
            {
              case VOIDP:
                vpdbf = (VOIDPDBField *) &dbf;
                vpdbf->getValue(value, size);
                stmt->Set(i++, value, size);
                free(value);
                value = (void *) 0;
                break;
              case CHARP:
                cpdbf = (CHARPDBField *) &dbf;
                cpdbf->getValue((unsigned char *&) value);
                stmt->Set(i++, (char *) value);
                free(value);
                value = (void *) 0;
                break;
              case BLOBT:
                bdbf = (BlobDBField *) &dbf;
                bdbf->getValue(b);
                stmt->Set(i++, b);
                break;
            }
          }
        }
      }

      stmt->Execute();
      return tableID;
    }

    void DatabaseModule::updateTable(const int64_t tableId, const string &tablePrefix, 
                                     const string &tableName, boost::ptr_vector<DBField> &dbfv, 
                                     Transaction *tr)
                                     /* throw (...) */
    {
      if(this->m_loggedUserId == 0)
        throw UserNotLoggedMirException("DatabaseModule::updateTable(const int64_t tableId, const string &tablePrefix, const string &tableName, boost::ptr_vector<DBField> &dbfv, Transaction *tr) ");

      if(!this->m_dbAssigned)
        throw DatabaseNotAssignedMirException("DatabaseModule::updateTable(const int64_t tableId, const string &tablePrefix, const string &tableName, boost::ptr_vector<DBField> &dbfv, Transaction *tr) ");
      
      if(!tr)
        throw TransactionNotAssignedMirException("DatabaseModule::updateTable(const int64_t tableId, const string &tablePrefix, const string &tableName, boost::ptr_vector<DBField> &dbfv, Transaction *tr) ");

      ostringstream ostrstmt;
      ostrstmt << "UPDATE " << tableName;

      ostrstmt << " SET " << tablePrefix << "_AUDYT_UM_UZY_ID = "  << this->m_loggedUserId
                         << "," << tablePrefix << "_AUDYT_DM = CURRENT_TIMESTAMP"  ;
      
      DBFieldType dbfk;
      bool isVOIDPorCHARP = false;

      BOOST_FOREACH(DBField &dbf, dbfv) 
      {
        ostrstmt << "," << dbf.getFieldName();
        dbfk = dbf.getDBFieldType();
        if (dbfk < BOOLT)
        {
          ostrstmt << " = ?";
          isVOIDPorCHARP = true;
        } else
          ostrstmt << " = " << dbf.getAsDBString();
      }
      ostrstmt << " WHERE " << tablePrefix << "_ID = " << tableId;

      Statement stmt = StatementFactory(this->m_db, *tr, ostrstmt.str());

      if (isVOIDPorCHARP)
      {
        VOIDPDBField *vpdbf;
        CHARPDBField *cpdbf;
        BlobDBField  *bdbf;
        Blob          b;
        int           i = 1;
        void         *value = (void *) 0;
        int           size;

        BOOST_FOREACH(DBField &dbf, dbfv) 
        {
          dbfk = dbf.getDBFieldType();
          if (dbfk < BOOLT)
          {
            switch(dbfk)
            {
              case VOIDP:
                vpdbf = (VOIDPDBField *) &dbf;
                vpdbf->getValue(value, size);
                stmt->Set(i++, value, size);
                free(value);
                value = (void *) 0;
                break;
              case CHARP:
                cpdbf = (CHARPDBField *) &dbf;
                cpdbf->getValue((unsigned char *&) value);
                stmt->Set(i++, (char *) value);
                free(value);
                value = (void *) 0;
                break;
              case BLOBT:
                bdbf = (BlobDBField *) &dbf;
                bdbf->getValue(b);
                stmt->Set(i++, b);
                break;
            }
          }
        }
      }

      stmt->Execute();
    }


    void DatabaseModule::updateTable(const string  &whereSql, const string &tablePrefix, 
                                    const string &tableName, boost::ptr_vector<DBField> &dbfv, 
                                    Transaction *tr)
                                    /* throw (...) */
    {
      if(this->m_loggedUserId == 0)
        throw UserNotLoggedMirException("DatabaseModule::updateTable(const string  &whereSql, const string &tablePrefix, const string &tableName, boost::ptr_vector<DBField> &dbfv, Transaction *tr) ");

      if(!this->m_dbAssigned)
        throw DatabaseNotAssignedMirException("DatabaseModule::updateTable(const string  &whereSql, const string &tablePrefix, const string &tableName, boost::ptr_vector<DBField> &dbfv, Transaction *tr) ");
      
      if(!tr)
        throw TransactionNotAssignedMirException("DatabaseModule::updateTable(const string  &whereSql, const string &tablePrefix, const string &tableName, boost::ptr_vector<DBField> &dbfv, Transaction *tr) ");

      ostringstream ostrstmt;
      ostrstmt << "UPDATE " << tableName;

      ostrstmt << " SET " << tablePrefix << "_AUDYT_UM_UZY_ID = "  << this->m_loggedUserId
                         << "," << tablePrefix << "_AUDYT_DM = CURRENT_TIMESTAMP"  ;
      
      DBFieldType dbfk;
      bool isVOIDPorCHARP = false;

      BOOST_FOREACH(DBField &dbf, dbfv) 
      {
        ostrstmt << "," << dbf.getFieldName();
        dbfk = dbf.getDBFieldType();
        if (dbfk < BOOLT)
        {
          ostrstmt << " = ?";
          isVOIDPorCHARP = true;
        } else
          ostrstmt << " = " << dbf.getAsDBString();
      }
      ostrstmt << " WHERE " << whereSql;

      Statement stmt = StatementFactory(this->m_db, *tr, ostrstmt.str());

      if (isVOIDPorCHARP)
      {
        VOIDPDBField *vpdbf;
        CHARPDBField *cpdbf;
        BlobDBField  *bdbf;
        Blob          b;
        int           i = 1;
        void         *value = (void *) 0;
        int           size;

        BOOST_FOREACH(DBField &dbf, dbfv) 
        {
          dbfk = dbf.getDBFieldType();
          if (dbfk < BOOLT)
          {
            switch(dbfk)
            {
              case VOIDP:
                vpdbf = (VOIDPDBField *) &dbf;
                vpdbf->getValue(value, size);
                stmt->Set(i++, value, size);
                free(value);
                value = (void *) 0;
                break;
              case CHARP:
                cpdbf = (CHARPDBField *) &dbf;
                cpdbf->getValue((unsigned char *&) value);
                stmt->Set(i++, (char *) value);
                free(value);
                value = (void *) 0;
                break;
              case BLOBT:
                bdbf = (BlobDBField *) &dbf;
                bdbf->getValue(b);
                stmt->Set(i++, b);
                break;
            }
          }
        }
      }

      stmt->Execute();
    }

    void DatabaseModule::updateTableNA(const int64_t tableId, const string &tablePrefix,
                                     const string &tableName, boost::ptr_vector<DBField> &dbfv,
                                     Transaction *tr)
                                     /* throw (...) */
    {
      if(this->m_loggedUserId == 0)
        throw UserNotLoggedMirException("DatabaseModule::updateTableNA(const int64_t tableId, const string &tablePrefix, const string &tableName, boost::ptr_vector<DBField> &dbfv, Transaction *tr) ");

      if(!this->m_dbAssigned)
        throw DatabaseNotAssignedMirException("DatabaseModule::updateTableNA(const int64_t tableId, const string &tablePrefix, const string &tableName, boost::ptr_vector<DBField> &dbfv, Transaction *tr) ");

      if(!tr)
        throw TransactionNotAssignedMirException("DatabaseModule::updateTableNA(const int64_t tableId, const string &tablePrefix, const string &tableName, boost::ptr_vector<DBField> &dbfv, Transaction *tr) ");

      ostringstream ostrstmt;
      ostrstmt << "UPDATE " << tableName;

      ostrstmt << " SET ";

      DBFieldType dbfk;
      bool isVOIDPorCHARP = false, useComma = false;

      BOOST_FOREACH(DBField &dbf, dbfv)
      {

        ostrstmt << (useComma ? "," : "") << dbf.getFieldName();
        useComma = true;

        dbfk = dbf.getDBFieldType();
        if (dbfk < BOOLT)
        {
          ostrstmt << " = ?";
          isVOIDPorCHARP = true;
        } else
          ostrstmt << " = " << dbf.getAsDBString();
      }
      ostrstmt << " WHERE " << tablePrefix << "_ID = " << tableId;

      Statement stmt = StatementFactory(this->m_db, *tr, ostrstmt.str());

      if (isVOIDPorCHARP)
      {
        VOIDPDBField *vpdbf;
        CHARPDBField *cpdbf;
        BlobDBField  *bdbf;
        Blob          b;
        int           i = 1;
        void         *value = (void *) 0;
        int           size;

        BOOST_FOREACH(DBField &dbf, dbfv)
        {
          dbfk = dbf.getDBFieldType();
          if (dbfk < BOOLT)
          {
            switch(dbfk)
            {
              case VOIDP:
                vpdbf = (VOIDPDBField *) &dbf;
                vpdbf->getValue(value, size);
                stmt->Set(i++, value, size);
                free(value);
                value = (void *) 0;
                break;
              case CHARP:
                cpdbf = (CHARPDBField *) &dbf;
                cpdbf->getValue((unsigned char *&) value);
                stmt->Set(i++, (char *) value);
                free(value);
                value = (void *) 0;
                break;
              case BLOBT:
                bdbf = (BlobDBField *) &dbf;
                bdbf->getValue(b);
                stmt->Set(i++, b);
                break;
            }
          }
        }
      }

      stmt->Execute();
    }


    void DatabaseModule::updateTableNA(const string  &whereSql, const string &tablePrefix,
                                    const string &tableName, boost::ptr_vector<DBField> &dbfv,
                                    Transaction *tr)
                                    /* throw (...) */
    {
      if(this->m_loggedUserId == 0)
        throw UserNotLoggedMirException("DatabaseModule::updateTableNA(const string  &whereSql, const string &tablePrefix, const string &tableName, boost::ptr_vector<DBField> &dbfv, Transaction *tr) ");

      if(!this->m_dbAssigned)
        throw DatabaseNotAssignedMirException("DatabaseModule::updateTableNA(const string  &whereSql, const string &tablePrefix, const string &tableName, boost::ptr_vector<DBField> &dbfv, Transaction *tr) ");

      if(!tr)
        throw TransactionNotAssignedMirException("DatabaseModule::updateTableNA(const string  &whereSql, const string &tablePrefix, const string &tableName, boost::ptr_vector<DBField> &dbfv, Transaction *tr) ");

      ostringstream ostrstmt;
      ostrstmt << "UPDATE " << tableName;

      ostrstmt << " SET ";

      DBFieldType dbfk;
      bool isVOIDPorCHARP = false, useComma = false;

      BOOST_FOREACH(DBField &dbf, dbfv)
      {
        ostrstmt << (useComma ? "," : "") << dbf.getFieldName();
        useComma = true;

        dbfk = dbf.getDBFieldType();
        if (dbfk < BOOLT)
        {
          ostrstmt << " = ?";
          isVOIDPorCHARP = true;
        } else
          ostrstmt << " = " << dbf.getAsDBString();
      }
      ostrstmt << " WHERE " << whereSql;

      Statement stmt = StatementFactory(this->m_db, *tr, ostrstmt.str());

      if (isVOIDPorCHARP)
      {
        VOIDPDBField *vpdbf;
        CHARPDBField *cpdbf;
        BlobDBField  *bdbf;
        Blob          b;
        int           i = 1;
        void         *value = (void *) 0;
        int           size;

        BOOST_FOREACH(DBField &dbf, dbfv)
        {
          dbfk = dbf.getDBFieldType();
          if (dbfk < BOOLT)
          {
            switch(dbfk)
            {
              case VOIDP:
                vpdbf = (VOIDPDBField *) &dbf;
                vpdbf->getValue(value, size);
                stmt->Set(i++, value, size);
                free(value);
                value = (void *) 0;
                break;
              case CHARP:
                cpdbf = (CHARPDBField *) &dbf;
                cpdbf->getValue((unsigned char *&) value);
                stmt->Set(i++, (char *) value);
                free(value);
                value = (void *) 0;
                break;
              case BLOBT:
                bdbf = (BlobDBField *) &dbf;
                bdbf->getValue(b);
                stmt->Set(i++, b);
                break;
            }
          }
        }
      }

      stmt->Execute();
    }

    void  DatabaseModule::deleteFromTable(const int64_t tableId, const string &tablePrefix, 
                                          const string &tableName, Transaction *tr)
                                          /* throw (...) */
    {
      if(this->m_loggedUserId == 0)
        throw UserNotLoggedMirException("DatabaseModule::deleteFromTable(const int64_t tableId, const string &tablePrefix, const string &tableName, Transaction *tr)");

      if(!this->m_dbAssigned)
        throw DatabaseNotAssignedMirException("DatabaseModule::deleteFromTable(const int64_t tableId, const string &tablePrefix, const string &tableName, Transaction *tr)");
      
      if(!tr)
        throw TransactionNotAssignedMirException("DatabaseModule::deleteFromTable(const int64_t tableId, const string &tablePrefix, const string &tableName, Transaction *tr)");

      ostringstream ostrstmt;
      ostrstmt << "DELETE FROM " << tableName;
      if (tableId > 0)
        ostrstmt << " WHERE " << tablePrefix << "_ID = "  << tableId;

      Statement stmt = StatementFactory(this->m_db, *tr, ostrstmt.str());
      stmt->Execute();
    }


    void DatabaseModule::deleteFromTable(const string &whereSql, const string &tablePrefix, 
                                    const string &tableName, Transaction *tr)
                                    /* throw (...) */
    {
      if(this->m_loggedUserId == 0)
        throw UserNotLoggedMirException("DatabaseModule::deleteFromTable(const string &whereSql, const string &tablePrefix, const string &tableName, Transaction *tr)");

      if(!this->m_dbAssigned)
        throw DatabaseNotAssignedMirException("DatabaseModule::deleteFromTable(const string &whereSql, const string &tablePrefix, const string &tableName, Transaction *tr)");
      
      if(!tr)
        throw TransactionNotAssignedMirException("DatabaseModule::deleteFromTable(const string &whereSql, const string &tablePrefix, const string &tableName, Transaction *tr)");

      ostringstream ostrstmt;
      ostrstmt << "DELETE FROM " << tableName;
      ostrstmt << " WHERE " << whereSql;

      Statement stmt = StatementFactory(this->m_db, *tr, ostrstmt.str());
      stmt->Execute();
    }

    void DatabaseModule::login(const string login, const string password, Transaction *tr, bool isSha1) /* throw (...) */
    {
      if(!this->m_dbAssigned)
        throw DatabaseNotAssignedMirException("DatabaseModule::login(const string login, const string password, Transaction *tr, bool isSha1)");
      
      if(!tr)
        throw TransactionNotAssignedMirException("DatabaseModule::login(const string login, const string password, Transaction *tr, bool isSha1)");

      ostringstream ostrstmt;
      
      ostrstmt << "SELECT UZY_HASLO, UZY_ID, UZY_IMIE, UZY_NAZWISKO, UZY_PRAWA FROM MIR_UZYTKOWNIK WHERE UZY_IDENTYFIKATOR = " << toSql(login);

      Statement stmt = StatementFactory(this->m_db, *tr, ostrstmt.str());
      stmt->Execute();
      if (stmt->Fetch())
      {
        char    hashdb[20], hashpass[20];
        int     size = 20;
        stmt->Get(1, (void *) hashdb, size);
        if (!isSha1)
        {
          CSHA1 cs;
          cs.Update((const UINT_8*) password.c_str(), (UINT_32) password.size());
          cs.Final();
          cs.GetHash((unsigned char *) hashpass);
        }
        else
        {
          memcpy(hashpass, password.c_str(), 20);
        }

        for(register int len = 0; len < 20; len++)
        {
          if (hashdb[len] !=  hashpass[len])
          {
            this->m_loggedUserId = 0;
            throw IncorrectPasswordMirException("DatabaseModule::login(const string login, const string password, Transaction *tr, bool isSha1)");
          }
        }
        stmt->Get(2, this->m_loggedUserId);
        stmt->Get(3, this->m_name);
        stmt->Get(4, this->m_surname);
        this->m_hashpass = string(hashpass, 20);
        stmt->Get(5, this->m_userPrivilages);
      }
      else
      {
        this->m_loggedUserId = 0;
        this->m_userPrivilages = 0;
        throw UserDoesntExistsMirException("DatabaseModule::login(const string login, const string password, Transaction *tr, bool isSha1)");
      }
    }

    bool DatabaseModule::logged()
    {
      return this->m_loggedUserId != 0;
    }

    string DatabaseModule::getName()
    {
      return this->m_name;
    }

    string DatabaseModule::getSurname()
    {
      return this->m_surname;
    }

    string DatabaseModule::getHashPassword()
    {
      return this->m_hashpass;
    }

    int64_t DatabaseModule::getUserID()
    {
      return this->m_loggedUserId;
    }

    bool DatabaseModule::isUser(int64_t privilage)
    {
      return (privilage & this->m_userPrivilages);
    }

    bool DatabaseModule::logout()
    {
      this->m_loggedUserId = 0;
      this->m_userPrivilages = MIR_NOTACTIVE;
      return true;
    }


//    const boost::regex DatabaseModule::mcs_regex("'");
//
//    const boost::regex DatabaseModule::mcs_similar_to_regex("_");

  } // namespace MIR
} // namespace SyNaT
