﻿//#include "mir_stringutils.h"
#include "mir_exceptions.h"

namespace SyNaT
{
  namespace MIR
  {
    
    CommonMirException::CommonMirException()
    {
    }

    CommonMirException::CommonMirException(const CommonMirException &e) : m_origin(e.m_origin), m_what(e.m_what)
    {
    }

    CommonMirException::CommonMirException(const UnicodeString &origin)
    {
      origin.toUTF8String(this->m_origin);
    }

    CommonMirException::CommonMirException(const UnicodeString &origin, const UnicodeString &what)
    {
      what.toUTF8String(this->m_what);
      origin.toUTF8String(this->m_origin);
    }

    CommonMirException::CommonMirException(const string &origin) : m_origin(origin)
    {
    }

    CommonMirException::CommonMirException(const string &origin, const string &what) : m_what(what), m_origin(origin)
    {
    }

    CommonMirException::CommonMirException(const char *origin) : m_origin(origin)
    {
    }

    CommonMirException::CommonMirException(const char *origin, const char *what) : m_what(what), m_origin(origin)
    {
    }

    CommonMirException::~CommonMirException() throw()
    {
    }

    const char     *CommonMirException::Origin() const throw()
    {
      return this->m_origin.c_str();
    }

    const char     *CommonMirException::what() const throw()
    {
      return this->m_what.c_str();
    }

    const char     *CommonMirException::ErrorMessage() const throw()
    {
      return this->m_what.c_str();
    }

    UnicodeString CommonMirException::icuorigin() const throw()
    {
      return UnicodeString::fromUTF8(this->m_origin);
    }

    UnicodeString CommonMirException::icuwhat() const throw()
    {
      return UnicodeString::fromUTF8(this->m_what);
    }

    void           CommonMirException::setOrigin(string &origin) throw()
    {
      this->m_origin = origin;
    }

    void           CommonMirException::setWhat(string &what) throw()
    {
      this->m_what = what;
    }

    string         CommonMirException::getOrigin() const throw()
    {
      return string(this->m_origin);
    }

    string         CommonMirException::getWhat() const throw()
    {
      return string(this->m_what);
    }



    DatabaseNotAssignedMirException::DatabaseNotAssignedMirException() : CommonMirException()
    {
    }

    DatabaseNotAssignedMirException::DatabaseNotAssignedMirException(const DatabaseNotAssignedMirException &e) : CommonMirException(e)
    {
    }

    DatabaseNotAssignedMirException::DatabaseNotAssignedMirException(const UnicodeString &origin) : CommonMirException(origin)
    {
    }

    DatabaseNotAssignedMirException::DatabaseNotAssignedMirException(const UnicodeString &origin, const UnicodeString &what) : CommonMirException(origin, what)
    {
    }

    DatabaseNotAssignedMirException::DatabaseNotAssignedMirException(const string &origin) : CommonMirException(origin)
    {
    }

    DatabaseNotAssignedMirException::DatabaseNotAssignedMirException(const string &origin, const string &what) : CommonMirException(origin, what)
    {
    }

    DatabaseNotAssignedMirException::DatabaseNotAssignedMirException(const char *origin) : CommonMirException(origin)
    {
    }

    DatabaseNotAssignedMirException::DatabaseNotAssignedMirException(const char *origin, const char *what) : CommonMirException(origin, what)
    {
    }



    TransactionNotAssignedMirException::TransactionNotAssignedMirException() : CommonMirException()
    {
    }

    TransactionNotAssignedMirException::TransactionNotAssignedMirException(const TransactionNotAssignedMirException &e) : CommonMirException(e)
    {
    }

    TransactionNotAssignedMirException::TransactionNotAssignedMirException(const UnicodeString &origin) : CommonMirException(origin)
    {
    }

    TransactionNotAssignedMirException::TransactionNotAssignedMirException(const UnicodeString &origin, const UnicodeString &what) : CommonMirException(origin, what)
    {
    }

    TransactionNotAssignedMirException::TransactionNotAssignedMirException(const string &origin) : CommonMirException(origin)
    {
    }

    TransactionNotAssignedMirException::TransactionNotAssignedMirException(const string &origin, const string &what) : CommonMirException(origin, what)
    {
    }

    TransactionNotAssignedMirException::TransactionNotAssignedMirException(const char *origin) : CommonMirException(origin)
    {
    }

    TransactionNotAssignedMirException::TransactionNotAssignedMirException(const char *origin, const char *what) : CommonMirException(origin, what)
    {
    }



    UserNotLoggedMirException::UserNotLoggedMirException() : CommonMirException()
    {
    }

    UserNotLoggedMirException::UserNotLoggedMirException(const UserNotLoggedMirException &e) : CommonMirException(e)
    {
    }

    UserNotLoggedMirException::UserNotLoggedMirException(const UnicodeString &origin) : CommonMirException(origin)
    {
    }

    UserNotLoggedMirException::UserNotLoggedMirException(const UnicodeString &origin, const UnicodeString &what) : CommonMirException(origin, what)
    {
    }

    UserNotLoggedMirException::UserNotLoggedMirException(const string &origin) : CommonMirException(origin)
    {
    }

    UserNotLoggedMirException::UserNotLoggedMirException(const string &origin, const string &what) : CommonMirException(origin, what)
    {
    }

    UserNotLoggedMirException::UserNotLoggedMirException(const char *origin) : CommonMirException(origin)
    {
    }

    UserNotLoggedMirException::UserNotLoggedMirException(const char *origin, const char *what) : CommonMirException(origin, what)
    {
    }



    UserDoesntExistsMirException::UserDoesntExistsMirException() : CommonMirException()
    {
    }

    UserDoesntExistsMirException::UserDoesntExistsMirException(const UserDoesntExistsMirException &e) : CommonMirException(e)
    {
    }

    UserDoesntExistsMirException::UserDoesntExistsMirException(const UnicodeString &origin) : CommonMirException(origin)
    {
    }

    UserDoesntExistsMirException::UserDoesntExistsMirException(const UnicodeString &origin, const UnicodeString &what) : CommonMirException(origin, what)
    {
    }

    UserDoesntExistsMirException::UserDoesntExistsMirException(const string &origin) : CommonMirException(origin)
    {
    }

    UserDoesntExistsMirException::UserDoesntExistsMirException(const string &origin, const string &what) : CommonMirException(origin, what)
    {
    }

    UserDoesntExistsMirException::UserDoesntExistsMirException(const char *origin) : CommonMirException(origin)
    {
    }

    UserDoesntExistsMirException::UserDoesntExistsMirException(const char *origin, const char *what) : CommonMirException(origin, what)
    {
    }



    IncorrectPasswordMirException::IncorrectPasswordMirException() : CommonMirException()
    {
    }

    IncorrectPasswordMirException::IncorrectPasswordMirException(const IncorrectPasswordMirException &e) : CommonMirException(e)
    {
    }

    IncorrectPasswordMirException::IncorrectPasswordMirException(const UnicodeString &origin) : CommonMirException(origin)
    {
    }

    IncorrectPasswordMirException::IncorrectPasswordMirException(const UnicodeString &origin, const UnicodeString &what) : CommonMirException(origin, what)
    {
    }

    IncorrectPasswordMirException::IncorrectPasswordMirException(const string &origin) : CommonMirException(origin)
    {
    }

    IncorrectPasswordMirException::IncorrectPasswordMirException(const string &origin, const string &what) : CommonMirException(origin, what)
    {
    }

    IncorrectPasswordMirException::IncorrectPasswordMirException(const char *origin) : CommonMirException(origin)
    {
    }

    IncorrectPasswordMirException::IncorrectPasswordMirException(const char *origin, const char *what) : CommonMirException(origin, what)
    {
    }



    DbTableIsEmptyMirException::DbTableIsEmptyMirException() : CommonMirException()
    {
    }

    DbTableIsEmptyMirException::DbTableIsEmptyMirException(const DbTableIsEmptyMirException &e) : CommonMirException(e)
    {
    }

    DbTableIsEmptyMirException::DbTableIsEmptyMirException(const UnicodeString &origin) : CommonMirException(origin)
    {
    }

    DbTableIsEmptyMirException::DbTableIsEmptyMirException(const UnicodeString &origin, const UnicodeString &what) : CommonMirException(origin, what)
    {
    }

    DbTableIsEmptyMirException::DbTableIsEmptyMirException(const string &origin) : CommonMirException(origin)
    {
    }

    DbTableIsEmptyMirException::DbTableIsEmptyMirException(const string &origin, const string &what) : CommonMirException(origin, what)
    {
    }

    DbTableIsEmptyMirException::DbTableIsEmptyMirException(const char *origin) : CommonMirException(origin)
    {
    }

    DbTableIsEmptyMirException::DbTableIsEmptyMirException(const char *origin, const char *what) : CommonMirException(origin, what)
    {
    }




    BaseURIIsAReferenceMirException::BaseURIIsAReferenceMirException() : CommonMirException()
    {
    }

    BaseURIIsAReferenceMirException::BaseURIIsAReferenceMirException(const BaseURIIsAReferenceMirException &e) : CommonMirException(e)
    {
    }

    BaseURIIsAReferenceMirException::BaseURIIsAReferenceMirException(const UnicodeString &origin) : CommonMirException(origin)
    {
    }

    BaseURIIsAReferenceMirException::BaseURIIsAReferenceMirException(const UnicodeString &origin, const UnicodeString &what) : CommonMirException(origin, what)
    {
    }

    BaseURIIsAReferenceMirException::BaseURIIsAReferenceMirException(const string &origin) : CommonMirException(origin)
    {
    }

    BaseURIIsAReferenceMirException::BaseURIIsAReferenceMirException(const string &origin, const string &what) : CommonMirException(origin, what)
    {
    }

    BaseURIIsAReferenceMirException::BaseURIIsAReferenceMirException(const char *origin) : CommonMirException(origin)
    {
    }

    BaseURIIsAReferenceMirException::BaseURIIsAReferenceMirException(const char *origin, const char *what) : CommonMirException(origin, what)
    {
    }




    RegexMatcherMirException::RegexMatcherMirException() : CommonMirException()
    {
    }

    RegexMatcherMirException::RegexMatcherMirException(const RegexMatcherMirException &e) : CommonMirException(e)
    {
    }

    RegexMatcherMirException::RegexMatcherMirException(const UnicodeString &origin) : CommonMirException(origin)
    {
    }

    RegexMatcherMirException::RegexMatcherMirException(const UnicodeString &origin, const UnicodeString &what) : CommonMirException(origin, what)
    {
    }

    RegexMatcherMirException::RegexMatcherMirException(const string &origin) : CommonMirException(origin)
    {
    }

    RegexMatcherMirException::RegexMatcherMirException(const string &origin, const string &what) : CommonMirException(origin, what)
    {
    }

    RegexMatcherMirException::RegexMatcherMirException(const char *origin) : CommonMirException(origin)
    {
    }

    RegexMatcherMirException::RegexMatcherMirException(const char *origin, const char *what) : CommonMirException(origin, what)
    {
    }

  }
}
