#include <typeinfo>

#include <Wt/WLogger>
#include <Wt/WMenu>
#include <Wt/WSubMenuItem>
#include <Wt/WHBoxLayout>
#include <Wt/WVBoxLayout>
#include <Wt/WStackedWidget>
#include <Wt/WContainerWidget>
#include <Wt/WText>
#include <Wt/WTableCell>

#include "utils/mir_trkeys.h"
#include "utils/mir_cssclasses.h"
#include "utils/mir_internalpaths.h"
#include "widgets/mir_deferredwidget.h"
#include "controls/mir_sp_musicrobotcontrols.h"
#include "controls/mir_sp_userscontrols.h"
#include "controls/mir_sp_explorercontrols.h"
#include "controls/mir_sp_searchenginecontrols.h"
#include "controls/mir_sp_signoutcontrols.h"
#include "controls/mir_sp_logincontrols.h"
#include "mir_sp_menuandcontents.h"

namespace SyNaT
{
  namespace MIR
  {
    namespace SP
    {
      using namespace Wt;

      MenuAndContents::MenuAndContents(SyNaTPGApp *app, DatabaseModule *dm)
        : WContainerWidget(), m_dm(dm), m_app(app)
      {
        this->m_contentsStack_ = new WStackedWidget();
        WAnimation fade(WAnimation::Fade, WAnimation::Linear, 250);
        this->m_contentsStack_->setTransitionAnimation(fade);
        // Show scrollbars when needed ...
        this->m_contentsStack_->setOverflow(WContainerWidget::OverflowAuto);
        // ... and work around a bug in IE (see setOverflow() documentation)
        this->m_contentsStack_->setPositionScheme(Relative);
        this->m_contentsStack_->setStyleClass(MIR_STRCAT3(MIR_CSS_CONTENTS," ",MIR_CSS_MAIN_PAGE));
        /*
         * Setup the menu (and submenus)
         */
        WMenu *menu = new WMenu(this->m_contentsStack_, Vertical, 0);
        menu->setRenderAsList(true);
        menu->setStyleClass(MIR_STRINGIFY(MIR_CSS_MENU));
        menu->setInternalPathEnabled();
        //menu->setInternalBasePath("/");
        WMenuItem *mi;
        mi = menu->addItem(tr(MIR_MENU_SIGNIN), new LoginControls(this->m_app, menu, this->m_dm, WApplication::instance()->internalPath(), true, MIR_STRINGIFY(MIR_CSS_MARGIN_FOR_ANIM)));
        mi->setPathComponent(MIR_IPATH_SIGNIN);

        if (this->m_dm->isUser(MIR_MUSIC_ROBOT_OPERATOR))
        {
//            MenuAndContents::addToMenu(menu, tr(MIR_MENU_MUSICROBOT), new MusicRobotControls(this->m_app, menu, this->m_dm, MIR_STRINGIFY(MIR_CSS_MARGIN_FOR_ANIM)))
//              ->setPathComponent(MIR_IPATH_MUSICROBOT);
        }

        if (this->m_dm->isUser(MIR_USER_OPERATOR))
        {
//            MenuAndContents::addToMenu(menu, tr(MIR_MENU_USERS), new UsersControls(this->m_app, menu, this->m_dm, MIR_STRINGIFY(MIR_CSS_MARGIN_FOR_ANIM)))
//              ->setPathComponent(MIR_IPATH_USERS);
        }

        mi = MenuAndContents::addToMenu(menu, tr(MIR_MENU_EXPLORER), new ExplorerControls(this->m_app, menu, this->m_dm, MIR_STRINGIFY(MIR_CSS_MARGIN_FOR_ANIM)));
        mi->setPathComponent(MIR_IPATH_EXPLORER);
        mi->hide();

//          MenuAndContents::addToMenu(menu, tr(MIR_MENU_SEARCHENGINE), new SearchEngineControls(this->m_app, menu, this->m_dm, MIR_STRINGIFY(MIR_CSS_MARGIN_FOR_ANIM)))
//            ->setPathComponent(MIR_IPATH_SEARCHENGINE);

        mi = MenuAndContents::addToMenu(menu, tr(MIR_MENU_SIGNOUT), new SignOutControls(this->m_app, menu, this->m_dm, MIR_STRINGIFY(MIR_CSS_MARGIN_FOR_ANIM)));
        mi->setPathComponent(MIR_IPATH_SIGNOUT);
        mi->hide();

        menu->itemSelected().connect(this, &MenuAndContents::slt_menuItemSelected);
        /*
         * Add it all inside a layout
         */
        WHBoxLayout *horizLayout = new WHBoxLayout(this);
        WVBoxLayout *vertLayout = new WVBoxLayout;

        horizLayout->addWidget(menu, 0);
        horizLayout->addLayout(vertLayout, 1);
        vertLayout->addWidget(m_contentsStack_, 1);

        horizLayout->setResizable(0, true);
      }

      WMenuItem *MenuAndContents::addToMenu(WMenu *menu, const WString& name,
                  ControlsWidget *controls)
      {
        if (controls->hasSubMenu()) {
          WSubMenuItem *smi = new WSubMenuItem(name, controls);
          WMenu *subMenu = new WMenu(menu->contentsStack()/*contentsStack_*/, Vertical, 0);
          subMenu->setRenderAsList(true);

          smi->setSubMenu(subMenu);
          WMenuItem *wmi = menu->addItem(smi);

          subMenu->setInternalPathEnabled();
          subMenu->setInternalBasePath("/" + smi->pathComponent());
          subMenu->setStyleClass(MIR_STRCAT3(MIR_CSS_MENU," ",MIR_CSS_SUBMENU));

          controls->populateSubMenu(subMenu);
          return wmi;
        } else
        {
          return menu->addItem(name, controls);
        }
      }

      void MenuAndContents::slt_menuItemSelected(WMenuItem *mi)
      {
        WContainerWidget *contents = dynamic_cast<WContainerWidget *>(mi->contents());

        if (mi->text() == tr(MIR_MENU_SIGNIN))
        {
          try
          {
            LoginControls *lc = dynamic_cast<LoginControls *>(contents->widget(0));
            if (lc->m_tcell)
              lc->m_tcell->removeWidget(this->m_app->m_wtappmessage);
            this->m_app->m_wtappmessage->setText(WString("", UTF8));
            lc->clear();
            lc->m_tcell = NULL;
            lc->addWidget(lc->create_signin_page());
          }
          catch(std::exception &e)
          {
            this->m_app->log("error") << e.what();
          }

        } else if (mi->text() == tr(MIR_MENU_EXPLORER))

        {
          ExplorerControls *ec = dynamic_cast<ExplorerControls *>(contents->widget(0));
          if (this->m_dm->logged())
          {
            try
            {
              if (!ec->wasRendered())
                ec->addWidget(ec->create_explorer_controls());
            }
            catch(std::exception &e)
            {
              this->m_app->log("error") << e.what();
            }
          }
          else
            ec->clear();

        } else if (mi->text() == tr(MIR_MENU_SIGNOUT))

        {
          SignOutControls *soc = dynamic_cast<SignOutControls *>(contents->widget(0));
          if (this->m_dm->logged())
          {
            try
            {
              if (!soc->wasRendered())
                soc->addWidget(soc->create_signout_controls());
            }
            catch(std::exception &e)
            {
              this->m_app->log("error") << e.what();
            }
          }
          else
            soc->clear();
        }

        return;
      }
    }
  }
}
