#include <Wt/WApplication>
#include <Wt/WLogger>
#include <Wt/WObject>
#include <Wt/WStreamResource>
#include <Wt/Http/Request>
#include <Wt/Http/Response>
#include <Utils.h>

#include <unicode/unistr.h>

#include <boost/lexical_cast.hpp>
#include <boost/filesystem.hpp>
#include <boost/filesystem/fstream.hpp>
#include <boost/smart_ptr/scoped_array.hpp>


#include <Poco/Platform.h>
#include <Poco/Path.h>

#include <ibpp.h>

#include <iostream>

#include <ZenLib/Ztring.h>

#include "utils/mir_utils.h"
#include "utils/mir_stringutils.h"
#include "mir_databasemodule.h"
#include "mir_sp_synatpgapp.h"
#include "resources/mir_sp_musicresource.h"

namespace SyNaT
{
  namespace MIR
  {
    namespace SP
    {
      using namespace Wt;
      using namespace IBPP;
      using namespace boost::filesystem;
      using namespace std;

      MusicResource::MusicResource(int iterations, DatabaseModule *dm,
          long long utwId, WObject *parent) :
          WStreamResource(parent), m_iterations(iterations), m_dm(dm), m_utwId(utwId)
      {
        this->setBufferSize(100 * 1024);
      }

      MusicResource::~MusicResource()
      {
        this->beingDeleted();
      }

      void MusicResource::handleRequest(const Wt::Http::Request& request,
          Wt::Http::Response& response)
      {
        Wt::Http::ResponseContinuation *continuation = request.continuation();
        bool isIE = false;
        bool isChrome = false;
        bool isLynx = false;
        isIE = request.userAgent().find("MSIE") != std::string::npos;
        isChrome = request.userAgent().find("Chrome") != std::string::npos;
        isLynx = request.userAgent().find("Lynx") != std::string::npos;
        string ext;
        if (!continuation)
        {
          ostringstream oss;
          bool audioFetched = !this->m_sciezka_do_pliku.empty();
          int iteration;
          try
          {
            if (this->m_sciezka_do_pliku.empty())
            {
              Transaction tr = this->m_dm->createTransaction(amRead,
                  ilReadCommitted, lrWait);
              tr->Start();
              ostringstream sql;
              sql << "select u.UTW_SCIEZKA_UTWORU, g.GAT_GATUNEK, ar.ART_NAZWA_ARTYSTY "
                     " , a.ALB_NAZWA_ALBUMU, u.UTW_TYTUL_UTWORU"
                     " from MIR_UTWOR u "
                     " inner join MIR_ALBUM a on a.ALB_ID = u.UTW_ALB_ID "
                     " inner join MIR_ARTYSTA ar on ar.ART_ID = u.UTW_ART_ID"
                     " inner join MIR_GATUNEK g on g.GAT_ID = u.UTW_GAT_ID"
                     " where u.UTW_ID = " << this->m_utwId;
              string sqlstmt = sql.str();
              Statement stmt = this->m_dm->createStatement(&tr);
              stmt->Prepare(sqlstmt);
              stmt->Execute();

              if (stmt->Fetch())
              {
                stmt->Get(1, this->m_sciezka_do_pliku);
                if (SyNaTPGApp::mcs_osid == POCO_OS_WINDOWS_NT)
                  this->m_sciezka_do_pliku = SyNaT::MIR::SP::SyNaTPGApp::mcs_root_elem
                      + this->m_sciezka_do_pliku;

                if (!(isIE || isChrome || isLynx))
                {
                  string dbS;
                  stmt->Get(2, dbS);
                  oss << dbS << "_";
                  stmt->Get(3, dbS);
                  oss << dbS << "_";
                  stmt->Get(4, dbS);
                  oss << dbS << "_";
                  stmt->Get(5, dbS);
                  oss << dbS;
                }
                else
                {
                  oss << "utwor" << this->m_utwId;
                }
                this->m_suggestedFileName = oss.str();
                if (this->m_suggestedFileName.length() > 245)
                  this->m_suggestedFileName = this->m_suggestedFileName.substr(0, 245);

                this->m_suggestedFileName += "." + ext;

                UnicodeString icusuggestedFileName = UnicodeString::fromUTF8(this->m_suggestedFileName).findAndReplace("\"", "_");
                this->m_suggestedFileName.clear();
                icusuggestedFileName.toUTF8String(this->m_suggestedFileName);

              }
              tr->Commit();
              audioFetched = true;
            }
          }
          catch(std::exception &e)
          {
            WApplication::instance()->log("error") << e.what();
          }
          if (audioFetched)
          {
            if (this->m_generic_wstr.empty())
              this->m_generic_wstr = ZenLib::Ztring(((SyNaT::MIR::SP::SyNaTPGApp::mcs_osid == POCO_OS_WINDOWS_NT ? "\\\\?\\" : "") + Poco::Path(this->m_sciezka_do_pliku, Poco::Path::PATH_UNIX).toString()).c_str()).To_Unicode();

            if (!continuation && !this->m_suggestedFileName.empty())
            {
        //// Browser incompatibility hell: internatianalized filename suggestions
        //// First filename is for browsers that don't support RFC 5987
        //// Second filename is for browsers that do support RFC 5987
        //std::string fileField;
        //// We cannot query wApp here, because wApp doesn't exist for
        //// static resources.
        //bool isIE = webRequest->userAgent().find("MSIE") != std::string::npos;
        //bool isChrome = webRequest->userAgent().find("Chrome") != std::string::npos;
        //if (isIE || isChrome) {
        //  // filename="foo-%c3%a4-%e2%82%ac.html"
        //  // Note: IE never converts %20 back to space, so avoid escaping
        //  // IE wil also not url decode the filename if the file has no ASCII
        //  // extension (e.g. .txt)
        //  fileField = "filename=\"" +
        //    Utils::urlEncode(suggestedFileName_.toUTF8(), " ") + "\";";
        //} else {
        //  // Binary UTF-8 sequence: for FF3, Safari, Chrome, Chrome9
        //  fileField = "filename=\"" + suggestedFileName_.toUTF8() + "\";";
        //}
        //// Next will be picked by RFC 5987 in favour of the
        //// one without specified encoding (Chrome9,
        //fileField+= Utils::EncodeHttpHeaderField("filename", suggestedFileName_);
        //response.addHeader("Content-Disposition",
        //  theDisposition + ";" + fileField);
              m_file_size = file_size(m_generic_wstr);

              response.setContentLength(m_file_size);

              std::string fileField;
              if (isIE || isChrome)
              {
                fileField = "filename=\""
                    + Wt::Utils::urlEncode(this->m_suggestedFileName, " ") + "\";";
              }
              else
              {
                fileField = "filename=\"" + this->m_suggestedFileName + "\";";
              }
              // Next will be picked by RFC 5987 in favour of the
              // one without specified encoding (Chrome9,
              fileField+= Wt::Utils::EncodeHttpHeaderField("filename", WString::fromUTF8(this->m_suggestedFileName));

              response.addHeader("Content-Disposition", "attachment;" + fileField);

            }

            ext = this->m_sciezka_do_pliku.substr(this->m_sciezka_do_pliku.find_last_of(".")).substr(1);

            if (!ext.empty())
            {
              string mime_type = Utils::findContentTypeByExt(ext);
              if (!mime_type.empty())
                response.setMimeType(mime_type);
            }

            response.addHeader("Cache-Control", "no-cache, must-revalidate");
            response.addHeader("Expires", "Mon, 26 May 1997 05:00:00 GMT");
          }


        }
        if (!this->m_generic_wstr.empty())
        {
          boost::filesystem::ifstream f(m_generic_wstr, boost::filesystem::ifstream::binary);
          if (!isLynx)
            this->handleRequestPiecewise(request, response, f);
          else
          {
            f.seekg(0, ios_base::beg);
            boost::scoped_array<char> buf(new char[m_file_size + 1]);

            f.read(buf.get(), m_file_size + 1);

            // if we have not yet finished, create a continuation to serve more
            f.close();
            response.out().write(buf.get(), m_file_size);

          }
        }
      }
    }
  }
}
