#include <Wt/WApplication>
#include <Wt/WLogger>
#include <Wt/WObject>
#include <Wt/WResource>
#include <Wt/Http/Request>
#include <Wt/Http/Response>
#include <Utils.h>

#include <unicode/unistr.h>

#include <boost/filesystem.hpp>

#include <Poco/Platform.h>

#include <ibpp.h>

#include <iostream>
#include <fstream>

#include "utils/mir_utils.h"
#include "utils/mir_stringutils.h"
#include "mir_databasemodule.h"
#include "mir_sp_synatpgapp.h"
#include "resources/mir_sp_paramsresource.h"

namespace SyNaT
{
  namespace MIR
  {
    namespace SP
    {
      using namespace Wt;
      using namespace IBPP;
      using namespace boost::filesystem;
      using namespace std;

      ParamsResource::ParamsResource(DatabaseModule *dm,
          long long utwId, WObject *parent) :
          WResource(parent), m_dm(dm), m_utwId(utwId)
      {
      }

      ParamsResource::~ParamsResource()
      {
        this->beingDeleted();
      }

      void ParamsResource::handleRequest(const Wt::Http::Request& request,
          Wt::Http::Response& response)
      {
        Wt::Http::ResponseContinuation *continuation = request.continuation();

        bool isIE = false;
        bool isChrome = false;
        ostringstream oss, oss_response;
        double params[173];
        string paramsNames[173];
        bool paramsFetched = false;
        string param, str_response;
        string::size_type dotPos;

        try
        {
          Transaction tr = this->m_dm->createTransaction(amRead,
              ilReadCommitted, lrWait);
          tr->Start();
          ostringstream sql;
          string        str_output;
          sql << "select g.GAT_GATUNEK, ar.ART_NAZWA_ARTYSTY, "
                 " a.ALB_NAZWA_ALBUMU, u.UTW_TYTUL_UTWORU,"
                 " p.*, u.UTW_SCIEZKA_UTWORU from MIR_UTWOR u "
                 " inner join MIR_ALBUM a on a.ALB_ID = u.UTW_ALB_ID "
                 " inner join MIR_ARTYSTA ar on ar.ART_ID = u.UTW_ART_ID"
                 " inner join MIR_GATUNEK g on g.GAT_ID = u.UTW_GAT_ID"
                 " inner join MIR_PARAMETRY p on p.PAR_UTW_ID = u.UTW_ID"
                 " where u.UTW_ID = "
                 << this->m_utwId;
          string sqlstmt = sql.str();
          
          Statement stmt = this->m_dm->createStatement(&tr);
          stmt->Prepare(sqlstmt);
          stmt->Execute();

          if (!continuation)
          {
            isIE = request.userAgent().find("MSIE") != std::string::npos;
            isChrome = request.userAgent().find("Chrome")
              != std::string::npos;
          }
          if (stmt->Fetch())
          {
            if (!(isIE || isChrome))
            {
              string dbS;
              stmt->Get(1, dbS);
              oss << dbS << "_";
              stmt->Get(2, dbS);
              oss << dbS << "_";
              stmt->Get(3, dbS);
              oss << dbS << "_";
              stmt->Get(4, dbS);
              oss << dbS;
            }
            else
            {
              oss << "utwor" << this->m_utwId;
            }
//          stmt->Get(181, dbS);
//          boost::filesystem::path p(dbS);
//          oss << p.extension().string();
            m_suggestedFileName = oss.str();
            if (m_suggestedFileName.length() > 250)
              m_suggestedFileName = m_suggestedFileName.substr(0, 250);

            m_suggestedFileName += ".csv";

            UnicodeString icusuggestedFileName = UnicodeString::fromUTF8(m_suggestedFileName).findAndReplace("\"", "_");
            m_suggestedFileName.clear();
            icusuggestedFileName.toUTF8String(m_suggestedFileName);

            for (int i = 4 + 4; i < 4 + 4 + 173; i++)
            {
              stmt->Get(i, params[i - 8]);
              paramsNames[i - 8] = stmt->ColumnName(i);
            }
            paramsFetched = true;
          }

          tr->Commit();
        }
        catch(std::exception &e)
        {
          WApplication::instance()->log("error") << e.what();
        }

        if (paramsFetched)
        {
          // calculate the current start
          int iteration =
              continuation ? boost::any_cast<int>(continuation->data()) : 0;

          // if we have not yet finished, create a continuation to serve more
          if (/*!f.eof()*/false)
          {
            continuation = response.createContinuation();
            // remember what to do next
            continuation->setData(++iteration);
          }
  //        line.precision(18);
          oss_response << paramsNames[0].substr(4);
          for(int i = 1; i < 173; i++)
          {
            oss_response << ";" << paramsNames[i].substr(4);
          }
          oss_response << "\r\n";
          oss.setf(ios::fixed, ios::floatfield);
          oss.precision(18);
          oss.str("");
          oss << params[0];
          param = oss.str();
          dotPos = param.find(".");
          param.replace(dotPos, 1, ",");
          oss_response << param;
          for(int i = 1; i < 173; i++)
          {
            oss.str("");
            oss << params[i];
            param = oss.str();
            dotPos = param.find(".");
            param.replace(dotPos, 1, ",");
            oss_response << ";" << param;
          }

          str_response = oss_response.str();

          if (!continuation && !m_suggestedFileName.empty())
          {

      //// Browser incompatibility hell: internatianalized filename suggestions
      //// First filename is for browsers that don't support RFC 5987
      //// Second filename is for browsers that do support RFC 5987
      //std::string fileField;
      //// We cannot query wApp here, because wApp doesn't exist for
      //// static resources.
      //bool isIE = webRequest->userAgent().find("MSIE") != std::string::npos;
      //bool isChrome = webRequest->userAgent().find("Chrome") != std::string::npos;
      //if (isIE || isChrome) {
      //  // filename="foo-%c3%a4-%e2%82%ac.html"
      //  // Note: IE never converts %20 back to space, so avoid escaping
      //  // IE wil also not url decode the filename if the file has no ASCII
      //  // extension (e.g. .txt)
      //  fileField = "filename=\"" +
      //    Utils::urlEncode(suggestedFileName_.toUTF8(), " ") + "\";";
      //} else {
      //  // Binary UTF-8 sequence: for FF3, Safari, Chrome, Chrome9
      //  fileField = "filename=\"" + suggestedFileName_.toUTF8() + "\";";
      //}
      //// Next will be picked by RFC 5987 in favour of the
      //// one without specified encoding (Chrome9, 
      //fileField+= Utils::EncodeHttpHeaderField("filename", suggestedFileName_);
      //response.addHeader("Content-Disposition",
      //  theDisposition + ";" + fileField);

            response.setContentLength(str_response.length());
            std::string fileField;
            if (isIE || isChrome)
            {
              fileField = "filename=\""
                  + Wt::Utils::urlEncode(m_suggestedFileName, " ") + "\";";
            }
            else
            {
              fileField = "filename=\"" + m_suggestedFileName + "\";";
            }
            // Next will be picked by RFC 5987 in favour of the
            // one without specified encoding (Chrome9,
            fileField+= Wt::Utils::EncodeHttpHeaderField("filename", WString::fromUTF8(m_suggestedFileName));
            response.addHeader("Content-Disposition", "attachment;" + fileField);
          }

          if (iteration == 0)
          {
            response.setMimeType("text/csv");
          }

          response.out().write(str_response.c_str(), str_response.length());

        }
      }
    }
  }
}
