#include <Wt/WApplication>
#include <Wt/WLogger>

#include <unicode/unistr.h>

#include <iostream>

#include "utils/mir_charsvparser.h"

using namespace std;

namespace SyNaT
{
  namespace MIR
  {
  bool CHARSVParser::HasMoreTokens()
  {
      if ( this->m_string.length() > 0)
      {
          if ( this->m_pos >= this->m_string.length())
              return false;

          if (this->find_first_not_of(this->m_string, this->m_separator, this->m_pos) != -1)
              // there are non delimiter characters left, so we do have more tokens
              return true;

          if (this->m_string.char32At(this->m_pos) == UChar32('\n'))
              return false;
      }
      return this->m_pos == 0 && !this->m_string.isEmpty();
  }

  UnicodeString CHARSVParser::GetNextToken()
  {
      UnicodeString token;

      if ( !HasMoreTokens() )
          return token;

      // skip leading blanks if not quoted.
      while (this->m_pos < this->m_string.length() && this->m_string.char32At(this->m_pos) == UChar32(' '))
          this->m_pos ++;

      // Are we a quoted field?  Must handle this special.
      bool quoted_string = (this->m_string.char32At(this->m_pos) == this->m_quote);
      bool inquote = false;

      int32_t pos = this->m_pos;

      // find the end of this token.
      for (; pos < this->m_string.length(); pos++)
      {
          if (quoted_string && this->m_string.char32At(pos) == this->m_quote)
              inquote = !inquote;

          if (!inquote)
          {
              // Check to see if we have found the end of this token.
              // Tokens normally end with a m_separator delimiter.
              if (this->m_string.char32At(pos) == this->m_separator)
                  break;

              // Last token is delimited by '\n' or by end of string.
              if (this->m_string.char32At(pos) == UChar32('\n') && pos == this->m_string.length()-1)
                  break;
          }
      }

      if (quoted_string && !inquote)
      {
          token.setTo(this->m_string, this->m_pos + 1, pos - this->m_pos - 2);  // Remove leading and trailing quotes

          // Remove double quote chars, replace with single quote chars
          token.findAndReplace(UnicodeString(this->m_quote) + UnicodeString(this->m_quote), UnicodeString(this->m_quote));
      }
      else
          token.setTo(this->m_string, this->m_pos, pos - this->m_pos);

      string str;
      if (quoted_string && inquote)
        wApp->log("error") << "unterminated quoted string: " << token.toUTF8String(str);

      this->m_pos = pos + 1;    // Skip token and delimiter

      if (this->m_pos > this->m_string.length())  // Perhaps no delimiter if at end of string if orig string didn't have '\n'.
          this->m_pos = this->m_string.length();

      return token;
    }
  }
}
