#include <boost/regex.hpp>
#include <boost/ptr_container/ptr_map.hpp>
#include <boost/thread/mutex.hpp>
#include <boost/filesystem.hpp>

#include <stdio.h>
#include <string>
#include <ctype.h>

#include <Poco/Pipe.h>
#include <Poco/Process.h>
#include <Poco/PipeStream.h>

#include "utils/mir_uri.h"
#include "mir_exceptions.h"
#include "mir_sp_synatpgapp.h"
#include "mir_databasemodule.h"
#include "utils/mir_utils.h"

#include <unicode/unistr.h>
#include <unicode/utypes.h>
#include <unicode/regex.h>

namespace SyNaT
{
  namespace MIR
  {
    using namespace std;
    using namespace Poco;
    using namespace boost::filesystem;
          //this->m_wxregex_ahref.Compile(wxT("<a\\s(?!<a\\s)href\\s*=\\s*(\"[^\">]*(\"|>)?|'[^'>]*('|>)?|[^>'\" ]*)"), wxRE_ADVANCED | wxRE_ICASE); // more like Firefox works, 
                                                                                                                             //need to update match_base_href and match_a_href
          //this->m_wxregex_a_tag.Compile(wxT("<a\\s.*?href.*?>.*?</a>"), wxRE_ADVANCED | wxRE_ICASE);
          //this->m_wxregex_hrefReplacer.Compile(wxT("href\\s*=\\s*(\"[^\"]*\"?|'[^']*'?|[^>\\s]*)"), wxRE_ADVANCED | wxRE_ICASE);
          //this->m_wxregex_hrefReplacerWOEQ.Compile(wxT("(href\\s|href>)"), wxRE_ADVANCED | wxRE_ICASE);

          // 2 version
          //this->m_wxregex_atag.Compile(wxT("<a(\\s+[^>=\\s]+\\s*=\\s*\"[^\"]*\"|\\s+[^>=\\s]+\\s*=\\s*'[^']*'|\\s+[^>=\\s]+=?[^>\\s]*|[^>=\\s]+=?[^>\\s]*)+[^>]*?(>|</a>)"), wxRE_ADVANCED | wxRE_ICASE);
          //this->m_wxregex_basetag.Compile(wxT("<base(\\s+[^>=\\s]+\\s*=\\s*\"[^\"]*\"|\\s+[^>=\\s]+\\s*=\\s*'[^']*'|\\s+[^>=\\s]+=?[^>\\s]*|[^>=\\s]+=?[^>\\s]*)+[^<]*<"), wxRE_ADVANCED | wxRE_ICASE);

          // 1 version
          //this->m_wxregex_ahref.Compile(wxT("<a(\\s+.+?\\s*=\\s*\"[^\"]*\"?|\\s+.+?\\s*=\\s*'[^']*'?|\\s+[^=>\\s]+?\\s*=?\\s*[^>\\s]*)+?.*?</a>"), wxRE_ADVANCED | wxRE_ICASE);
          //this->m_wxregex_basehref.Compile(wxT("<base(\\s+.+?\\s*=\\s*\"[^\"]*\"?|\\s+.+?\\s*=\\s*'[^']*'?|\\s+[^=>\\s]+\\s*=?\\s*[^>\\s]*)+?[^<]*<"), wxRE_ADVANCED | wxRE_ICASE);

          // 0 version
          //this->m_wxregex_ahref.Compile(wxT("<a(\\s+[a-z]+\\s*=?\\s*\"[^\"]*\"?|\\s[a-z]+\\s*=?\\s*'[^']*'?|\\s[a-z]+\\s*=?\\s*[^>\\s]*)+[^>]*>"), wxRE_ADVANCED | wxRE_ICASE);
          //this->m_wxregex_basehref.Compile(wxT("<base(\\s+[a-z]+\\s*=?\\s*\"[^\"]*\"?|\\s[a-z]+\\s*=?\\s*'[^']*'?|\\s[a-z]+\\s*=?\\s*[^>\\s]*)+[^<]*<"), wxRE_ADVANCED | wxRE_ICASE);

          //this->m_wxregex_sessionIDremoval.Compile(wxT("(&|\\?)?")+std2wx(this->m_sessionID, &wxConvUTF8)+wxT("=[^&]*?"), wxRE_ADVANCED | wxRE_ICASE);
          //this->m_wxregex_attributeWOQ.Compile(wxT("\\s[-.:\\xB7_0-9a-z\\xC0-\\xD6\\xD8-\\xF6\\xF8-\\x2FF\\x0300-\\x036F\\x370-\\x37D\\x37F-\\x1FFF\\x200C-\\x200D\\x203F-\\x2040\\x2070-\\x218F\\x2C00-\\x2FEF\\x2C00-\\x2FEF\\x3001-\\xD7FF\\xF900-\\xFDCF\\xFDF0-\\xFFFD\\x10000-\\xEFFFF]+\\s*=?\\s*[^>\\s]+"), wxRE_ADVANCED | wxRE_ICASE);
          //this->m_wxregex_attributeWSQ.Compile(wxT("\\s[-.:\\xB7_0-9a-z\\xC0-\\xD6\\xD8-\\xF6\\xF8-\\x2FF\\x0300-\\x036F\\x370-\\x37D\\x37F-\\x1FFF\\x200C-\\x200D\\x203F-\\x2040\\x2070-\\x218F\\x2C00-\\x2FEF\\x2C00-\\x2FEF\\x3001-\\xD7FF\\xF900-\\xFDCF\\xFDF0-\\xFFFD\\x10000-\\xEFFFF]+\\s*=\\s*'[^']*'?"), wxRE_ADVANCED | wxRE_ICASE);
          //this->m_wxregex_attributeWDQ.Compile(wxT("\\s[-.:\\xB7_0-9a-z\\xC0-\\xD6\\xD8-\\xF6\\xF8-\\x2FF\\x0300-\\x036F\\x370-\\x37D\\x37F-\\x1FFF\\x200C-\\x200D\\x203F-\\x2040\\x2070-\\x218F\\x2C00-\\x2FEF\\x2C00-\\x2FEF\\x3001-\\xD7FF\\xF900-\\xFDCF\\xFDF0-\\xFFFD\\x10000-\\xEFFFF]+\\s*=\\s*\"[^\"]*\"?"), wxRE_ADVANCED | wxRE_ICASE);
          //this->m_wxregex_attributeWOQ.Compile(wxT("\\s[a-z]+(\\s*=\\s*[^>\\s]*|[^>\\s]+>?|\\s*)"), wxRE_ADVANCED | wxRE_ICASE);   // interesuj� mnie tylko atrybuty z zakresu a-zA-Z
          //this->m_wxregex_attributeWSQ.Compile(wxT("\\s[a-z]+\\s*=\\s*'[^']*'?"), wxRE_ADVANCED | wxRE_ICASE);    // interesuj� mnie tylko atrybuty z zakresu a-zA-Z
          //this->m_wxregex_attributeWDQ.Compile(wxT("\\s[a-z]+\\s*=\\s*\"[^\"]*\"?"), wxRE_ADVANCED | wxRE_ICASE); // interesuj� mnie tylko atrybuty z zakresu a-zA-Z
          //this->m_wxregex_spaces.Compile(wxT("\\s+"), wxRE_ADVANCED | wxRE_ICASE);
    Utils::Utils(const string &sessionID) : m_sessionID(sessionID)
    {
      UErrorCode status_regexpatt = U_ZERO_ERROR;
      this->m_regexpatt_imgtag = RegexPattern::compile("<img(\\s+[^>=\\s]+\\s*=\\s*\"[^\"]*\"|\\s+[^>=\\s]+\\s*=\\s*\\'[^\\']*\\'|\\s+[^>=\\s]+=?[^>\\s]*|[^>=\\s]+=?[^>\\s]*)+[^>]*>", UREGEX_CASE_INSENSITIVE, status_regexpatt);
      status_regexpatt = U_ZERO_ERROR;
      this->m_regexpatt_linktag = RegexPattern::compile("<link(\\s+[^>=\\s]+\\s*=\\s*\"[^\"]*\"|\\s+[^>=\\s]+\\s*=\\s*\\'[^\\']*\\'|\\s+[^>=\\s]+=?[^>\\s]*|[^>=\\s]+=?[^>\\s]*)+[^>]*>", UREGEX_CASE_INSENSITIVE, status_regexpatt);
      status_regexpatt = U_ZERO_ERROR;
      this->m_regexpatt_atag = RegexPattern::compile("<a(\\s+[^>=\\s]+\\s*=\\s*\"[^\"]*\"|\\s+[^>=\\s]+\\s*=\\s*\\'[^\\']*\\'|\\s+[^>=\\s]+=?[^>\\s]*|[^>=\\s]+=?[^>\\s]*)+[^>]*>", UREGEX_CASE_INSENSITIVE, status_regexpatt);
      status_regexpatt = U_ZERO_ERROR;
      this->m_regexpatt_basetag = RegexPattern::compile("<base(\\s+[^>=\\s]+\\s*=\\s*\"[^\"]*\"|\\s+[^>=\\s]+\\s*=\\s*\\'[^\\']*\\'|\\s+[^>=\\s]+=?[^>\\s]*|[^>=\\s]+=?[^>\\s]*)+[^<]*<", UREGEX_CASE_INSENSITIVE, status_regexpatt);
      status_regexpatt = U_ZERO_ERROR;
      this->m_regexpatt_attributeWOQ = RegexPattern::compile("\\s[a-z]+(\\s*=\\s*[^>\\s]*|[^>\\s]+>?|\\s*)", UREGEX_CASE_INSENSITIVE, status_regexpatt);
      status_regexpatt = U_ZERO_ERROR;
      this->m_regexpatt_attributeWSQ = RegexPattern::compile("\\s[a-z]+\\s*=\\s*\\'[^\\']*\\'?", UREGEX_CASE_INSENSITIVE, status_regexpatt);
      status_regexpatt = U_ZERO_ERROR;
      this->m_regexpatt_attributeWDQ = RegexPattern::compile("\\s[a-z]+\\s*=\\s*\"[^\"]*\"?", UREGEX_CASE_INSENSITIVE, status_regexpatt);
      status_regexpatt = U_ZERO_ERROR;
      this->m_regexpatt_spaces = RegexPattern::compile("\\s+", UREGEX_CASE_INSENSITIVE, status_regexpatt);
      status_regexpatt = U_ZERO_ERROR;
      this->m_regexpatt_srcReplacer = RegexPattern::compile("src\\s*=\\s*(\"[^\"]*\"?|\\'[^\\']*\\'?|[^>\\s]*)", UREGEX_CASE_INSENSITIVE, status_regexpatt);
      status_regexpatt = U_ZERO_ERROR;
      this->m_regexpatt_srcReplacerWOEQ = RegexPattern::compile("(src\\s|src>)", UREGEX_CASE_INSENSITIVE, status_regexpatt);
      status_regexpatt = U_ZERO_ERROR;
      this->m_regexpatt_hrefReplacer = RegexPattern::compile("href\\s*=\\s*(\"[^\"]*\"?|\\'[^\\']*\\'?|[^>\\s]*)", UREGEX_CASE_INSENSITIVE, status_regexpatt);
      status_regexpatt = U_ZERO_ERROR;
      this->m_regexpatt_hrefReplacerWOEQ = RegexPattern::compile("(href\\s|href>)", UREGEX_CASE_INSENSITIVE, status_regexpatt);
    }

    Utils::~Utils()
    {
      delete this->m_regexpatt_imgtag;
      delete this->m_regexpatt_linktag;
      delete this->m_regexpatt_atag;
      delete this->m_regexpatt_basetag;
      delete this->m_regexpatt_attributeWOQ;
      delete this->m_regexpatt_attributeWSQ;
      delete this->m_regexpatt_attributeWDQ;
      delete this->m_regexpatt_spaces;
      delete this->m_regexpatt_srcReplacer;
      delete this->m_regexpatt_srcReplacerWOEQ;
      delete this->m_regexpatt_hrefReplacer;
      delete this->m_regexpatt_hrefReplacerWOEQ;
    }

    IcuURI Utils::match_img_tag(const UnicodeString &icu_part_of_page, const IcuURI &baseURI, bool &hasImgSrc, bool &hasImgTag, int32_t &start, int32_t &len, int32_t &startOfTag, int32_t &lenOfTag, KindOfAttrValue &koav, const string &sessionID, string &sessionIDVal)
    {
      UnicodeString atag, match, struri, attribWDQ, attribWSQ, attribWOQ, attribWOQ1;
      bool singleQuote, doubleQuote;
      UErrorCode status_regex_imgtag = U_ZERO_ERROR;
      RegexMatcher *regex_imgtag = this->m_regexpatt_imgtag->matcher(status_regex_imgtag);
          //new RegexMatcher("<img(\\s+[^>=\\s]+\\s*=\\s*\"[^\"]*\"|\\s+[^>=\\s]+\\s*=\\s*\\'[^\\']*\\'|\\s+[^>=\\s]+=?[^>\\s]*|[^>=\\s]+=?[^>\\s]*)+[^>]*>", UREGEX_CASE_INSENSITIVE, status_regex_imgtag);


      hasImgSrc = false;
      hasImgTag = false;
      int32_t attribStart, attribLen, offset, startWDQ, lenWDQ, startWSQ, lenWSQ, startWOQ, lenWOQ;

      bool hasWSQ = true,
           hasWDQ = true,
           hasWOQ = true;

      start = 0;
      len = 0;
      startOfTag = 0;
      lenOfTag = 0;
      UErrorCode status = U_ZERO_ERROR;
      /*while*/if(regex_imgtag->reset(icu_part_of_page).find()
        && (hasWDQ || hasWSQ || hasWOQ) 
        && (!hasImgSrc))
      {
          status = U_ZERO_ERROR;
          start = regex_imgtag->start(status);
          status = U_ZERO_ERROR;
          len = regex_imgtag->end(status) - start;

          delete regex_imgtag;
          UErrorCode status_regex = U_ZERO_ERROR;
          RegexMatcher *regex_attributeWOQ = this->m_regexpatt_attributeWOQ->matcher(status_regex);
              //new RegexMatcher("\\s[a-z]+(\\s*=\\s*[^>\\s]*|[^>\\s]+>?|\\s*)", UREGEX_CASE_INSENSITIVE, status_regex_attributeWOQ);
          status_regex = U_ZERO_ERROR;
          RegexMatcher *regex_attributeWSQ = this->m_regexpatt_attributeWSQ->matcher(status_regex);
              //new RegexMatcher("\\s[a-z]+\\s*=\\s*\\'[^\\']*\\'?", UREGEX_CASE_INSENSITIVE, status_regex_attributeWSQ);
          status_regex = U_ZERO_ERROR;
          RegexMatcher *regex_attributeWDQ = this->m_regexpatt_attributeWDQ->matcher(status_regex);
              //new RegexMatcher("\\s[a-z]+\\s*=\\s*\"[^\"]*\"?", UREGEX_CASE_INSENSITIVE, status_regex_attributeWDQ);
          status_regex = U_ZERO_ERROR;
          RegexMatcher *regex_spaces = this->m_regexpatt_spaces->matcher(status_regex);
              //new RegexMatcher("\\s+", UREGEX_CASE_INSENSITIVE, status_regex_spaces);
//          status_regex = U_ZERO_ERROR;
//          RegexMatcher *regex_hrefReplacer = this->m_regexpatt_hrefReplacer->matcher(status_regex);
//            //new RegexMatcher("href\\s*=\\s*(\"[^\"]*\"?|\\'[^\\']*\\'?|[^>\\s]*)", UREGEX_CASE_INSENSITIVE, this->m_status_regex_hrefReplacer);
//          status_regex = U_ZERO_ERROR;
//          RegexMatcher *regex_hrefReplacerWOEQ = this->m_regexpatt_hrefReplacerWOEQ->matcher(status_regex);
//            //new RegexMatcher("(href\\s|href>)", UREGEX_CASE_INSENSITIVE, this->m_status_regex_hrefReplacerWOEQ);
          status_regex = U_ZERO_ERROR;
          RegexMatcher *regex_srcReplacer = this->m_regexpatt_srcReplacer->matcher(status_regex);
              //new RegexMatcher("src\\s*=\\s*(\"[^\"]*\"?|\\'[^\\']*\\'?|[^>\\s]*)", UREGEX_CASE_INSENSITIVE, status_regex);
          status_regex = U_ZERO_ERROR;
          RegexMatcher *regex_srcReplacerWOEQ = this->m_regexpatt_srcReplacerWOEQ->matcher(status_regex);
              //new RegexMatcher("(src\\s|src>)", UREGEX_CASE_INSENSITIVE, status_regex);

          startOfTag = start;
          lenOfTag = len;
          start = start + 2;
          len = len - 2;
          offset = start;
          hasImgTag = true;

          while ((hasWDQ || hasWSQ || hasWOQ) && (!hasImgSrc))
          {
            startWDQ = 0;
            lenWDQ = 0;
            startWSQ = 0;
            lenWSQ = 0;
            startWOQ = 0;
            lenWOQ = 0;

            hasWDQ = false;
            hasWSQ = false;
            hasWOQ = false;

            atag = UnicodeString(icu_part_of_page, offset, len);
            if(regex_attributeWDQ->reset(atag).find())
            {
              status = U_ZERO_ERROR;
              attribStart = regex_attributeWDQ->start(status);
              status = U_ZERO_ERROR;
              attribLen = regex_attributeWDQ->end(status) - attribStart;

              hasWDQ = true;
              attribWDQ = UnicodeString(atag, attribStart + 1, attribLen - 1);
              startWDQ = offset + attribStart;
              lenWDQ = attribLen;
            }

            if(regex_attributeWSQ->reset(atag).find())
            {
              status = U_ZERO_ERROR;
              attribStart = regex_attributeWSQ->start(status);
              status = U_ZERO_ERROR;
              attribLen = regex_attributeWSQ->end(status) - attribStart;

              hasWSQ = true;
              attribWSQ = UnicodeString(atag, attribStart + 1, attribLen - 1);
              startWSQ = offset + attribStart;
              lenWSQ = attribLen;
            }

            if(regex_attributeWOQ->reset(atag).find())
            {
              status = U_ZERO_ERROR;
              attribStart = regex_attributeWOQ->start(status);
              status = U_ZERO_ERROR;
              attribLen = regex_attributeWOQ->end(status) - attribStart;

              hasWOQ = true;
              attribWOQ = UnicodeString(atag, attribStart + 1, attribLen - 1);
              startWOQ = offset + attribStart;
              lenWOQ = attribLen;
//              }
//              if (regex_attributeWOQ->GetMatch(&attribStart, &attribLen, 1))
//              {
              status = U_ZERO_ERROR;
              attribStart = regex_attributeWOQ->start(1, status);
              status = U_ZERO_ERROR;
              attribLen = regex_attributeWOQ->end(1, status) - attribStart;

              attribWOQ1 = UnicodeString(atag, attribStart, attribLen);
            }
            


            if (   ((hasWDQ && (!hasWSQ) && (!hasWOQ)))
                || ((hasWDQ && hasWSQ && (!hasWOQ)) && (startWDQ <= startWSQ))
                || ((hasWDQ && (!hasWSQ) && hasWOQ) && (startWDQ <= startWOQ))
                || ((hasWDQ && hasWSQ && hasWOQ && (startWDQ <= startWOQ) && (startWDQ <= startWSQ))))
            {
              if (UnicodeString(attribWDQ).toUpper().startsWith("SRC"))
              {
                if(regex_srcReplacer->reset(attribWDQ).find())
                {
                  status = U_ZERO_ERROR;
                  attribStart = regex_srcReplacer->start(1, status);
                  status = U_ZERO_ERROR;
                  attribLen = regex_srcReplacer->end(1, status) - attribStart;

                  match = UnicodeString(attribWDQ, attribStart, attribLen);

                  // usuwam poczatkowy znak "
                  doubleQuote = (match.indexOf("\"") == 0);
                  if (doubleQuote)
                    match = UnicodeString(match, 1);

                  if (doubleQuote)
                  {
                    if (match.endsWith("\""))
                      match.truncate(match.length() - 1);
                  }
                  hasImgSrc = true;
                  start = startWDQ;
                  len   = lenWDQ;
                  koav  = koavWDQimg;
                  break;
                }
              }
              offset = startWDQ + lenWDQ;
              len = lenOfTag + startOfTag - offset;
            }
            else if (   ((hasWSQ && (!hasWDQ) && (!hasWOQ)))
                     || ((hasWSQ && hasWDQ && (!hasWOQ)) && (startWSQ <= startWDQ))
                     || ((hasWSQ && (!hasWDQ) && hasWOQ) && (startWSQ <= startWOQ))
                     || ((hasWSQ && hasWDQ && hasWOQ && (startWSQ <= startWOQ) && (startWSQ <= startWDQ))))
            {
              if (UnicodeString(attribWSQ).toUpper().startsWith("SRC"))
              {
                if(regex_srcReplacer->reset(attribWSQ).find())
                {
                  status = U_ZERO_ERROR;
                  attribStart = regex_srcReplacer->start(1, status);
                  status = U_ZERO_ERROR;
                  attribLen = regex_srcReplacer->end(1, status) - attribStart;

                  match = UnicodeString(attribWSQ, attribStart, attribLen);

                  // usuwam poczatkowy znak ' lub "
                  singleQuote = (match.indexOf("'") == 0);
                  if (singleQuote)
                    match = UnicodeString(match, 1);

                  // usuwam koncowy znak ' lub "
                  if (singleQuote)
                  {
                    if (match.endsWith("'"))
                      match.truncate(match.length() - 1);
                  }
                  hasImgSrc = true;
                  start = startWSQ;
                  len   = lenWSQ;
                  koav  = koavWSQimg;

                  break;
                }
              }
              offset = startWSQ + lenWSQ;
              len = lenOfTag + startOfTag - offset;
            }
            else if (   ((hasWOQ && (!hasWDQ) && (!hasWSQ)))
                     || ((hasWOQ && hasWDQ && (!hasWSQ)) && (startWOQ <= startWDQ))
                     || ((hasWOQ && (!hasWDQ) && hasWSQ) && (startWOQ <= startWSQ))
                     || ((hasWOQ && hasWDQ && hasWSQ && (startWOQ <= startWSQ) && (startWOQ <= startWDQ))))
            {
              if (UnicodeString(attribWOQ).toUpper().startsWith("SRC"))
              {
                if(regex_srcReplacer->reset(attribWOQ).find())
                {
                  status = U_ZERO_ERROR;
                  attribStart = regex_srcReplacer->start(1, status);
                  status = U_ZERO_ERROR;
                  attribLen = regex_srcReplacer->end(1, status) - attribStart;

                  match = UnicodeString(attribWOQ, attribStart, attribLen);

                  hasImgSrc = true;
                  start = startWOQ;
                  len   = lenWOQ;
                  koav  = koavWOQimg;

                  if (attribWOQ1.endsWith(">"))
                    len   = lenWOQ - 1;

                  break;
                }
                else if (regex_srcReplacerWOEQ->reset(attribWOQ).find())
                {

                  match = baseURI.BuildURI();
                  hasImgSrc = true;
                  start = startWOQ;
                  koav  = koavWOQimg;

                  if (attribWOQ1.endsWith(">"))
                    len   = lenWOQ - 1;
                  else if (regex_spaces->reset(attribWOQ1).find()) // test dla przypadku gdy attribWOQ1 zawiera same znaki \\s
                  {
                    status = U_ZERO_ERROR;
                    attribStart = regex_spaces->start(0, status);
                    status = U_ZERO_ERROR;
                    attribLen = regex_spaces->end(0, status) - attribStart;

                    if ((attribStart == 0) && (attribLen == attribWOQ1.length()))
                    {
                      len = lenWOQ - attribLen;
                    }
                  }
                  else
                    len = lenWOQ;

                  break;
                }

              }
              offset = startWOQ + lenWOQ;
              len = lenOfTag + startOfTag - offset;
            }

          }
          delete regex_attributeWOQ;
          delete regex_attributeWSQ;
          delete regex_attributeWDQ;
          delete regex_spaces;
//          RegexMatcher *regex_hrefReplacer = new RegexMatcher("href\\s*=\\s*(\"[^\"]*\"?|\\'[^\\']*\\'?|[^>\\s]*)", UREGEX_CASE_INSENSITIVE, this->m_status_regex_hrefReplacer);
//          RegexMatcher *regex_hrefReplacerWOEQ = new RegexMatcher("(href\\s|href>)", UREGEX_CASE_INSENSITIVE, this->m_status_regex_hrefReplacerWOEQ);
          delete regex_srcReplacer;
          delete regex_srcReplacerWOEQ;

      }

      IcuURI uri(match);
      uri.Resolve(baseURI, IcuURI::IcuURI_STRICT);
      if (!sessionID.empty())
      {
        struri = uri.GetScheme() + "://";
        if (uri.HasUserInfo())
        {
          struri += uri.GetUser();
          if (!uri.GetPassword().isEmpty())
            struri += ":" + uri.GetPassword();
          struri += "@";
        }
        struri += uri.GetServer();
        if (uri.HasPort())
        {
          struri += ":" + uri.GetPort();
        }

        struri += uri.GetPath();

        match = UnicodeString(uri.GetQuery());

        UErrorCode status_regex_sessionIDremoval = U_ZERO_ERROR;
        RegexMatcher *regex_sessionIDremoval = new RegexMatcher(("(&|\\?)?"+this->m_sessionID+"=([^&]*)").c_str(), UREGEX_CASE_INSENSITIVE, status_regex_sessionIDremoval);

        if(regex_sessionIDremoval->reset(match).find())
        {
          status = U_ZERO_ERROR;
          start = regex_sessionIDremoval->start(2, status);
          status = U_ZERO_ERROR;
          len = regex_sessionIDremoval->end(2, status) - start;

          UnicodeString(match, start, len).toUTF8String(sessionIDVal);

          status = U_ZERO_ERROR;
          match = regex_sessionIDremoval->replaceAll("", status);
        }

        delete regex_sessionIDremoval;

        if (!match.isEmpty())
        {
          if (match.indexOf("&") == 0)
            match = UnicodeString(match, 1);
          match = "?" + match;
        }


        if (uri.HasFragment())
          struri += match + "#" + uri.GetFragment();
        else
          struri += match;
        uri.Create(struri);
      }
      return uri;      
    }

    IcuURI Utils::match_link_tag(const UnicodeString &icu_part_of_page, const IcuURI &baseURI, bool &hasLinkHref, bool &hasLinkTag, int32_t &start, int32_t &len, int32_t &startOfTag, int32_t &lenOfTag, KindOfAttrValue &koav, const string &sessionID, string &sessionIDVal)
    {
      UnicodeString atag, match, struri, attribWDQ, attribWSQ, attribWOQ, attribWOQ1;
      bool singleQuote, doubleQuote;

      UErrorCode status_regex_linktag = U_ZERO_ERROR;
      RegexMatcher *regex_linktag = this->m_regexpatt_linktag->matcher(status_regex_linktag);

      hasLinkHref = false;
      hasLinkTag = false;
      int32_t attribStart, attribLen, offset, startWDQ, lenWDQ, startWSQ, lenWSQ, startWOQ, lenWOQ;

      bool hasWSQ = true,
           hasWDQ = true,
           hasWOQ = true;

      start = 0;
      len = 0;
      startOfTag = 0;
      lenOfTag = 0;

      UErrorCode status = U_ZERO_ERROR;

      /*while*/if(regex_linktag->reset(icu_part_of_page).find()
        && (hasWDQ || hasWSQ || hasWOQ) 
        && (!hasLinkHref))
      {
        status = U_ZERO_ERROR;
        start = regex_linktag->start(status);
        status = U_ZERO_ERROR;
        len = regex_linktag->end(status) - start;

        delete regex_linktag;
        UErrorCode status_regex = U_ZERO_ERROR;
        RegexMatcher *regex_attributeWOQ = this->m_regexpatt_attributeWOQ->matcher(status_regex);
            //new RegexMatcher("\\s[a-z]+(\\s*=\\s*[^>\\s]*|[^>\\s]+>?|\\s*)", UREGEX_CASE_INSENSITIVE, status_regex_attributeWOQ);
        status_regex = U_ZERO_ERROR;
        RegexMatcher *regex_attributeWSQ = this->m_regexpatt_attributeWSQ->matcher(status_regex);
            //new RegexMatcher("\\s[a-z]+\\s*=\\s*\\'[^\\']*\\'?", UREGEX_CASE_INSENSITIVE, status_regex_attributeWSQ);
        status_regex = U_ZERO_ERROR;
        RegexMatcher *regex_attributeWDQ = this->m_regexpatt_attributeWDQ->matcher(status_regex);
            //new RegexMatcher("\\s[a-z]+\\s*=\\s*\"[^\"]*\"?", UREGEX_CASE_INSENSITIVE, status_regex_attributeWDQ);
        status_regex = U_ZERO_ERROR;
        RegexMatcher *regex_spaces = this->m_regexpatt_spaces->matcher(status_regex);
            //new RegexMatcher("\\s+", UREGEX_CASE_INSENSITIVE, status_regex_spaces);
        status_regex = U_ZERO_ERROR;
        RegexMatcher *regex_hrefReplacer = this->m_regexpatt_hrefReplacer->matcher(status_regex);
          //new RegexMatcher("href\\s*=\\s*(\"[^\"]*\"?|\\'[^\\']*\\'?|[^>\\s]*)", UREGEX_CASE_INSENSITIVE, this->m_status_regex_hrefReplacer);
        status_regex = U_ZERO_ERROR;
        RegexMatcher *regex_hrefReplacerWOEQ = this->m_regexpatt_hrefReplacerWOEQ->matcher(status_regex);
          //new RegexMatcher("(href\\s|href>)", UREGEX_CASE_INSENSITIVE, this->m_status_regex_hrefReplacerWOEQ);
//        status_regex = U_ZERO_ERROR;
//        RegexMatcher *regex_srcReplacer = this->m_regexpatt_srcReplacer->matcher(status_regex);
//            //new RegexMatcher("src\\s*=\\s*(\"[^\"]*\"?|\\'[^\\']*\\'?|[^>\\s]*)", UREGEX_CASE_INSENSITIVE, status_regex);
//        status_regex = U_ZERO_ERROR;
//        RegexMatcher *regex_srcReplacerWOEQ = this->m_regexpatt_srcReplacerWOEQ->matcher(status_regex);
//            //new RegexMatcher("(src\\s|src>)", UREGEX_CASE_INSENSITIVE, status_regex);


        startOfTag = start;
        lenOfTag = len;
        start = start + 2;
        len = len - 2;
        offset = start;
        hasLinkTag = true;

        while ((hasWDQ || hasWSQ || hasWOQ) && (!hasLinkHref))
        {
          startWDQ = 0;
          lenWDQ = 0;
          startWSQ = 0;
          lenWSQ = 0;
          startWOQ = 0;
          lenWOQ = 0;

          hasWDQ = false;
          hasWSQ = false;
          hasWOQ = false;

          atag = UnicodeString(icu_part_of_page, offset, len);
          if (regex_attributeWDQ->reset(atag).find())
          {
            status = U_ZERO_ERROR;
            attribStart = regex_attributeWDQ->start(status);
            status = U_ZERO_ERROR;
            attribLen = regex_attributeWDQ->end(status) - attribStart;

            hasWDQ = true;
            attribWDQ = UnicodeString(atag, attribStart + 1, attribLen - 1);
            startWDQ = offset + attribStart;
            lenWDQ = attribLen;
          }

          if (regex_attributeWSQ->reset(atag).find())
          {
            status = U_ZERO_ERROR;
            attribStart = regex_attributeWSQ->start(status);
            status = U_ZERO_ERROR;
            attribLen = regex_attributeWSQ->end(status) - attribStart;

            hasWSQ = true;
            attribWSQ = UnicodeString(atag, attribStart + 1, attribLen - 1);
            startWSQ = offset + attribStart;
            lenWSQ = attribLen;
          }

          if (regex_attributeWOQ->reset(atag).find())
          {
              status = U_ZERO_ERROR;
              attribStart = regex_attributeWOQ->start(status);
              status = U_ZERO_ERROR;
              attribLen = regex_attributeWOQ->end(status) - attribStart;

              hasWOQ = true;
              attribWOQ = UnicodeString(atag, attribStart + 1, attribLen - 1);
              startWOQ = offset + attribStart;
              lenWOQ = attribLen;

              status = U_ZERO_ERROR;
              attribStart = regex_attributeWOQ->start(1, status);
              status = U_ZERO_ERROR;
              attribLen = regex_attributeWOQ->end(1, status) - attribStart;

              attribWOQ1 = UnicodeString(atag, attribStart, attribLen);
          }



          if (   ((hasWDQ && (!hasWSQ) && (!hasWOQ)))
              || ((hasWDQ && hasWSQ && (!hasWOQ)) && (startWDQ <= startWSQ))
              || ((hasWDQ && (!hasWSQ) && hasWOQ) && (startWDQ <= startWOQ))
              || ((hasWDQ && hasWSQ && hasWOQ && (startWDQ <= startWOQ) && (startWDQ <= startWSQ))))
          {
            if (UnicodeString(attribWDQ).toUpper().startsWith("HREF"))
            {
              if (regex_hrefReplacer->reset(attribWDQ).find())
              {
                status = U_ZERO_ERROR;
                attribStart = regex_hrefReplacer->start(1, status);
                status = U_ZERO_ERROR;
                attribLen = regex_hrefReplacer->end(1, status) - attribStart;

                match = UnicodeString(attribWDQ, attribStart, attribLen);

                // usuwam poczatkowy znak "
                doubleQuote = (match.indexOf("\"") == 0);
                if (doubleQuote)
                  match = UnicodeString(match, 1);

                if (doubleQuote)
                {
                  if (match.endsWith("\""))
                    match.truncate(match.length() - 1);
                }
                hasLinkHref = true;
                start = startWDQ;
                len   = lenWDQ;
                koav  = koavWDQ;
                break;
              }
            }
            offset = startWDQ + lenWDQ;
            len = lenOfTag + startOfTag - offset;
          }
          else if (   ((hasWSQ && (!hasWDQ) && (!hasWOQ)))
                   || ((hasWSQ && hasWDQ && (!hasWOQ)) && (startWSQ <= startWDQ))
                   || ((hasWSQ && (!hasWDQ) && hasWOQ) && (startWSQ <= startWOQ))
                   || ((hasWSQ && hasWDQ && hasWOQ && (startWSQ <= startWOQ) && (startWSQ <= startWDQ))))
          {
            if (UnicodeString(attribWSQ).toUpper().startsWith("HREF"))
            {
              if (regex_hrefReplacer->reset(attribWSQ).find())
              {
                status = U_ZERO_ERROR;
                attribStart = regex_hrefReplacer->start(1, status);
                status = U_ZERO_ERROR;
                attribLen = regex_hrefReplacer->end(1, status) - attribStart;

                match = UnicodeString(attribWSQ, attribStart, attribLen);

                // usuwam poczatkowy znak ' lub "
                singleQuote = (match.indexOf("'") == 0);
                if (singleQuote)
                  match = UnicodeString(match, 1);

                // usuwam koncowy znak ' lub "
                if (singleQuote)
                {
                  if (match.endsWith("'"))
                    match.truncate(match.length() - 1);
                }
                hasLinkHref = true;
                start = startWSQ;
                len   = lenWSQ;
                koav  = koavWSQ;

                break;
              }
            }
            offset = startWSQ + lenWSQ;
            len = lenOfTag + startOfTag - offset;
          }
          else if (   ((hasWOQ && (!hasWDQ) && (!hasWSQ)))
                   || ((hasWOQ && hasWDQ && (!hasWSQ)) && (startWOQ <= startWDQ))
                   || ((hasWOQ && (!hasWDQ) && hasWSQ) && (startWOQ <= startWSQ))
                   || ((hasWOQ && hasWDQ && hasWSQ && (startWOQ <= startWSQ) && (startWOQ <= startWDQ))))
          {
            if (UnicodeString(attribWOQ).toUpper().startsWith("HREF"))
            {
              if (regex_hrefReplacer->reset(attribWOQ).find())
              {
                status = U_ZERO_ERROR;
                attribStart = regex_hrefReplacer->start(1, status);
                status = U_ZERO_ERROR;
                attribLen = regex_hrefReplacer->end(1, status) - attribStart;

                match = UnicodeString(attribWOQ, attribStart, attribLen);

                hasLinkHref = true;
                start = startWOQ;
                len   = lenWOQ;
                koav  = koavWOQ;

                if (attribWOQ1.endsWith(">"))
                  len   = lenWOQ - 1;

                break;
              }
              else if (regex_hrefReplacerWOEQ->reset(attribWOQ).find())
              {

                match = baseURI.BuildURI();
                hasLinkHref = true;
                start = startWOQ;
                koav  = koavWOQ;

                if (attribWOQ1.endsWith(">"))
                  len   = lenWOQ - 1;
                else if (regex_spaces->reset(attribWOQ1).find()) // test dla przypadku gdy attribWOQ1 zawiera same znaki \\s
                {
                  status = U_ZERO_ERROR;
                  attribStart = regex_spaces->start(status);
                  status = U_ZERO_ERROR;
                  attribLen = regex_spaces->end(status) - attribStart;
                  if ((attribStart == 0) && (attribLen == attribWOQ1.length()))
                  {
                    len = lenWOQ - attribLen;
                  }
                }
                else
                  len = lenWOQ;

                break;
              }

            }
            offset = startWOQ + lenWOQ;
            len = lenOfTag + startOfTag - offset;
          }

        }

        delete regex_attributeWOQ;
        delete regex_attributeWSQ;
        delete regex_attributeWDQ;
        delete regex_spaces;
        delete regex_hrefReplacer;
        delete regex_hrefReplacerWOEQ;
//        delete regex_srcReplacer;
//        delete regex_srcReplacerWOEQ;

      }

      IcuURI uri(match);
      uri.Resolve(baseURI, IcuURI::IcuURI_STRICT);
      if (!sessionID.empty())
      {
        struri = uri.GetScheme() + "://";
        if (uri.HasUserInfo())
        {
          struri += uri.GetUser();
          if (!uri.GetPassword().isEmpty())
            struri += ":" + uri.GetPassword();
          struri += "@";
        }
        struri += uri.GetServer();
        if (uri.HasPort())
        {
          struri += ":" + uri.GetPort();
        }

        struri += uri.GetPath();

        match = uri.GetQuery();

        UErrorCode status_regex_sessionIDremoval = U_ZERO_ERROR;
        RegexMatcher *regex_sessionIDremoval = new RegexMatcher(("(&|\\?)?"+this->m_sessionID+"=([^&]*)").c_str(), UREGEX_CASE_INSENSITIVE, status_regex_sessionIDremoval);

        if(regex_sessionIDremoval->reset(match).find())
        {
          status = U_ZERO_ERROR;
          start = regex_sessionIDremoval->start(2, status);
          status = U_ZERO_ERROR;
          len = regex_sessionIDremoval->end(2, status) - start;

          UnicodeString(match, start, len).toUTF8String(sessionIDVal);

          status = U_ZERO_ERROR;
          match = regex_sessionIDremoval->replaceAll("", status);
        }
        delete regex_sessionIDremoval;
        if (!match.isEmpty())
        {
          if (match.indexOf("&") == 0)
            match = UnicodeString(match, 1);
          match = "?" + match;
        }


        if (uri.HasFragment())
          struri += match + "#" + uri.GetFragment();
        else
          struri += match;
        uri.Create(struri);
      }
      return uri;
    }

    IcuURI Utils::match_base_tag(const UnicodeString &icu_part_of_page, bool &hasBaseHref, bool &hasBaseTag, int32_t &start, int32_t &len, int32_t &startOfTag, int32_t &lenOfTag, IcuURI *icuuri)
    {

      UnicodeString atag, match, struri, attribWDQ, attribWSQ, attribWOQ, attribWOQ1;
      bool singleQuote, doubleQuote;

      UErrorCode status_regex_basetag = U_ZERO_ERROR;
      RegexMatcher *regex_basetag = this->m_regexpatt_basetag->matcher(status_regex_basetag);
          //new RegexMatcher("<base(\\s+[^>=\\s]+\\s*=\\s*\"[^\"]*\"|\\s+[^>=\\s]+\\s*=\\s*\\'[^\\']*\\'|\\s+[^>=\\s]+=?[^>\\s]*|[^>=\\s]+=?[^>\\s]*)+[^<]*<", UREGEX_CASE_INSENSITIVE, status_regex_basetag);

      hasBaseHref = false;
      hasBaseTag = false;
      int32_t attribStart, attribLen, offset, startWDQ, lenWDQ, startWSQ, lenWSQ, startWOQ, lenWOQ;

      bool hasWSQ = true,
           hasWDQ = true,
           hasWOQ = true;

      start = 0;
      len = 0;
      startOfTag = 0;
      lenOfTag = 0;

      UErrorCode status;

      /*while*/if(regex_basetag->reset(icu_part_of_page).find()
        && (hasWDQ || hasWSQ || hasWOQ)  
        && (!hasBaseHref))
      {
        status = U_ZERO_ERROR;
        start = regex_basetag->start(status);
        status = U_ZERO_ERROR;
        len = regex_basetag->end(status) - start;

        delete regex_basetag;
        UErrorCode status_regex = U_ZERO_ERROR;
        RegexMatcher *regex_attributeWOQ = this->m_regexpatt_attributeWOQ->matcher(status_regex);
            //new RegexMatcher("\\s[a-z]+(\\s*=\\s*[^>\\s]*|[^>\\s]+>?|\\s*)", UREGEX_CASE_INSENSITIVE, status_regex_attributeWOQ);
        status_regex = U_ZERO_ERROR;
        RegexMatcher *regex_attributeWSQ = this->m_regexpatt_attributeWSQ->matcher(status_regex);
            //new RegexMatcher("\\s[a-z]+\\s*=\\s*\\'[^\\']*\\'?", UREGEX_CASE_INSENSITIVE, status_regex_attributeWSQ);
        status_regex = U_ZERO_ERROR;
        RegexMatcher *regex_attributeWDQ = this->m_regexpatt_attributeWDQ->matcher(status_regex);
            //new RegexMatcher("\\s[a-z]+\\s*=\\s*\"[^\"]*\"?", UREGEX_CASE_INSENSITIVE, status_regex_attributeWDQ);
        status_regex = U_ZERO_ERROR;
        RegexMatcher *regex_spaces = this->m_regexpatt_spaces->matcher(status_regex);
            //new RegexMatcher("\\s+", UREGEX_CASE_INSENSITIVE, status_regex_spaces);
        status_regex = U_ZERO_ERROR;
        RegexMatcher *regex_hrefReplacer = this->m_regexpatt_hrefReplacer->matcher(status_regex);
          //new RegexMatcher("href\\s*=\\s*(\"[^\"]*\"?|\\'[^\\']*\\'?|[^>\\s]*)", UREGEX_CASE_INSENSITIVE, this->m_status_regex_hrefReplacer);
        status_regex = U_ZERO_ERROR;
        RegexMatcher *regex_hrefReplacerWOEQ = this->m_regexpatt_hrefReplacerWOEQ->matcher(status_regex);
          //new RegexMatcher("(href\\s|href>)", UREGEX_CASE_INSENSITIVE, this->m_status_regex_hrefReplacerWOEQ);
//        status_regex = U_ZERO_ERROR;
//        RegexMatcher *regex_srcReplacer = this->m_regexpatt_srcReplacer->matcher(status_regex);
//            //new RegexMatcher("src\\s*=\\s*(\"[^\"]*\"?|\\'[^\\']*\\'?|[^>\\s]*)", UREGEX_CASE_INSENSITIVE, status_regex);
//        status_regex = U_ZERO_ERROR;
//        RegexMatcher *regex_srcReplacerWOEQ = this->m_regexpatt_srcReplacerWOEQ->matcher(status_regex);
//            //new RegexMatcher("(src\\s|src>)", UREGEX_CASE_INSENSITIVE, status_regex);


        lenOfTag = len - 1;
        startOfTag = start;
        hasBaseTag = true;

        start = start + 5;
        len = len - 5;
        offset = start;

        while ((hasWDQ || hasWSQ || hasWOQ) && (!hasBaseHref))
        {
          startWDQ = 0;
          lenWDQ = 0;
          startWSQ = 0;
          lenWSQ = 0;
          startWOQ = 0;
          lenWOQ = 0;

          hasWDQ = false;
          hasWSQ = false;
          hasWOQ = false;

          atag = UnicodeString(icu_part_of_page, offset, len);
          if (regex_attributeWDQ->reset(atag).find())
          {
            status = U_ZERO_ERROR;
            attribStart = regex_attributeWDQ->start(status);
            status = U_ZERO_ERROR;
            attribLen = regex_attributeWDQ->end(status) - attribStart;

            hasWDQ = true;
            attribWDQ = UnicodeString(atag, attribStart + 1, attribLen - 1);
            startWDQ = offset + attribStart;
            lenWDQ = attribLen;
          }

          if (regex_attributeWSQ->reset(atag).find())
          {
            status = U_ZERO_ERROR;
            attribStart = regex_attributeWSQ->start(status);
            status = U_ZERO_ERROR;
            attribLen = regex_attributeWSQ->end(status) - attribStart;

            hasWSQ = true;
            attribWSQ = UnicodeString(atag, attribStart + 1, attribLen - 1);
            startWSQ = offset + attribStart;
            lenWSQ = attribLen;
          }

          if (regex_attributeWOQ->reset(atag).find())
          {
            status = U_ZERO_ERROR;
            attribStart = regex_attributeWOQ->start(status);
            status = U_ZERO_ERROR;
            attribLen = regex_attributeWOQ->end(status) - attribStart;


            hasWOQ = true;
            attribWOQ = UnicodeString(atag, attribStart + 1, attribLen - 1);
            startWOQ = offset + attribStart;
            lenWOQ = attribLen;

            status = U_ZERO_ERROR;
            attribStart = regex_attributeWOQ->start(1, status);
            status = U_ZERO_ERROR;
            attribLen = regex_attributeWOQ->end(1, status) - start;

            attribWOQ1 = UnicodeString(atag, attribStart, attribLen);
          }

          if (   ((hasWDQ && (!hasWSQ) && (!hasWOQ)))
              || ((hasWDQ && hasWSQ && (!hasWOQ)) && (startWDQ <= startWSQ))
              || ((hasWDQ && (!hasWSQ) && hasWOQ) && (startWDQ <= startWOQ))
              || ((hasWDQ && hasWSQ && hasWOQ && (startWDQ <= startWOQ) && (startWDQ <= startWSQ))))
          {
            if (UnicodeString(attribWDQ).toUpper().startsWith("HREF"))
            {
              if (regex_hrefReplacer->reset(attribWDQ).find())
              {
                status = U_ZERO_ERROR;
                attribStart = regex_hrefReplacer->start(1, status);
                status = U_ZERO_ERROR;
                attribLen = regex_hrefReplacer->end(1, status) - attribStart;

                match = UnicodeString(attribWDQ, attribStart, attribLen);

                // usuwam poczatkowy znak "
                doubleQuote = (match.indexOf("\"") == 0);
                if (doubleQuote)
                  match = UnicodeString(match, 1);

                // "
                if (doubleQuote)
                {
                  if (match.endsWith("\""))
                    match.truncate(match.length() - 1);
                }
                hasBaseHref = true;
                start = startWDQ;
                len   = lenWDQ;

                break;
              }
            }
            offset = startWDQ + lenWDQ;
            len = lenOfTag + startOfTag - offset;
          }
          else if (   ((hasWSQ && (!hasWDQ) && (!hasWOQ)))
                   || ((hasWSQ && hasWDQ && (!hasWOQ)) && (startWSQ <= startWDQ))
                   || ((hasWSQ && (!hasWDQ) && hasWOQ) && (startWSQ <= startWOQ))
                   || ((hasWSQ && hasWDQ && hasWOQ && (startWSQ <= startWOQ) && (startWSQ <= startWDQ))))
          {
            if (UnicodeString(attribWSQ).toUpper().startsWith("HREF"))
            {
              if (regex_hrefReplacer->reset(attribWSQ).find())
              {
                status = U_ZERO_ERROR;
                attribStart = regex_hrefReplacer->start(1, status);
                status = U_ZERO_ERROR;
                attribLen = regex_hrefReplacer->end(1, status) - attribStart;

                match = UnicodeString(attribWSQ, attribStart, attribLen);

                // usuwam poczatkowy znak '
                singleQuote = (match.indexOf("'") == 0);
                if (singleQuote)
                  match = UnicodeString(match, 1);

                // usuwam koncowy znak ' lub "
                if (singleQuote)
                {
                  if (match.endsWith("'"))
                    match.truncate(match.length() - 1);
                }
                hasBaseHref = true;
                start = startWSQ;
                len   = lenWSQ;

                break;
              }
            }
            offset = startWSQ + lenWSQ;
            len = lenOfTag + startOfTag - offset;
          }
          else if (   ((hasWOQ && (!hasWDQ) && (!hasWSQ)))
                   || ((hasWOQ && hasWDQ && (!hasWSQ)) && (startWOQ <= startWDQ))
                   || ((hasWOQ && (!hasWDQ) && hasWSQ) && (startWOQ <= startWSQ))
                   || ((hasWOQ && hasWDQ && hasWSQ && (startWOQ <= startWSQ) && (startWOQ <= startWDQ))))
          {
            if (UnicodeString(attribWOQ).toUpper().startsWith("HREF"))
            {
              if (regex_hrefReplacer->reset(attribWOQ).find())
              {
                status = U_ZERO_ERROR;
                attribStart = regex_hrefReplacer->start(1, status);
                status = U_ZERO_ERROR;
                attribLen = regex_hrefReplacer->end(1, status) - attribStart;

                match = UnicodeString(attribWOQ, attribStart, attribLen);
                hasBaseHref = true;
                start = startWOQ;
                len   = lenWOQ;
                if (attribWOQ1.endsWith(">"))
                  len   = lenWOQ - 1;

                break;
              }
              else if (regex_hrefReplacerWOEQ->reset(attribWOQ).find())
              {
                match = UnicodeString("");
                hasBaseHref = true;
                start = startWOQ;
                len   = lenWOQ;

                if (attribWOQ1.endsWith(">"))
                  len   = lenWOQ - 1;
                else if (regex_spaces->reset(attribWOQ1).find()) // test dla przypadku gdy attribWOQ1 zawiera same znaki \\s
                {
                  status = U_ZERO_ERROR;
                  attribStart = regex_spaces->start(status);
                  status = U_ZERO_ERROR;
                  attribLen = regex_spaces->end(status) - attribStart;

                  if ((attribStart == 0) && (attribLen == attribWOQ1.length()))
                  {
                    len = lenWOQ - attribLen;
                  }
                }
                else
                  len = lenWOQ;

                break;
              }
            }
            offset = startWOQ + lenWOQ;
            len = lenOfTag + startOfTag - offset;
          }

        }

        delete regex_attributeWOQ;
        delete regex_attributeWSQ;
        delete regex_attributeWDQ;
        delete regex_spaces;
        delete regex_hrefReplacer;
        delete regex_hrefReplacerWOEQ;
//        delete regex_srcReplacer;
//        delete regex_srcReplacerWOEQ;
      }

      UnicodeString icumatch;
      IcuURI uri(match);

      if (!(uri.HasScheme() && uri.HasServer()))
      {
        icumatch = icuuri->GetScheme() + "://";
        if (icuuri->HasUserInfo())
        {
          icumatch += icuuri->GetUser();
          if (!icuuri->GetPassword().isEmpty())
            icumatch += ":" + uri.GetPassword();
          icumatch += "@";
        }
        icumatch += icuuri->GetServer();
        if (icuuri->HasPort())
        {
          icumatch += ":" + icuuri->GetPort();
        }

        if (!icuuri->HasPath())
          icumatch += "/";
        else
        {
          icumatch += icuuri->GetPath();
          if (!icumatch.endsWith("/"))
            icumatch.truncate(icumatch.lastIndexOf('/') + 1);
        }
      }
      else
      {
        if ((!uri.GetScheme().compare("http")) ||
            (!uri.GetScheme().compare("https"))||
            (!uri.GetScheme().compare("ftp")))
        {
          icumatch = uri.GetScheme() + "://";
          if (uri.HasUserInfo())
          {
            icumatch += uri.GetUser();
            if (!uri.GetPassword().isEmpty())
              icumatch += ":" + uri.GetPassword();
            icumatch += "@";
          }
          icumatch += uri.GetServer();
          if (uri.HasPort())
          {
            icumatch += ":" + uri.GetPort();
          }
          if (!uri.HasPath())
            icumatch += "/";
          else
          {
            icumatch += uri.GetPath();
            if (!icumatch.endsWith("/"))
              icumatch.truncate(icumatch.lastIndexOf('/') + 1);
          }
        }
        else
        {
          icumatch = icuuri->GetScheme() + "://";
          if (icuuri->HasUserInfo())
          {
            icumatch += icuuri->GetUser();
            if (!icuuri->GetPassword().isEmpty())
              icumatch += ":" + uri.GetPassword();
            icumatch += "@";
          }
          icumatch += icuuri->GetServer();

          if (icuuri->HasPort())
          {
            icumatch += ":" + icuuri->GetPort();
          }

          if (!icuuri->HasPath())
            icumatch += "/";
          else
          {
            icumatch += icuuri->GetPath();
            if (!icumatch.endsWith("/"))
              icumatch.truncate(icumatch.lastIndexOf('/') + 1);
          }

        }

      }

      uri.Create(icumatch);

      return uri;
    }

    IcuURI Utils::match_a_tag(const UnicodeString &icu_part_of_page, const IcuURI &baseURI, bool &hasAHref, bool &hasATag, int32_t &start, int32_t &len, int32_t &startOfTag, int32_t &lenOfTag, KindOfAttrValue &koav, const string &sessionID, string &sessionIDVal)
    {
      UnicodeString atag, match, struri, attribWDQ, attribWSQ, attribWOQ, attribWOQ1;
      bool singleQuote, doubleQuote;

      UErrorCode status_regex_atag = U_ZERO_ERROR;
      RegexMatcher *regex_atag = this->m_regexpatt_atag->matcher(status_regex_atag);
          //new RegexMatcher("<a(\\s+[^>=\\s]+\\s*=\\s*\"[^\"]*\"|\\s+[^>=\\s]+\\s*=\\s*\\'[^\\']*\\'|\\s+[^>=\\s]+=?[^>\\s]*|[^>=\\s]+=?[^>\\s]*)+[^>]*>", UREGEX_CASE_INSENSITIVE, status_regex_atag);

      hasAHref = false;
      hasATag = false;
      int32_t attribStart, attribLen, offset, startWDQ, lenWDQ, startWSQ, lenWSQ, startWOQ, lenWOQ;

      bool hasWSQ = true,
           hasWDQ = true,
           hasWOQ = true;

      start = 0;
      len = 0;
      startOfTag = 0;
      lenOfTag = 0;

      UErrorCode status = U_ZERO_ERROR;

      /*while*/if(regex_atag->reset(icu_part_of_page).find()
        && (hasWDQ || hasWSQ || hasWOQ) 
        && (!hasAHref))
      {
        status = U_ZERO_ERROR;
        start = regex_atag->start(status);
        status = U_ZERO_ERROR;
        len = regex_atag->end(status) - start;

        delete regex_atag;
        UErrorCode status_regex = U_ZERO_ERROR;
        RegexMatcher *regex_attributeWOQ = this->m_regexpatt_attributeWOQ->matcher(status_regex);
            //new RegexMatcher("\\s[a-z]+(\\s*=\\s*[^>\\s]*|[^>\\s]+>?|\\s*)", UREGEX_CASE_INSENSITIVE, status_regex_attributeWOQ);
        status_regex = U_ZERO_ERROR;
        RegexMatcher *regex_attributeWSQ = this->m_regexpatt_attributeWSQ->matcher(status_regex);
            //new RegexMatcher("\\s[a-z]+\\s*=\\s*\\'[^\\']*\\'?", UREGEX_CASE_INSENSITIVE, status_regex_attributeWSQ);
        status_regex = U_ZERO_ERROR;
        RegexMatcher *regex_attributeWDQ = this->m_regexpatt_attributeWDQ->matcher(status_regex);
            //new RegexMatcher("\\s[a-z]+\\s*=\\s*\"[^\"]*\"?", UREGEX_CASE_INSENSITIVE, status_regex_attributeWDQ);
        status_regex = U_ZERO_ERROR;
        RegexMatcher *regex_spaces = this->m_regexpatt_spaces->matcher(status_regex);
            //new RegexMatcher("\\s+", UREGEX_CASE_INSENSITIVE, status_regex_spaces);
        status_regex = U_ZERO_ERROR;
        RegexMatcher *regex_hrefReplacer = this->m_regexpatt_hrefReplacer->matcher(status_regex);
          //new RegexMatcher("href\\s*=\\s*(\"[^\"]*\"?|\\'[^\\']*\\'?|[^>\\s]*)", UREGEX_CASE_INSENSITIVE, this->m_status_regex_hrefReplacer);
        status_regex = U_ZERO_ERROR;
        RegexMatcher *regex_hrefReplacerWOEQ = this->m_regexpatt_hrefReplacerWOEQ->matcher(status_regex);
          //new RegexMatcher("(href\\s|href>)", UREGEX_CASE_INSENSITIVE, this->m_status_regex_hrefReplacerWOEQ);
//        status_regex = U_ZERO_ERROR;
//        RegexMatcher *regex_srcReplacer = this->m_regexpatt_srcReplacer->matcher(status_regex);
//            //new RegexMatcher("src\\s*=\\s*(\"[^\"]*\"?|\\'[^\\']*\\'?|[^>\\s]*)", UREGEX_CASE_INSENSITIVE, status_regex);
//        status_regex = U_ZERO_ERROR;
//        RegexMatcher *regex_srcReplacerWOEQ = this->m_regexpatt_srcReplacerWOEQ->matcher(status_regex);
//            //new RegexMatcher("(src\\s|src>)", UREGEX_CASE_INSENSITIVE, status_regex);

        startOfTag = start;
        lenOfTag = len;
        start = start + 2;
        len = len - 2;
        offset = start;
        hasATag = true;

        while ((hasWDQ || hasWSQ || hasWOQ) && (!hasAHref))
        {
          startWDQ = 0;
          lenWDQ = 0;
          startWSQ = 0;
          lenWSQ = 0;
          startWOQ = 0;
          lenWOQ = 0;

          hasWDQ = false;
          hasWSQ = false;
          hasWOQ = false;

          atag = UnicodeString(icu_part_of_page, offset, len);
          if (regex_attributeWDQ->reset(atag).find())
          {
            status = U_ZERO_ERROR;
            attribStart = regex_attributeWDQ->start(status);
            status = U_ZERO_ERROR;
            attribLen = regex_attributeWDQ->end(status) - attribStart;

            hasWDQ = true;
            attribWDQ = UnicodeString(atag, attribStart + 1, attribLen - 1);
            startWDQ = offset + attribStart;
            lenWDQ = attribLen;
          }

          if (regex_attributeWSQ->reset(atag).find())
          {
              status = U_ZERO_ERROR;
              attribStart = regex_attributeWSQ->start(status);
              status = U_ZERO_ERROR;
              attribLen = regex_attributeWSQ->end(status) - attribStart;

              hasWSQ = true;
              attribWSQ = UnicodeString(atag, attribStart + 1, attribLen - 1);
              startWSQ = offset + attribStart;
              lenWSQ = attribLen;
          }

          if (regex_attributeWOQ->reset(atag).find())
          {
            status = U_ZERO_ERROR;
            attribStart = regex_attributeWOQ->start(status);
            status = U_ZERO_ERROR;
            attribLen = regex_attributeWOQ->end(status) - attribStart;

            hasWOQ = true;
            attribWOQ = UnicodeString(atag, attribStart + 1, attribLen - 1);
            startWOQ = offset + attribStart;
            lenWOQ = attribLen;

            status = U_ZERO_ERROR;
            attribStart = regex_attributeWOQ->start(1, status);
            status = U_ZERO_ERROR;
            attribLen = regex_attributeWOQ->end(1, status) - attribStart;

            attribWOQ1 = UnicodeString(atag, attribStart, attribLen);
          }



          if (   ((hasWDQ && (!hasWSQ) && (!hasWOQ)))
              || ((hasWDQ && hasWSQ && (!hasWOQ)) && (startWDQ <= startWSQ))
              || ((hasWDQ && (!hasWSQ) && hasWOQ) && (startWDQ <= startWOQ))
              || ((hasWDQ && hasWSQ && hasWOQ && (startWDQ <= startWOQ) && (startWDQ <= startWSQ))))
          {
            if (UnicodeString(attribWDQ).toUpper().startsWith("HREF"))
            {
              if (regex_hrefReplacer->reset(attribWDQ).find())
              {
                status = U_ZERO_ERROR;
                attribStart = regex_hrefReplacer->start(1, status);
                status = U_ZERO_ERROR;
                attribLen = regex_hrefReplacer->end(1, status) - attribStart;

                match = UnicodeString(attribWDQ, attribStart, attribLen);

                // usuwam pocz�tkowy znak "
                doubleQuote = (match.indexOf("\"") == 0);
                if (doubleQuote)
                  match = UnicodeString(match, 1);

                if (doubleQuote)
                {
                  if (match.endsWith("\""))
                    match.truncate(match.length() - 1);
                }
                hasAHref = true;
                start = startWDQ;
                len   = lenWDQ;
                koav  = koavWDQ;
                break;
              }
            }
            offset = startWDQ + lenWDQ;
            len = lenOfTag + startOfTag - offset;
          }
          else if (   ((hasWSQ && (!hasWDQ) && (!hasWOQ)))
                   || ((hasWSQ && hasWDQ && (!hasWOQ)) && (startWSQ <= startWDQ))
                   || ((hasWSQ && (!hasWDQ) && hasWOQ) && (startWSQ <= startWOQ))
                   || ((hasWSQ && hasWDQ && hasWOQ && (startWSQ <= startWOQ) && (startWSQ <= startWDQ))))
          {
            if (UnicodeString(attribWSQ).toUpper().startsWith("HREF"))
            {
              if (regex_hrefReplacer->reset(attribWSQ).find())
              {
                status = U_ZERO_ERROR;
                attribStart = regex_hrefReplacer->start(1, status);
                status = U_ZERO_ERROR;
                attribLen = regex_hrefReplacer->end(1, status) - attribStart;

                match = UnicodeString(attribWSQ, attribStart, attribLen);

                // usuwam poczatkowy znak ' lub "
                singleQuote = (match.indexOf("'") == 0);
                if (singleQuote)
                  match = UnicodeString(match, 1);

                // usuwam koncowy znak ' lub "
                if (singleQuote)
                {
                  if (match.endsWith("'"))
                    match.truncate(match.length() - 1);
                }
                hasAHref = true;
                start = startWSQ;
                len   = lenWSQ;
                koav  = koavWSQ;

                break;
              }
            }
            offset = startWSQ + lenWSQ;
            len = lenOfTag + startOfTag - offset;
          }
          else if (   ((hasWOQ && (!hasWDQ) && (!hasWSQ)))
                   || ((hasWOQ && hasWDQ && (!hasWSQ)) && (startWOQ <= startWDQ))
                   || ((hasWOQ && (!hasWDQ) && hasWSQ) && (startWOQ <= startWSQ))
                   || ((hasWOQ && hasWDQ && hasWSQ && (startWOQ <= startWSQ) && (startWOQ <= startWDQ))))
          {
            if (UnicodeString(attribWOQ).toUpper().startsWith("HREF"))
            {
              if (regex_hrefReplacer->reset(attribWOQ).find())
              {
                status = U_ZERO_ERROR;
                attribStart = regex_hrefReplacer->start(1, status);
                status = U_ZERO_ERROR;
                attribLen = regex_hrefReplacer->end(1, status) - attribStart;

                match = UnicodeString(attribWOQ, attribStart, attribLen);

                hasAHref = true;
                start = startWOQ;
                len   = lenWOQ;
                koav  = koavWOQ;

                if (attribWOQ1.endsWith(">"))
                  len   = lenWOQ - 1;

                break;
              }
              else if (regex_hrefReplacerWOEQ->reset(attribWOQ).find())
              {

                match = baseURI.BuildURI();
                hasAHref = true;
                start = startWOQ;
                koav  = koavWOQ;

                if (attribWOQ1.endsWith(">"))
                  len   = lenWOQ - 1;
                else if (regex_spaces->reset(attribWOQ1).find()) // test dla przypadku gdy attribWOQ1 zawiera same znaki \\s
                {
                  status = U_ZERO_ERROR;
                  attribStart = regex_spaces->start(status);
                  status = U_ZERO_ERROR;
                  attribLen = regex_spaces->end(status) - attribStart;

                  if ((attribStart == 0) && (attribLen == attribWOQ1.length()))
                  {
                    len = lenWOQ - attribLen;
                  }
                }
                else
                  len = lenWOQ;

                break;
              }

            }
            offset = startWOQ + lenWOQ;
            len = lenOfTag + startOfTag - offset;
          }

        }

        delete regex_attributeWOQ;
        delete regex_attributeWSQ;
        delete regex_attributeWDQ;
        delete regex_spaces;
        delete regex_hrefReplacer;
        delete regex_hrefReplacerWOEQ;
//        delete regex_srcReplacer;
//        delete regex_srcReplacerWOEQ;
      }

      IcuURI uri(match);
      uri.Resolve(baseURI, IcuURI::IcuURI_STRICT);
      if (!sessionID.empty())
      {
        struri = uri.GetScheme() + "://";
        if (uri.HasUserInfo())
        {
          struri += uri.GetUser();
          if (!uri.GetPassword().isEmpty())
            struri += ":" + uri.GetPassword();
          struri += "@";
        }
        struri += uri.GetServer();
        if (uri.HasPort())
        {
          struri += ":" + uri.GetPort();
        }

        struri += uri.GetPath();

        match = uri.GetQuery();

        UErrorCode status_regex_sessionIDremoval = U_ZERO_ERROR;
        RegexMatcher *regex_sessionIDremoval = new RegexMatcher(("(&|\\?)?"+this->m_sessionID+"=([^&]*)").c_str(), UREGEX_CASE_INSENSITIVE, status_regex_sessionIDremoval);
        if(regex_sessionIDremoval->reset(match).find())
        {
          status = U_ZERO_ERROR;
          start = regex_sessionIDremoval->start(2, status);
          status = U_ZERO_ERROR;
          len = regex_sessionIDremoval->end(2, status) - start;

          UnicodeString(match, start, len).toUTF8String(sessionIDVal);

          status = U_ZERO_ERROR;
          match = regex_sessionIDremoval->replaceAll("", status);
        }
        delete regex_sessionIDremoval;
        if (!match.isEmpty())
        {
          if (match.indexOf("&") == 0)
            match = UnicodeString(match, 1);
          match = "?" + match;
        }


        if (uri.HasFragment())
          struri += match + "#" + uri.GetFragment();
        else
          struri += match;
        uri.Create(struri);
      }
      return uri;
    }
        
    IcuURI Utils::buildURIWithPathAfterServer(const IcuURI *icuuri)
    {
      UnicodeString uri;
      if (icuuri)
      {
        if (icuuri->HasScheme())
        {
          uri = icuuri->GetScheme();
          uri += "://";
        }
        if (icuuri->HasUserInfo())
        {
          uri += icuuri->GetUser();
          if (!icuuri->GetPassword().isEmpty())
            uri += ":" + icuuri->GetPassword();
          uri += "@";
        }
        if (icuuri->HasServer())
          uri += icuuri->GetServer();
        if (icuuri->HasPath())
        {
          uri += icuuri->GetPath();
        }
        else
        {
          if (icuuri->HasServer())
            uri += "/";
        }
        if (icuuri->HasQuery())
          uri += "?" + icuuri->GetQuery();
        if (icuuri->HasFragment())
          uri += "#" + icuuri->GetFragment();
      }
      return IcuURI(uri);
    }
        
    IcuURI Utils::buildURIWithQueryParam(const IcuURI *icuuri, const UnicodeString &qparam, const UnicodeString &qvalue)
    {
      UnicodeString uri;
      if (icuuri)
      {
        if (icuuri->HasScheme())
        {
          uri = icuuri->GetScheme();
          uri += "://";
        }
        if (icuuri->HasUserInfo())
        {
          uri += icuuri->GetUser();
          if (!icuuri->GetPassword().isEmpty())
            uri += ":" + icuuri->GetPassword();
          uri += "@";
        }
        if (icuuri->HasServer())
          uri += icuuri->GetServer();
        if (icuuri->HasPath())
        {
          uri += icuuri->GetPath();
        }
        else
        {
          if (icuuri->HasServer())
            uri += "/";
        }
        uri += (icuuri->HasQuery() ? ("?" + icuuri->GetQuery() + "&" + qparam + "=" + qvalue)
                                 :  ("?" + qparam + "=" + qvalue));

        if (icuuri->HasFragment())
          uri += "#" + icuuri->GetFragment();
      }
      return IcuURI(uri);
    }


    KindOfContentTypeValue Utils::match_content_type(const string &httpResponseHeader)
    {
      string content_type;
      // checking the content type in http response

      boost::match_results<string::const_iterator> matchesi;
      //boost::match_results<string::const_iterator> matches;

      string::const_iterator begini;

      begini = httpResponseHeader.begin();

      while (boost::regex_search(begini, httpResponseHeader.end(), matchesi, Utils::mcs_regex_contentType, boost::match_default))
      {
        if (matchesi.size() > 0)
          content_type.assign(matchesi[1].first, matchesi[1].second);
        begini = matchesi[1].second;
      }

      //if (boost::regex_search(this->m_httpResponseHeader, matches, DownloadThread::mcs_regex_contentType, boost::match_default))
      //{
      //  if (matches.size() > 0)
      //    content_type.assign(matches[1].first, matches[1].second);
      //}
      UnicodeString u_content_type = UnicodeString::fromUTF8(content_type);
      u_content_type.toUpper();
      content_type.clear();
      u_content_type.toUTF8String(content_type);

      SyNaT::MIR::SP::SyNaTPGApp::ptr_map_string_string::const_iterator i = SyNaT::MIR::SP::SyNaTPGApp::mcs_mime.find(content_type);

      KindOfContentTypeValue koctv_content_type;

      if ((!content_type.empty()) && (i != SyNaT::MIR::SP::SyNaTPGApp::mcs_mime.end()))
      {
        if ((i->second->find("xml") != string::npos))
          koctv_content_type = koctvXHTML;
        else if (i->second->find("htm") != string::npos)
          koctv_content_type = koctvHTML;
        else if (i->second->find("txt") != string::npos)
          koctv_content_type = koctvPlainText;
        else if (i->second->find("css") != string::npos)
          koctv_content_type = koctvCSS;
        else if (i->second->find("js") != string::npos)
          koctv_content_type = koctvJavaScript;

        else if (i->second->find("snd") != string::npos)
          koctv_content_type = koctvAudioContent;
        else if (i->second->find("mid") != string::npos)
          koctv_content_type = koctvAudioContent;
        else if (i->second->find("mp3") != string::npos)
          koctv_content_type = koctvAudioContent;
        else if (i->second->find("mp4") != string::npos)
          koctv_content_type = koctvAudioContent;
        else if (i->second->find("oga") != string::npos)
          koctv_content_type = koctvAudioContent;
        else if (i->second->find("wma") != string::npos)
          koctv_content_type = koctvAudioContent;
        else if (i->second->find("wax") != string::npos)
          koctv_content_type = koctvAudioContent;
        else if (i->second->find("wmv") != string::npos)
          koctv_content_type = koctvAudioContent;
        else if (i->second->find("mka") != string::npos)
          koctv_content_type = koctvAudioContent;
        else if (i->second->find("ra") != string::npos)
          koctv_content_type = koctvAudioContent;
        else if (i->second->find("wav") != string::npos)
          koctv_content_type = koctvAudioContent;
        else if (i->second->find("mka") != string::npos)
          koctv_content_type = koctvAudioContent;

        else if (i->second->find("m3u") != string::npos)
          koctv_content_type = koctvAudioM3U;

        else if (i->second->find("bmp") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("cgm") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("g3") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("gif") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("ief") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("jp2") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("jpeg") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("png") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("btif") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("svg") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("tiff") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("psd") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("djvu") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("dwg") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("dxf") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("fbs") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("fpx") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("fst") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("mmr") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("rlc") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("mdi") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("npx") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("wbmp") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("xif") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("ras") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("cmx") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("fh") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("ico") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("pcx") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("pic") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("pnm") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("pbm") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("pgm") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("ppm") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("rgb") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("xbm") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("xpm") != string::npos)
          koctv_content_type = koctvImage;
        else if (i->second->find("xwd") != string::npos)
          koctv_content_type = koctvImage;

      }
      else
      {
        koctv_content_type = koctvOther;
      }
      return koctv_content_type;
    }


    string Utils::match_content_type_ext(const string &httpResponseHeader)
    {
      string content_type;
      // checking the content type in http response

      boost::match_results<string::const_iterator> matchesi;
      //boost::match_results<string::const_iterator> matches;

      string::const_iterator begini;

      begini = httpResponseHeader.begin();

      while (boost::regex_search(begini, httpResponseHeader.end(), matchesi, Utils::mcs_regex_contentType, boost::match_default))
      {
        if (matchesi.size() > 0)
          content_type.assign(matchesi[1].first, matchesi[1].second);
        begini = matchesi[1].second;
      }

      //if (boost::regex_search(this->m_httpResponseHeader, matches, DownloadThread::mcs_regex_contentType, boost::match_default))
      //{
      //  if (matches.size() > 0)
      //    content_type.assign(matches[1].first, matches[1].second);
      //}
      UnicodeString u_content_type = UnicodeString::fromUTF8(content_type);
      u_content_type.toUpper();
      content_type.clear();
      u_content_type.toUTF8String(content_type);

      SyNaT::MIR::SP::SyNaTPGApp::ptr_map_string_string::const_iterator i = SyNaT::MIR::SP::SyNaTPGApp::mcs_mime.find(content_type);

      string content_type_ext;

      if ((!content_type.empty()) && (i != SyNaT::MIR::SP::SyNaTPGApp::mcs_mime.end()))
      {
        if ((i->second->find("xml") != string::npos))
          content_type_ext = "xml";
        else if (i->second->find("htm") != string::npos)
          content_type_ext = "htm";

        else if (i->second->find("txt") != string::npos)
          content_type_ext = "txt";

        else if (i->second->find("snd") != string::npos)
          content_type_ext = "snd";
        else if (i->second->find("mid") != string::npos)
          content_type_ext = "mid";
        else if (i->second->find("mp3") != string::npos)
          content_type_ext = "mp3";
        else if (i->second->find("mp4") != string::npos)
          content_type_ext = "mp4";
        else if (i->second->find("oga") != string::npos)
          content_type_ext = "oga";
        else if (i->second->find("wma") != string::npos)
          content_type_ext = "wma";
        else if (i->second->find("wax") != string::npos)
          content_type_ext = "wax";
        else if (i->second->find("wmv") != string::npos)
          content_type_ext = "wmv";
        else if (i->second->find("mka") != string::npos)
          content_type_ext = "mka";
        else if (i->second->find("ra") != string::npos)
          content_type_ext = "ra";
        else if (i->second->find("wav") != string::npos)
          content_type_ext = "wav";
        else if (i->second->find("mka") != string::npos)
          content_type_ext = "mka";

        else if (i->second->find("m3u") != string::npos)
          content_type_ext = "m3u";
        else if (i->second->find("m3u8") != string::npos)
          content_type_ext = "m3u8";


        else if (i->second->find("css") != string::npos)
          content_type_ext = "css";

        else if (i->second->find("js") != string::npos)
          content_type_ext = "js";

        else if (i->second->find("bmp") != string::npos)
          content_type_ext = "bmp";
        else if (i->second->find("cgm") != string::npos)
          content_type_ext = "cgm";
        else if (i->second->find("g3") != string::npos)
          content_type_ext = "g3";
        else if (i->second->find("gif") != string::npos)
          content_type_ext = "gif";
        else if (i->second->find("ief") != string::npos)
          content_type_ext = "ief";
        else if (i->second->find("jp2") != string::npos)
          content_type_ext = "jp2";
        else if (i->second->find("jpeg") != string::npos)
          content_type_ext = "jpeg";
        else if (i->second->find("png") != string::npos)
          content_type_ext = "png";
        else if (i->second->find("btif") != string::npos)
          content_type_ext = "btif";
        else if (i->second->find("svg") != string::npos)
          content_type_ext = "svg";
        else if (i->second->find("tiff") != string::npos)
          content_type_ext = "tiff";
        else if (i->second->find("psd") != string::npos)
          content_type_ext = "psd";
        else if (i->second->find("djvu") != string::npos)
          content_type_ext = "djvu";
        else if (i->second->find("dwg") != string::npos)
          content_type_ext = "dwg";
        else if (i->second->find("dxf") != string::npos)
          content_type_ext = "dxf";
        else if (i->second->find("fbs") != string::npos)
          content_type_ext = "fbs";
        else if (i->second->find("fpx") != string::npos)
          content_type_ext = "fpx";
        else if (i->second->find("fst") != string::npos)
          content_type_ext = "fst";
        else if (i->second->find("mmr") != string::npos)
          content_type_ext = "mmr";
        else if (i->second->find("rlc") != string::npos)
          content_type_ext = "rlc";
        else if (i->second->find("mdi") != string::npos)
          content_type_ext = "mdi";
        else if (i->second->find("npx") != string::npos)
          content_type_ext = "npx";
        else if (i->second->find("wbmp") != string::npos)
          content_type_ext = "wbmp";
        else if (i->second->find("xif") != string::npos)
          content_type_ext = "xif";
        else if (i->second->find("ras") != string::npos)
          content_type_ext = "ras";
        else if (i->second->find("cmx") != string::npos)
          content_type_ext = "cmx";
        else if (i->second->find("fh") != string::npos)
          content_type_ext = "fh";
        else if (i->second->find("ico") != string::npos)
          content_type_ext = "ico";
        else if (i->second->find("pcx") != string::npos)
          content_type_ext = "pcx";
        else if (i->second->find("pic") != string::npos)
          content_type_ext = "pic";
        else if (i->second->find("pnm") != string::npos)
          content_type_ext = "pnm";
        else if (i->second->find("pbm") != string::npos)
          content_type_ext = "pbm";
        else if (i->second->find("pgm") != string::npos)
          content_type_ext = "pgm";
        else if (i->second->find("ppm") != string::npos)
          content_type_ext = "ppm";
        else if (i->second->find("rgb") != string::npos)
          content_type_ext = "rgb";
        else if (i->second->find("xbm") != string::npos)
          content_type_ext = "xbm";
        else if (i->second->find("xpm") != string::npos)
          content_type_ext = "xpm";
        else if (i->second->find("xwd") != string::npos)
          content_type_ext = "xwd";

      }
      else
      {
        content_type_ext = "";
      }
      return content_type_ext;
    }

    string Utils::match_charset(const string &httpResponse, const string &httpResponseHeader)
    {
      boost::match_results<string::const_iterator> matches;
      string charset;

      // we search in html document first - becouse it is more important
      if (boost::regex_search(httpResponse, matches, Utils::mcs_regex_metacharset, boost::match_default))
      {
        if (matches.size() > 0)
          charset.assign(matches[1].first, matches[1].second);
      }

      // we search for charset inside response header
      if (charset.empty())
      {
        if (boost::regex_search(httpResponseHeader, matches, Utils::mcs_regex_contentTypeCharset, boost::match_default))
        {
          if (matches.size() > 0)
            charset.assign(matches[1].first, matches[1].second);
        }
      }
      return charset;
    }

    UnicodeString Utils::CreateTempFileName()
    {
      boost::filesystem::path bptmpp = boost::filesystem::unique_path(unique_path(path("%%%%%%%%%%%%%%%%")));
      return UnicodeString::fromUTF8(bptmpp.string());
    }

    UnicodeString Utils::buildDestHref(UnicodeString &icudirplusfilename, IcuURI &icu_a_href, ContentTypeRetriever &ctr, boost::filesystem::path &bprootwww)
    {
      UnicodeString icu_dest_href = Utils::subAllNotAllowedCharsInFilenames(icu_a_href.GetServer());
      UnicodeString icu_a_hrefPathExt;
      //if (icu_a_href.HasPath())
      //{
      UnicodeString icu_a_hrefPath = icu_a_href.GetPath();
      icu_a_hrefPath.findAndReplace("%", "");
      icu_a_hrefPath = Utils::subAllNotAllowedCharsInFilenames(icu_a_hrefPath, false, true);
      //  if (!icu_a_hrefPath.endsWith("/"))
      //  {
      int lastDotPos = icu_a_hrefPath.lastIndexOf('.');
      //    if (lastDotPos < 0)
      //      icu_a_hrefPathExt = "";
      //    else
      //    {
      icu_a_hrefPathExt = UnicodeString(icu_a_hrefPath, lastDotPos + 1);
        //    icu_a_hrefPath = UnicodeString(icu_a_hrefPath, 0, lastDotPos);
        //  }                                                          
        //}
      
      //  icu_dest_href += icu_a_hrefPath;
      //}
      //else
      //{
      //icu_dest_href += "/";
      //}

      //if (icu_a_href.HasQuery())
      //  icu_dest_href += Utils::subAllNotAllowedCharsInFilenames("?" + icu_a_href.GetQuery());
      //if (icu_a_href.HasFragment())
      //  icu_dest_href += Utils::subAllNotAllowedCharsInFilenames("#" + icu_a_href.GetFragment());


      //int32_t filename_pos = icu_dest_href.lastIndexOf('/');
      UnicodeString filename(unique_path(path("%%%%%%%%%%%%%%%%")).string().c_str());
      //icu_dest_href = UnicodeString(icu_dest_href, 0, filename_pos);

      // dodaje ograniczenie na długość 255 znaków w nazwie pliku (przyjmuje długość rozszerzenia 5 (4 + 1 na znak kropki)
      //if (filename.length() > 250)
      //	filename = UnicodeString(filename, 0, 250);
      icu_dest_href += UnicodeString(icu_a_hrefPath, 0, icu_a_hrefPath.lastIndexOf("/"));
      icu_dest_href += "/" + filename;

      //if ((icu_a_href.HasPath() && (!icu_a_href.GetPath().endsWith("/")))
      //    || icu_a_href.HasQuery()
      //    || icu_a_href.HasFragment())
      //{
        if (!icu_a_hrefPathExt.compare("m3u") || !icu_a_hrefPathExt.compare("m3u8")
          || !ctr.getContentTypeExtension().compare("m3u") || !ctr.getContentTypeExtension().compare("m3u8"))
          icu_dest_href += ".htm";
        else
          icu_dest_href += ctr.getContentTypeExtension().empty() ? icu_a_hrefPathExt.startsWith("mp3") ? ".mp3" : "." + icu_a_hrefPathExt : UnicodeString::fromUTF8("." + ctr.getContentTypeExtension());
        boost::filesystem::path bpfn;
        string sicu_dest_href;
        icu_dest_href.toUTF8String(sicu_dest_href);
        (bpfn /= bprootwww)
            /= sicu_dest_href;
//        fn.Assign(fnrootwww.GetFullPath(wxPATH_UNIX)
//                   + fn.GetPathSeparator(wxPATH_UNIX) + wx_dest_href, wxPATH_UNIX);
//        fn.Normalize(wxPATH_NORM_LONG|wxPATH_NORM_DOTS|wxPATH_NORM_TILDE|wxPATH_NORM_ABSOLUTE);
        icudirplusfilename = UnicodeString::fromUTF8(bpfn.generic_string().c_str());

//      }
//      else 
//      {
//        UnicodeString icufilename;
//
//        icufilename = Utils::CreateTempFileName(2);
//        string sfilename;
//        icufilename.toUTF8String(sfilename);
//        boost::filesystem::path bpfn(sfilename);
//        string sicu_dest_href;
//        icu_dest_href.toUTF8String(sicu_dest_href);
//        bpfn = (bprootwww /= sicu_dest_href)
//                        /= bpfn.stem();
////        fn.Assign(fnrootwww.GetFullPath(wxPATH_UNIX)
////          + fn.GetPathSeparator(wxPATH_UNIX)
////          + wx_dest_href
////          + fn.GetName());
//
////        fn.Normalize(wxPATH_NORM_LONG|wxPATH_NORM_DOTS|wxPATH_NORM_TILDE|wxPATH_NORM_ABSOLUTE);
//        if (!icu_a_hrefPathExt.compare("m3u") || !icu_a_hrefPathExt.compare("m3u8")
//          || !ctr.getContentTypeExtension().compare("m3u") || !ctr.getContentTypeExtension().compare("m3u8"))
//          icufilename = UnicodeString::fromUTF8(ZenLib::Ztring((bpfn.filename() /= ".htm").c_str()).To_UTF8());
//        else
//          icufilename = UnicodeString::fromUTF8(ZenLib::Ztring(bpfn.filename().c_str()).To_UTF8()) + (ctr.getContentTypeExtension().empty() ? icu_a_hrefPathExt.startsWith("mp3") ? UnicodeString::fromUTF8(string(".mp3")) : "." + icu_a_hrefPathExt : UnicodeString::fromUTF8("." + ctr.getContentTypeExtension()));
//
//        icu_dest_href += icufilename;
//        string sicufilename;
//        icufilename.toUTF8String(sicufilename);
//        icudirplusfilename = UnicodeString::fromUTF8(ZenLib::Ztring(((boost::filesystem::path("/")
//        																		/= bpfn.relative_path().string())
//        																		/= sicufilename).generic_string().c_str()).To_UTF8());
//      }
      return icu_dest_href;
    }

    bool Utils::isMusicRobotWorking(DatabaseModule *dm, Transaction *tr)
    {
      if(!dm)
        throw DatabaseNotAssignedMirException(string("Utils::stateOfMusicRobot(DatabaseModule *db, Transaction *tr)"));

      if(!tr)
        throw TransactionNotAssignedMirException(string("Utils::stateOfMusicRobot(DatabaseModule *db, Transaction *tr)"));

      bool musicRobotState = false;
      string sql("SELECT SMR_PRACUJE FROM MIR_STAN_MUZCZYNEGO_ROBOTA");
      Statement stmt = dm->createStatement(tr, &sql);
      stmt->Execute();
      if (stmt->Fetch())
      {
        stmt->Get(1, musicRobotState);
      }
      else
      {
        throw DbTableIsEmptyMirException(string("Utils::stateOfMusicRobot(DatabaseModule *db, Transaction *tr)"));
      }
      return musicRobotState;
    }


    UnicodeString Utils::subAllNotAllowedCharsInFilenames(UnicodeString input, bool replaceSlashes, bool doNotReplaceLastDot)
    {
      if (replaceSlashes)
        input.findAndReplace("/",  "2f");
      input.findAndReplace("\\", "5c");
      input.findAndReplace("?", "3f");
      input.findAndReplace("%", "25");
      input.findAndReplace("*", "2a");
      input.findAndReplace(":", "3a");
      input.findAndReplace("|", "7c");
      input.findAndReplace("\"", "22");
      input.findAndReplace("<", "3c");
      input.findAndReplace(">", "3e");
      if (!doNotReplaceLastDot)
        input.findAndReplace(".", "2e");
      else
      {
        int32_t indexOfLastDot = input.lastIndexOf('.');
        int32_t indexOfLastSlash = input.lastIndexOf('/');
        if (indexOfLastSlash > indexOfLastDot)
          input.findAndReplace(".",  "2e");
        else
        {
          UnicodeString withoutLastDot(input);
          if (indexOfLastDot >= 0)
            withoutLastDot = UnicodeString(input, 0, indexOfLastDot);
          withoutLastDot.findAndReplace(".", "2e");
          input = withoutLastDot;
          if (indexOfLastDot >= 0)
            input += UnicodeString(input, indexOfLastDot);
        }
      }
      return input;
    }

    /**
     * path musi zaczynać się od /
     */
    UnicodeString Utils::buildParameterMapFromPath(string path, ptr_map_string_UnicodeString &params, string &charset)
    {
//      UnicodeString icupath(path, charset);
//      wxURI uri(wxpath);
//      wxpath = wxURI::Unescape(uri.BuildURI());
//      CHARSVParser parser(wxpath, wxT('/'));
//      parser.GetNextToken();
//      wxString wxReturnPath = parser.HasMoreTokens() ? parser.GetNextToken() : wxT("");
//      for(; parser.HasMoreTokens() ; )
//      {
//        wxString parName = parser.GetNextToken();
//        if (parser.HasMoreTokens())
//        {
//          wxString parValue = parser.GetNextToken();
//          params[wx2std(parName, &wxCharSetConv)] = *new wxString(parValue);
//        }
//        else
//        {
//          params[wx2std(parName, &wxCharSetConv)] = *new wxString();
//        }
//      }
//      return wxReturnPath;
      return *new UnicodeString();
    }

    void Utils::replaceNewlines(UnicodeString &input)
    {
      int32_t j = 0;
      for(int32_t i = 0; i < input.length(); i++, j++)
      {
        while ((j < input.length()) && ((input[j] == '\n') || (input[j] == '\r')))
          j++;

        if (i != j)
          input = UnicodeString(input, 0, i) + UnicodeString(input, j);
        i = j;
      }
    }


    int Utils::isASCIIalpha(int ch)
    {
      if (((ch - 'A') >= 0 && (ch - 'Z') <= 0)
		  || ((ch - 'a') >= 0 && (ch - 'z') <= 0))
		  return 1;
      return 0;
    }

    int Utils::isASCIIdigit(int ch)
    {
      if (((ch - '0') >= 0 && (ch - '9') <= 0))
		  return 1;
      return 0;
    }

    string Utils::findContentTypeByExt(const string &ext)
    {
      string content_type;
      SP::SyNaTPGApp::ptr_map_string_string::const_iterator mb = SP::SyNaTPGApp::mcs_mime.begin();
      SP::SyNaTPGApp::ptr_map_string_string::const_iterator me = SP::SyNaTPGApp::mcs_mime.end();

      for(; mb != me; mb++)
      {
        if (mb->second->find(ext) != string::npos)
        {
          content_type = string(mb->first.c_str());
          break;
        }
      }
      //BOOST_FOREACH(SP::SyNaTPGApp::ptr_map_string_string::const_reference p, SP::SyNaTPGApp::mcs_mime)
      //{
      //  if (p.second->find(ext) != string::npos)
      //  {
      //    content_type = string(p.first.c_str());
      //    break;
      //  }
      //}
      UnicodeString uct = UnicodeString::fromUTF8(content_type);
      content_type.clear();
      uct.toLower().toUTF8String(content_type);
      return content_type;
    }

    string Utils::encode(const string &input, const bool encodeSlash)
    {
      /*
      uric          = reserved | unreserved | escaped
      reserved      = ";" | "/" | "?" | ":" | "@" | "&" | "=" | "+" |
                      "$" | ","
      unreserved    = alphanum | mark
      mark          = "-" | "_" | "." | "!" | "~" | "*" | "'" |
                      "(" | ")"

      escaped       = "%" hex hex
      hex           = digit | "A" | "B" | "C" | "D" | "E" | "F" |
                              "a" | "b" | "c" | "d" | "e" | "f"

      alphanum      = alpha | digit
      alpha         = lowalpha | upalpha

      lowalpha = "a" | "b" | "c" | "d" | "e" | "f" | "g" | "h" | "i" |
                 "j" | "k" | "l" | "m" | "n" | "o" | "p" | "q" | "r" |
                 "s" | "t" | "u" | "v" | "w" | "x" | "y" | "z"
      upalpha  = "A" | "B" | "C" | "D" | "E" | "F" | "G" | "H" | "I" |
                 "J" | "K" | "L" | "M" | "N" | "O" | "P" | "Q" | "R" |
                 "S" | "T" | "U" | "V" | "W" | "X" | "Y" | "Z"
      digit    = "0" | "1" | "2" | "3" | "4" | "5" | "6" | "7" |
                 "8" | "9"
       */
      size_t len = input.length(), pos = 0;
      char ch;
      string ret;
      while(pos < len)
      {
        ch = input.at(pos);
        if (!(Utils::isASCIIalpha(ch) || Utils::isASCIIdigit(ch)
              || ch == '-' || ch == '.' || ch == '~' || ch == '_' || ch == '&'
              || ch == '!' || ch == '*' || ch == '\'' || ch == ')'
              || ch == '(' || ch == ';' || ch == '?' || ch == '%'
              || ch == ':' || ch == '@' || ch == '+' || ch == '$'
              || ch == ',' || ch == '='
              || (encodeSlash ? false: ch == '/')))
          ret.append(Utils::mcs_hex[(unsigned char) ch]);
        else
          ret.append(1, (char) ch);
        pos++;
      }
      return ret;
    }

    string Utils::decode(const string &inputEscaped)
    {
      size_t len = inputEscaped.length(), pos = 0;
      char ch;
      unsigned char uch;
      string ret;
      int lb, hb;
      while(pos < len)
      {
        ch = inputEscaped.at(pos);
        if (ch == '%')
        {
          uch = (unsigned char) toupper(inputEscaped.at(++pos));
          hb = Utils::isASCIIdigit(ch) ? ch - '0' : 10 + uch - 'A';
          uch = (unsigned char) toupper(inputEscaped.at(++pos));
          lb = Utils::isASCIIdigit(ch) ? ch - '0'  : 10 + uch - 'A' ;
          ret.append(1, (hb << 4) | lb);
        }
        else
          ret.append(1, ch);
        pos++;
      }
      return ret;
    }


    UnicodeString Utils::encode(const UnicodeString &input, const bool encodeSlash)
    {
      string utf8stdstr;
      input.toUTF8String(utf8stdstr);
      return UnicodeString::fromUTF8(Utils::encode(utf8stdstr, encodeSlash));
    }

    UnicodeString Utils::decode(const UnicodeString &inputEscaped)
    {
      string utf8stdstr;
      inputEscaped.toUTF8String(utf8stdstr);
      return UnicodeString::fromUTF8(Utils::decode(utf8stdstr));
    }

    string Utils::icusToUTF8(UnicodeString str)
    {
      string ret;
      str.toUTF8String(ret);
      return ret;
    }

    void Utils::changePermsOfDirs(const path &dir, const string perms, const path &startingPoint)
    {
      string stddir = dir.string();
      string stdstartingPoint = startingPoint.string();
      string stdinsideDir;
      if (stddir.find(stdstartingPoint) == 0)
      {
        stdinsideDir = string(stddir, stdstartingPoint.length());
        stddir = stdstartingPoint;
      }
      else
      {
        stdinsideDir = stddir;
        stddir = "";
      }

      path bpinsideDir(stdinsideDir);

      for (path::iterator it(++bpinsideDir.begin()), it_end(bpinsideDir.end()); it != it_end; ++it)
      {
        stddir += "/" + (*it).string();
        Pipe outPipe;
        Process::Args args;
        args.push_back(perms);
        args.push_back(stddir);
        ProcessHandle ph(Process::launch("chmod", args, 0, &outPipe, 0));
        PipeInputStream istr(outPipe);
        int rc = ph.wait();
      }
    }


    const boost::regex Utils::mcs_regex_metacharset("<meta[\\s\\x00]+.*?http-equiv[\\s\\x00]*=[\\s\\x00]*[\"']?Content-Type[\"']?.+?content[\\s\\x00]*=[\\s\\x00]*[\"']?.*?charset[\\s\\x00]*=[\\s\\x00]*([-a-z0-9_/]*)", boost::regex::icase);
    const boost::regex Utils::mcs_regex_contentType("Content-Type:[\\s\\x00]*([-a-z0-9_/]*)", boost::regex::icase);
    const boost::regex Utils::mcs_regex_contentTypeCharset("Content-Type:[\\s\\x00]*[^;]*;[\\s\\x00]*charset[\\s\\x00]*=[\\s\\x00]*([-a-z0-9_/]*)", boost::regex::icase);
    const boost::regex Utils::mcs_regex_contentdispositionFilename("content-disposition:[\\s\\x00]*attachment[\\s\\x00]*;[\\s\\x00]*filename[\\s\\x00]*=[\\s\\x00]*(\"[^\"]\"|'[^']'|.*)", boost::regex::icase);
    const string Utils::mcs_hex[256] = {
        "%00", "%01", "%02", "%03", "%04", "%05", "%06", "%07",
        "%08", "%09", "%0A", "%0B", "%0C", "%0D", "%0E", "%0F",
        "%10", "%11", "%12", "%13", "%14", "%15", "%16", "%17",
        "%18", "%19", "%1A", "%1B", "%1C", "%1D", "%1E", "%1F",
        "%20", "%21", "%22", "%23", "%24", "%25", "%26", "%27",
        "%28", "%29", "%2A", "%2B", "%2C", "%2D", "%2E", "%2F",
        "%30", "%31", "%32", "%33", "%34", "%35", "%36", "%37",
        "%38", "%39", "%3A", "%3B", "%3C", "%3D", "%3E", "%3F",
        "%40", "%41", "%42", "%43", "%44", "%45", "%46", "%47",
        "%48", "%49", "%4A", "%4B", "%4C", "%4D", "%4E", "%4F",
        "%50", "%51", "%52", "%53", "%54", "%55", "%56", "%57",
        "%58", "%59", "%5A", "%5B", "%5C", "%5D", "%5E", "%5F",
        "%60", "%61", "%62", "%63", "%64", "%65", "%66", "%67",
        "%68", "%69", "%6A", "%6B", "%6C", "%6D", "%6E", "%6F",
        "%70", "%71", "%72", "%73", "%74", "%75", "%76", "%77",
        "%78", "%79", "%7A", "%7B", "%7C", "%7D", "%7E", "%7F",
        "%80", "%81", "%82", "%83", "%84", "%85", "%86", "%87",
        "%88", "%89", "%8A", "%8B", "%8C", "%8D", "%8E", "%8F",
        "%90", "%91", "%92", "%93", "%94", "%95", "%96", "%97",
        "%98", "%99", "%9A", "%9B", "%9C", "%9D", "%9E", "%9F",
        "%A0", "%A1", "%A2", "%A3", "%A4", "%A5", "%A6", "%A7",
        "%A8", "%A9", "%AA", "%AB", "%AC", "%AD", "%AE", "%AF",
        "%B0", "%B1", "%B2", "%B3", "%B4", "%B5", "%B6", "%B7",
        "%B8", "%B9", "%BA", "%BB", "%BC", "%BD", "%BE", "%BF",
        "%C0", "%C1", "%C2", "%C3", "%C4", "%C5", "%C6", "%C7",
        "%C8", "%C9", "%CA", "%CB", "%CC", "%CD", "%CE", "%CF",
        "%D0", "%D1", "%D2", "%D3", "%D4", "%D5", "%D6", "%D7",
        "%D8", "%D9", "%DA", "%DB", "%DC", "%DD", "%DE", "%DF",
        "%E0", "%E1", "%E2", "%E3", "%E4", "%E5", "%E6", "%E7",
        "%E8", "%E9", "%EA", "%EB", "%EC", "%ED", "%EE", "%EF",
        "%F0", "%F1", "%F2", "%F3", "%F4", "%F5", "%F6", "%F7",
        "%F8", "%F9", "%FA", "%FB", "%FC", "%FD", "%FE", "%FF"
      };

  }
}
