#include <Wt/WCompositeWidget>
#include <Wt/WContainerWidget>
#include <Wt/WAbstractItemModel>
#include <Wt/WTableView>
#include <Wt/WText>

#include <boost/lexical_cast.hpp>
#include <boost/foreach.hpp>

#include <string>

#include "utils/mir_trkeys.h"
#include "utils/mir_cssclasses.h"
#include "widgets/mir_wcustomtableviewpushbutton.h"
#include "widgets/mir_wcustomtableview.h"

namespace SyNaT
{
  namespace MIR
  {
    using namespace Wt;
    using namespace std;

    void WCustomTableView::load()
    {
      if (!this->loaded())
      {
        this->m_page_nav_bar->addWidget(this->create_page_nav_bar());
        this->m_nav_bar_created = true;
      }
      this->WCompositeWidget::load();
    }

    WWidget *WCustomTableView::create_page_nav_bar()
    {
      WContainerWidget *wcw = new WContainerWidget();
      int page_count = this->m_wtv->pageCount();
      if (page_count > this->m_num_of_pages_in_nav_bar)
        page_count = this->m_num_of_pages_in_nav_bar;

      if (this->m_wtv->pageCount() > 0)
      {
        WCustomTableViewPushButton *wpb = new WCustomTableViewPushButton(tr(MIR_WCTV_FIRST), wcw, this);
        wpb->setStyleClass(MIR_STRINGIFY(MIR_CSS_PAGE_NAV_BUTTON));
        wpb->clicked().connect(wpb, &WCustomTableViewPushButton::slt_first_page_clicked);
        wpb = new WCustomTableViewPushButton(tr(MIR_WCTV_PREV), wcw, this);
        wpb->setStyleClass(MIR_STRINGIFY(MIR_CSS_PAGE_NAV_BUTTON));
        wpb->clicked().connect(wpb, &WCustomTableViewPushButton::slt_prev_page_clicked);
        wpb = new WCustomTableViewPushButton("<", wcw, this);
        wpb->setStyleClass(MIR_STRINGIFY(MIR_CSS_PAGE_NAV_BUTTON));
        wpb->clicked().connect(wpb, &WCustomTableViewPushButton::slt_prev_pages_clicked);

        this->m_first_page_in_nav_bar = 0;

        for(int i = 0; i < page_count; i++)
        {
          wpb = new WCustomTableViewPushButton(boost::lexical_cast<string>(i+1), wcw, this);
          wpb->setStyleClass(MIR_STRINGIFY(MIR_CSS_PAGE_NAV_BUTTON));
          wpb->clicked().connect(wpb, &WCustomTableViewPushButton::slt_page_clicked);
          this->m_page_numbers.push_back(wpb);
        }

        this->m_last_page_in_nav_bar = page_count - 1;

        wpb = new WCustomTableViewPushButton(">", wcw, this);
        wpb->setStyleClass(MIR_STRINGIFY(MIR_CSS_PAGE_NAV_BUTTON));
        wpb->clicked().connect(wpb, &WCustomTableViewPushButton::slt_next_pages_clicked);
        wpb = new WCustomTableViewPushButton(tr(MIR_WCTV_NEXT), wcw, this);
        wpb->setStyleClass(MIR_STRINGIFY(MIR_CSS_PAGE_NAV_BUTTON));
        wpb->clicked().connect(wpb, &WCustomTableViewPushButton::slt_next_page_clicked);
        wpb = new WCustomTableViewPushButton(tr(MIR_WCTV_LAST), wcw, this);
        wpb->setStyleClass(MIR_STRINGIFY(MIR_CSS_PAGE_NAV_BUTTON));
        wpb->clicked().connect(wpb, &WCustomTableViewPushButton::slt_last_page_clicked);
      }
      this->update_page_nav_bar();
      return wcw;
    }

    void WCustomTableView::update_page_nav_bar()
    {
      if (this->pageCount() == 0 && this->m_nav_bar_created)
      {
        this->m_page_nav_bar->clear();
        this->m_nav_bar_created = false;
      }
      else if (this->m_nav_bar_needs_update)
      {
        int first_page_in_nav_bar = this->m_first_page_in_nav_bar;
//        int pages_count_in_nav_bar = this->m_last_page_in_nav_bar - this->m_first_page_in_nav_bar + 1;
//
        // do we need to do something if model changes ?

        BOOST_FOREACH(WCustomTableViewPushButton *b, this->m_page_numbers)
        {
          b->setText(boost::lexical_cast<string>(first_page_in_nav_bar++ + 1));
        }
        this->m_nav_bar_needs_update = false;
      }
      BOOST_FOREACH(WCustomTableViewPushButton *b, this->m_page_numbers)
      {
        string current_page = boost::lexical_cast<string>(this->m_current_page + 1);
        if (b->text().toUTF8() == current_page)
        {
          b->setStyleClass(MIR_STRCAT3(MIR_CSS_PAGE_NAV_BUTTON," ",MIR_CSS_PAGE_NAV_BUTTON_SELECTED));
        }
        else
        {
          b->setStyleClass(MIR_STRINGIFY(MIR_CSS_PAGE_NAV_BUTTON));
        }
      }

    }

  }
}
